@@ -10,6 +10,7 @@ PROGRAM cdf16bit
   !!
   !! History : 2.1 !  11/2006    J.M. Molines   : Original code
   !!           3.0 !  12/2010    J.M. Molines   : Full Doctor form + Lic.
+  !!         : 4.0  : 03/2017  : J.M. Molines  
   !!----------------------------------------------------------------------
   !!----------------------------------------------------------------------
   !!   routines      : description
@@ -19,10 +20,11 @@ PROGRAM cdf16bit
   USE cdfio 
   USE modcdfnames
   !!----------------------------------------------------------------------
-  !! CDFTOOLS_3.0 , MEOM 2011
+  !! CDFTOOLS_4.0 , MEOM 2017 
   !! $Id$
-  !! Copyright (c) 2010, J.-M. Molines
+  !! Copyright (c) 2017, J.-M. Molines 
   !! Software governed by the CeCILL licence (Licence/CDFTOOLSCeCILL.txt)
+  !! @class data_transformation
   !!----------------------------------------------------------------------
   IMPLICIT NONE
 
@@ -40,12 +42,13 @@ PROGRAM cdf16bit
 
   REAL(KIND=4), DIMENSION(:,:),     ALLOCATABLE :: v2d                ! Array to read a layer of data
   REAL(KIND=4), DIMENSION(:),       ALLOCATABLE :: zmax, zmin         ! min and max of the field at level(jk)
-  REAL(KIND=4), DIMENSION(:),       ALLOCATABLE :: tim                ! time of file
   REAL(KIND=4)                                  :: sf, ao             ! scale_factor, add_offset
   REAL(KIND=4)                                  :: zchkmax, zchkmin   ! scale_factor, add_offset checking values
   REAL(KIND=4)                                  :: zzmax, zzmin       ! min and max of the full 3D field
   REAL(KIND=4)                                  :: spval              ! missing value, fill_value, spval ...
 
+  REAL(KIND=8), DIMENSION(:),       ALLOCATABLE :: dtim               ! time of file
+
   CHARACTER(LEN=256)                            :: cf_in              ! input file
   CHARACTER(LEN=256)                            :: cf_out='cdf16bit.nc' ! outputfile
   CHARACTER(LEN=256)                            :: cldum              ! dummy string
@@ -53,26 +56,34 @@ PROGRAM cdf16bit
 
   TYPE (variable), DIMENSION(:),    ALLOCATABLE :: stypvar            ! Type variable is defined in cdfio.
 
-  LOGICAL                                       :: l_chk=.false.      ! logical flags to save line options
-  LOGICAL                                       :: l_verbose=.false.  ! logical flags to save line options
+  LOGICAL                                       :: l_chk    = .FALSE. ! logical flags to save line options
+  LOGICAL                                       :: l_verbose= .FALSE. ! logical flags to save line options
+  LOGICAL                                       :: lnc4     = .FALSE. ! Use nc4 with chunking and deflation
   LOGICAL, DIMENSION(:,:),          ALLOCATABLE :: lmask              ! 2D logical land/sea mask (true on ocean)
   !!----------------------------------------------------------------------
   CALL ReadCdfNames()
 
   narg = iargc()
   IF ( narg == 0 ) THEN
-     PRINT *,' usage : cdf16bit 32BIT-file [ -check ] [ -verbose]'
+     PRINT *,' usage : cdf16bit -f 32BIT-file [-check] [-verbose] [-o OUT-file] [-nc4]'
      PRINT *,'      '
      PRINT *,'     PURPOSE :'
-     PRINT *,'       Convert input 32 bit precision file into 16 bit' 
-     PRINT *,'       precision file using add_offset and scale_factor'
+     PRINT *,'       Convert input 32 bit precision file into 16 bit precision file using' 
+     PRINT *,'       add_offset and scale_factor. '
+     PRINT *,'       Note that predifined values for these two parameters are defined '
+     PRINT *,'       according to the variable name. If variable name is not supported,'
+     PRINT *,'       no conversion is performed.'
      PRINT *,'      '
      PRINT *,'     ARGUMENTS :'
-     PRINT *,'       32BIT-file : input 32 bit file to be converted' 
+     PRINT *,'       -f 32BIT-file : input 32 bit file to be converted' 
      PRINT *,'      '
      PRINT *,'     OPTIONS :'
-     PRINT *,'       [ -check ]   : control than the scale factors are adequate'
-     PRINT *,'       [ -verbose ] : give information level by level.' 
+     PRINT *,'       [-check ]   : control than the scale factors are adequate.'
+     PRINT *,'       [-verbose ] : give information level by level.' 
+     PRINT *,'       [-o OUT-file] : Specify output file name instead of ', TRIM(cf_out)
+     PRINT *,'       [-nc4 ] : Use netcdf4 output with chunking and deflation level 1.'
+     PRINT *,'                 This option is effective only if cdftools are compiled with'
+     PRINT *,'                 a netcdf library supporting chunking and deflation.'
      PRINT *,'      '
      PRINT *,'     REQUIRED FILES :'
      PRINT *,'       none '
@@ -80,27 +91,26 @@ PROGRAM cdf16bit
      PRINT *,'     OUTPUT : '
      PRINT *,'       netcdf file : ', TRIM(cf_out) 
      PRINT *,'         variables : same names than in input file'
-     STOP
+     STOP 
   ENDIF
   !!
   ijarg = 1
-  CALL getarg (ijarg, cf_in) ; ijarg = ijarg + 1
-  IF ( chkfile(cf_in)  ) STOP 99 ! missing file
-
-  ! Check for options and reflect options on logical flags
   DO WHILE ( ijarg <= narg)  
      CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1
-
-     SELECT CASE (  cldum  )   ! Analyse option
-     CASE ( '-check'   )          ! control if the scale factors are OK
-        l_chk=.true.
-     CASE ( '-verbose' )          ! information will be given level by level
-        l_chk=.true. ; l_verbose=.true.
-     CASE DEFAULT
-        PRINT *,' OPTION ',TRIM(cldum),' not supported.' ; STOP 99
+     SELECT CASE (  cldum  )  
+     CASE ( '-f'      ) ; CALL getarg (ijarg, cf_in ) ; ijarg = ijarg + 1
+        ! options
+     CASE ( '-check'  ) ; l_chk     =.TRUE.
+     CASE ( '-verbose') ; l_chk     =.TRUE. 
+        ;                 l_verbose =.TRUE.
+     CASE ( '-o'      ) ; CALL getarg (ijarg, cf_out) ; ijarg = ijarg + 1
+     CASE ( '-nc4'    ) ; lnc4      =.TRUE.
+     CASE DEFAULT       ; PRINT *,' ERROR : ',TRIM(cldum),' : unknown option.' ; STOP 99
      END SELECT
   END DO
 
+  IF ( chkfile(cf_in)  ) STOP 99 ! missing file
+
   ! get domain dimension from input file
   npiglo = getdim (cf_in, cn_x)
   npjglo = getdim (cf_in, cn_y)
@@ -108,16 +118,14 @@ PROGRAM cdf16bit
 
   IF (ierr /= 0 ) THEN
      npk   = getdim (cf_in,'z',kstatus=ierr)
-     IF (ierr /= 0 ) THEN
-        PRINT *, 'depth dimension name not suported'
-        STOP 99
+     IF (ierr /= 0 ) THEN ;  PRINT *,' ERROR : depth dimension name not suported' ; STOP 99
      ENDIF
   ENDIF
   npt    = getdim (cf_in, cn_t)
 
   ! Allocate memory 
   ALLOCATE( v2d(npiglo,npjglo), i2d(npiglo,npjglo), lmask(npiglo, npjglo) )
-  ALLOCATE( zmin(npk) , zmax(npk) , tim(npt))
+  ALLOCATE( zmin(npk) , zmax(npk) , dtim(npt))
 
   ! Get the number of variables held in the file, allocate arrays
   nvars = getnvar(cf_in)
@@ -130,30 +138,8 @@ PROGRAM cdf16bit
   ! get list of variable names and collect attributes in stypvar (optional)
   cv_names(:)=getvarname(cf_in,nvars,stypvar) 
 
-  id_var(:)  = (/(jv, jv=1,nvars)/)
-  ! ipk gives the number of level or 0 if not a T[Z]YX  variable
-  ipk(:)     = getipk (cf_in,nvars)
+  CALL CreateOutput
 
-  ! flags variable not to be treated by changing their name to none
-  WHERE( ipk == 0 ) cv_names='none'
-  stypvar(:)%cname=cv_names
-
-  ! create output fileset
-  ! fills the scale_factor and add_offset attribute according to variable name
-  ! if the variable is not documented, then, sf=1, ao=0. and no conversion
-  ! is performed for this variable (It stays in REAL*4 )
-  DO jvar=1,nvars
-     IF (cv_names(jvar) /= 'none' ) CALL sf_ao(jvar)
-  END DO
-
-  ! create output file taking the sizes in cf_in
-  ncout =create(cf_out, cf_in,npiglo,npjglo,npk)
-  ! The variables are created as FLOAT or SHORT depending on the scale_factor AND add_offset attribute
-  ierr= createvar(ncout , stypvar,  nvars, ipk, id_varout )
-  ierr= putheadervar(ncout , cf_in, npiglo, npjglo, npk)
-
-  ! Get time and write time
-  tim=getvar1d(cf_in,cn_vtimec,npt) ; ierr=putvar1d(ncout,tim,npt,'T')
   ! Loop on all variables of the file
   DO jvar = 1,nvars
      IF (cv_names(jvar) == 'none' ) THEN
@@ -170,7 +156,7 @@ PROGRAM cdf16bit
                  ! write FLOATS
                  IF ( stypvar(jvar)%savelog10 == 1 ) THEN
                     WHERE ( v2d /= spval )
-                       v2d(:,:)= log10(v2d)
+                       v2d(:,:)= LOG10(v2d)
                     ELSEWHERE
                        v2d = 0.
                     END WHERE
@@ -188,7 +174,7 @@ PROGRAM cdf16bit
                  END WHERE
               ELSE  ! store log10  ao and sf refer to the log10 of the variable
                  WHERE( v2d /= spval ) 
-                    i2d(:,:)=NINT((log10(v2d(:,:))-ao)/sf)
+                    i2d(:,:)=NINT((LOG10(v2d(:,:))-ao)/sf)
                  ELSEWHERE
                     i2d(:,:)=0
                  END WHERE
@@ -502,7 +488,7 @@ CONTAINS
     !!----------------------------------------------------------------------
     IF ( l_chk ) THEN  ! with this option, check if the max value of the field  can be
        !mapped on I2 with actual values of Scale_factor and Add_offset
-       lmask=.true. ; WHERE (v2d == spval ) lmask=.false.
+       lmask=.TRUE. ; WHERE (v2d == spval ) lmask=.FALSE.
        ! Works with log10 of v2d in case of savelog10=1
        IF (stypvar(jvar)%savelog10 == 1 ) THEN
           WHERE( v2d /= 0. ) v2d=LOG10(v2d)
@@ -560,4 +546,42 @@ CONTAINS
     END IF   ! check mode
   END SUBROUTINE checkscaling
 
+  SUBROUTINE CreateOutput
+    !!---------------------------------------------------------------------
+    !!                  ***  ROUTINE CreateOutput  ***
+    !!
+    !! ** Purpose :  Create netcdf output file(s) 
+    !!
+    !! ** Method  :  Use stypvar global description of variables
+    !!
+    !!----------------------------------------------------------------------
+    id_var(:)  = (/(jv, jv=1,nvars)/)
+    ! ipk gives the number of level or 0 if not a T[Z]YX  variable
+    ipk(:)     = getipk (cf_in,nvars)
+
+    ! flags variable not to be treated by changing their name to none
+    WHERE( ipk == 0 ) cv_names='none'
+    stypvar(:)%cname=cv_names
+
+    ! create output fileset
+    ! fills the scale_factor and add_offset attribute according to variable name
+    ! if the variable is not documented, then, sf=1, ao=0. and no conversion
+    ! is performed for this variable (It stays in REAL*4 )
+    DO jvar=1,nvars
+       IF (cv_names(jvar) /= 'none' ) CALL sf_ao(jvar)
+       stypvar(jvar)%ichunk = (/npiglo,MAX(1,npjglo/30),1,1 /)
+    END DO
+
+    ! create output file taking the sizes in cf_in
+    ncout =create(cf_out, cf_in,npiglo,npjglo,npk           , ld_nc4=lnc4 )
+    ! The variables are created as FLOAT or SHORT depending on the scale_factor AND add_offset attribute
+    ierr= createvar(ncout , stypvar,  nvars, ipk, id_varout , ld_nc4=lnc4 )
+    ierr= putheadervar(ncout , cf_in, npiglo, npjglo, npk)
+
+    ! Get time and write time
+    dtim=getvar1d(cf_in,cn_vtimec,npt) 
+    ierr=putvar1d(ncout,dtim,npt,'T')
+
+  END SUBROUTINE CreateOutput
+
 END PROGRAM cdf16bit
