@@ -14,6 +14,7 @@ PROGRAM cdfbathy
   !! History : 2.1  : 11/2007  : J.M. Molines : Original code
   !!           3.0  : 12/2010  : J.M. Molines : Doctor norm + Lic.
   !!                : 04/1014  : P. Mathiot   : add fill_pool option
+  !!         : 4.0  : 03/2017  : J.M. Molines  
   !!----------------------------------------------------------------------
   !!----------------------------------------------------------------------
   !!   routines      : description
@@ -31,22 +32,26 @@ PROGRAM cdfbathy
   USE cdfio
   USE modcdfnames
   !!----------------------------------------------------------------------
-  !! CDFTOOLS_3.0 , MEOM 2011
+  !! CDFTOOLS_4.0 , MEOM 2017 
   !! $Id$
-  !! Copyright (c) 2010, J.-M. Molines
+  !! Copyright (c) 2017, J.-M. Molines 
   !! Software governed by the CeCILL licence (Licence/CDFTOOLSCeCILL.txt)
+  !! @class data_transformation
   !!----------------------------------------------------------------------
   IMPLICIT NONE
   !
+  INTEGER(KIND=4)                              :: jk, jt              ! loop index
   INTEGER(KIND=4)                              :: narg, iargc, ijarg  ! browse command line
   INTEGER(KIND=4)                              :: iimin, iimax        ! selected area
   INTEGER(KIND=4)                              :: ijmin, ijmax        ! selected area
+  INTEGER(KIND=4)                              :: iiref, ijref        ! reference point coordinates for _ij options
   INTEGER(KIND=4)                              :: ierr                ! error status
   INTEGER(KIND=4)                              :: icrit               ! maximal size of pool 
   INTEGER(KIND=4)                              :: iklev               ! selected level
   INTEGER(KIND=4)                              :: itime               ! selected time
   INTEGER(KIND=4)                              :: npiglo, npjglo      ! domain size
   INTEGER(KIND=4)                              :: npk, npt            ! domaine size
+  INTEGER(KIND=4)                              :: nnpk, nnpt            ! domaine size
   INTEGER(KIND=4)                              :: iversion=1          ! version counter for working copy
   INTEGER(KIND=4)                              :: iostat, ipos        ! used for version control
   INTEGER(KIND=4), DIMENSION(:,:), ALLOCATABLE :: mbathy              ! map of wet model level
@@ -78,7 +83,11 @@ PROGRAM cdfbathy
   LOGICAL :: lmodif     = .FALSE., loverwrite = .FALSE.       ! all required flags for options
   LOGICAL :: lraz       = .FALSE., ldumpn     = .FALSE.       ! all required flags for options
   LOGICAL :: lrazb      = .FALSE., lsetb      = .FALSE.       ! all required flags for options
+  LOGICAL :: lsetz      = .FALSE., lseta      = .FALSE.       ! all required flags for options
+  LOGICAL :: lrazb_ij   = .FALSE., lsetb_ij   = .FALSE.       ! all required flags for options
+  LOGICAL :: lsetz_ij   = .FALSE., lseta_ij   = .FALSE.       ! all required flags for options
   LOGICAL :: lchk       = .FALSE., lfillpool  = .FALSE.       ! all required flags for options
+  LOGICAL :: ll_lev     = .FALSE., ll_time    = .FALSE.       ! all level, all time ==> true
   !!----------------------------------------------------------------------
   CALL ReadCdfNames()
 
@@ -88,19 +97,19 @@ PROGRAM cdfbathy
      PRINT *,' usage : cdfbathy/cdfvar -f IN-file [options]'
      PRINT *,'      '
      PRINT *,'     PURPOSE :'
-     PRINT *,'       Allow manual modification of the input file. Very convenient' 
-     PRINT *,'       for bathymetric files, can also be used with any model file'
+     PRINT *,'       Allow manual modification of the input file. Very convenient for ' 
+     PRINT *,'       bathymetric files, can also be used with any other model file.'
      PRINT *,'       Keep a log.f90 file of the modifications for automatic reprocessing'
      PRINT *,'      '
      PRINT *,'     ARGUMENTS :'
-     PRINT *,'       IN-file : original input file. The program works on a copy of the'
+     PRINT *,'       -f IN-file : original input file. The program works on a copy of the'
      PRINT *,'                original file (default)' 
      PRINT *,'      '
      PRINT *,'     OPTIONS :'
      PRINT 9999, '   -file (or -f )       : name of input file '
      PRINT 9999, '   -var  (or -v )       : name of cdf variable [default: Bathymetry]'
-     PRINT 9999, '   -lev  (or -l )       : level to work with '
-     PRINT 9999, '   -time (or -t )       : time to work with '
+     PRINT 9999, '   -lev  (or -l )       : level to work with (0 means all levels)'
+     PRINT 9999, '   -time (or -t )       : time to work with (0 means all times)'
      PRINT 9999, '   -scale  s            : use s as a scale factor (divide when read the file)'
      PRINT 9999, '   -zoom (or -z )       : sub area of the bathy file to work with (imin imax jmin jmax)'
      PRINT 9999, '   -fillzone (or -fz )  : sub area will be filled with 0 up to the first coast line '
@@ -110,6 +119,20 @@ PROGRAM cdfbathy
      PRINT 9999, '      (or -rb depmin )  '
      PRINT 9999, '   -set_below depmin    : any depth less than depmin in subarea will be replaced by depmin '
      PRINT 9999, '      (or -sb depmin ) '
+     PRINT 9999, '   -set_above depmax    : any depth larger than depmax in subarea will be replaced by depmax '
+     PRINT 9999, '      (or -sa depmax ) '
+     PRINT 9999, '   -set_zone value      : all value in area will be set to value'
+     PRINT 9999, '      (or -sz value ) '
+
+     PRINT 9999, '   -raz_below_ij IREF JREF    : any depth less than v(IREF,JREF) in subarea will be replaced by 0 '
+     PRINT 9999, '      (or -rb_ij IREF JREF )  '
+     PRINT 9999, '   -set_below_ij IREF JREF    : any depth less than v(IREF,JREF) in subarea will be replaced by depmin '
+     PRINT 9999, '      (or -sb_ij IREF JREF ) '
+     PRINT 9999, '   -set_above_ij IREF JREF    : any depth larger than v(IREF,JREF) in subarea will be replaced by depmax '
+     PRINT 9999, '      (or -sa_ij IREF JREF ) '
+     PRINT 9999, '   -set_zone_ij IREF JREF      : all value in area will be set to v(IREF,JREF)'
+     PRINT 9999, '      (or -sz_ij IREF JREF ) '
+
      PRINT 9999, '   -fullstep depmin     : sub area will be reshaped as full-step, below depmin'
      PRINT 9999, '      (or -fs depmin )    requires the presence of the file zgr_bat.txt (from ocean.output, eg )'
      PRINT 9999, '   -dumpzone (or -d )   : sub area will be output to an ascii file, which can be used by -replace'
@@ -129,7 +152,7 @@ PROGRAM cdfbathy
      PRINT 9999, '             a sequence number is added at the end of the input file name, to keep'
      PRINT 9999, '             modifications.'
      PRINT *,'            variables : same as input file'
-     STOP
+     STOP 
   ENDIF
 9999 FORMAT(5x,a)
 
@@ -143,81 +166,60 @@ PROGRAM cdfbathy
   DO  WHILE (ijarg <=  narg)
      CALL getarg(ijarg, cldum) ;  ijarg = ijarg + 1
      SELECT CASE ( cldum )
-     !
-     CASE ( '-file' , '-f' )     ! name of input file
-        CALL getarg(ijarg, cf_in) ; ijarg = ijarg + 1 
-        lchk = ( lchk .OR. chkfile (cf_in) )
-     !
-     CASE ( '-var' , '-v' )      ! name of netcdf variable
-        CALL getarg(ijarg, cv_in) ; ijarg = ijarg + 1 
-     !
-     CASE ( '-lev' , '-k' )      ! level to work with
-        CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) iklev
-     !
-     CASE ( '-time' , '-t' )     ! time frame to work with
-        CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) itime
-     !
-     CASE ( '-scale'      )      ! dividing scale factor
-        CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) scale_factor
-     !
-     CASE ( '-zoom' , '-z' )     ! specify zoomed area
-        CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) iimin
-        CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) iimax
-        CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) ijmin
-        CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) ijmax
-    
-     CASE ( '-fillzone' , '-fz' )  ! Fill the area specified with zoom 
-                                   ! with 0 till a coast is encountered in the East
-        lfill=.TRUE. ; lmodif=.TRUE.
-     !
-     CASE ( '-raz_zone' , '-raz' ) ! Set a zoomed area to 0
-        lraz=.TRUE. ; lmodif=.TRUE.
-     !
-     CASE ( '-raz_below' , '-rb' ) ! Area below this value are set to 0
-        CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) rdepfill
-        lrazb=.TRUE. ; lmodif=.TRUE.
-     !
-     CASE ( '-set_below' , '-sb' ) ! Area below this value are set to values
-        CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) rdepfill
-        lsetb=.TRUE. ; lmodif=.TRUE.
-     !
-     CASE ( '-fullstep' , '-fs' ) ! Create a full-step like bathy in zoomed area
-        lfullstep=.TRUE. ; lmodif=.TRUE.
-        CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) rdepmin
-     !
-     CASE ( '-append' , '-a' )    ! Append modification to the f90 log file
-        lappend=.TRUE.
-     !
-     CASE ( '-overwrite' , '-o' ) ! Overwrite modifications in f90 log file
-        loverwrite=.TRUE.
-     !
-     CASE ( '-fillpool' , '-fp' ) ! Overwrite modifications in f90 log file
-        lfillpool=.TRUE. ; lmodif=.TRUE.
-        CALL getarg(ijarg, cldum) ; ijarg = ijarg +1 ; READ(cldum,*) icrit
-     !
-     CASE ( '-replace' , '-r' )   ! Replace zoomed area by values in ascii file
-        lreplace=.TRUE. ; lmodif=.TRUE.
-        CALL getarg(ijarg, cf_replace) ; ijarg = ijarg +1
-        lchk = ( lchk .OR. chkfile (cf_replace) )
-     !
-     CASE ( '-log'            )  !  log file name
-        CALL getarg(ijarg, cf_log) ; ijarg = ijarg +1
-     !
-     CASE ( '-dumpzone' , '-d' )  ! output the zoomed area in a formatted file
-        ldump=.TRUE.
-        CALL getarg(ijarg, cf_dump) ; ijarg = ijarg +1
-     !
-     CASE ( '-nicedumpzone' , '-nd' )  ! idem dumpzone above but with nicer format
-        ldumpn=.TRUE.
-        CALL getarg(ijarg, cf_dump) ; ijarg = ijarg +1
-     !
-     CASE DEFAULT
-        PRINT *, cldum,' : unknown option '
-        STOP
+     CASE ( '-file' , '-f') ; CALL getarg(ijarg, cf_in) ; ijarg = ijarg + 1 
+        ; lchk = ( lchk .OR. chkfile (cf_in) )
+     CASE ( '-var' , '-v' ) ; CALL getarg(ijarg, cv_in) ; ijarg = ijarg + 1 
+     CASE ( '-lev' , '-k' ) ; CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) iklev
+     CASE ( '-time' , '-t') ; CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) itime
+     CASE ( '-scale'      ) ; CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) scale_factor
+     CASE ( '-zoom' , '-z') ; CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) iimin
+        ;                     CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) iimax
+        ;                     CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) ijmin
+        ;                     CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) ijmax
+     CASE ('-fillzone','-fz' ) ; lfill =.TRUE. ; lmodif =.TRUE.
+     CASE ('-raz_zone','-raz') ; lraz  =.TRUE. ; lmodif =.TRUE.
+     CASE ('-raz_below','-rb') ; CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) rdepfill
+        ; lrazb =.TRUE. ; lmodif =.TRUE.
+     CASE ('-set_below','-sb') ; CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) rdepfill
+        ; lsetb =.TRUE. ; lmodif =.TRUE.
+     CASE ('-set_above','-sa') ; CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) rdepfill
+        ; lseta =.TRUE. ; lmodif =.TRUE.
+     CASE ('-set_zone','-sz') ; CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) rdepfill
+        ; lsetz =.TRUE. ; lmodif =.TRUE.
+
+     CASE ('-raz_below_ij','-rb_ij') ; CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) iiref
+        ;                              CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) ijref
+        ; lrazb_ij =.TRUE. ; lmodif =.TRUE.
+     CASE ('-set_below_ij','-sb_ij') ; CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) iiref
+        ;                              CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) ijref
+        ; lsetb_ij =.TRUE. ; lmodif =.TRUE.
+     CASE ('-set_above_ij','-sa_ij') ; CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) iiref
+        ;                              CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) ijref
+        ; lseta_ij =.TRUE. ; lmodif =.TRUE.
+     CASE ('-set_zone_ij','-sz_ij')  ; CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) iiref
+        ;                              CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) ijref
+        ; lsetz_ij =.TRUE. ; lmodif =.TRUE.
+
+
+     CASE ('-fullstep','-fs' ) ; lfullstep =.TRUE. ; lmodif=.TRUE.
+        ; CALL getarg(ijarg, cldum) ; ijarg = ijarg + 1 ; READ(cldum,*) rdepmin
+     CASE ('-append' , '-a'  ) ; lappend=.TRUE.
+     CASE ('-overwrite' ,'-o') ; loverwrite=.TRUE.
+     CASE ('-fillpool','-fp' ) ; lfillpool =.TRUE. ; lmodif =.TRUE.
+        ; CALL getarg(ijarg, cldum) ; ijarg = ijarg +1 ; READ(cldum,*) icrit
+     CASE ('-replace','-r'   ) ; lreplace =.TRUE. ; lmodif =.TRUE.
+        ; CALL getarg(ijarg, cf_replace) ; ijarg = ijarg +1
+        ; lchk = ( lchk .OR. chkfile (cf_replace) )
+     CASE ( '-log'           ) ; CALL getarg(ijarg, cf_log) ; ijarg = ijarg +1
+     CASE ( '-dumpzone','-d' ) ; ldump =.TRUE.
+        ; CALL getarg(ijarg, cf_dump) ; ijarg = ijarg +1
+     CASE ('-nicedumpzone','-nd'); ldumpn =.TRUE.
+        ; CALL getarg(ijarg, cf_dump) ; ijarg = ijarg +1
+     CASE DEFAULT              ; PRINT *,' ERROR : ',TRIM(cldum),' : unknown option.' ; STOP 99
      END SELECT
   END DO
-  
-  IF ( lchk ) STOP 99 ! missing files
+
+  IF ( lchk ) STOP 99  ! missing files
 
   IF ( lmodif .AND. .NOT. loverwrite) THEN
      ! creating a working copy of the file indexed by iversion
@@ -244,17 +246,39 @@ PROGRAM cdfbathy
 
   npiglo = getdim(cwkc,cn_x)
   npjglo = getdim(cwkc,cn_y)
-  npk    = getdim(cwkc,cn_z)
+  npk    = getdim(cwkc,cn_z,kstatus=ierr)
+  IF ( ierr /= 0 ) THEN
+    npk    = getdim(cwkc,'z')
+  ENDIF
   npt    = getdim(cwkc,cn_t)
   IF (npk == 0 ) npk = 1
   IF (npt == 0 ) npt = 1
 
+  nnpk=1
+  nnpt=1
+
+  IF ( iklev == 0 )  THEN
+     ll_lev=.true.
+     nnpk = npk
+  ENDIF
+
+  PRINT *,'CN_Z = ', TRIM(cn_z)
+  PRINT *,'LL_LEV = ', ll_lev
+  PRINT *,'NPK = ', npk
+  PRINT *,'NNPK = ', nnpk
+
+
   IF ( iklev > npk ) THEN
-    PRINT *,' ERROR : not enough levels in input file ', TRIM(cwkc)
+     PRINT *,' ERROR : not enough levels in input file ', TRIM(cwkc)
   ENDIF
 
   IF ( itime > npt ) THEN
-    PRINT *,' ERROR : not enough times in input file ', TRIM(cwkc)
+     PRINT *,' ERROR : not enough times in input file ', TRIM(cwkc)
+  ENDIF
+
+  IF ( itime == 0 )  THEN
+     ll_time=.true.
+     nnpt = npt
   ENDIF
 
   IF ( iimin == -10 ) THEN  ! no zoom option passed
@@ -269,30 +293,44 @@ PROGRAM cdfbathy
   ALLOCATE (mbathy(npiglo,npjglo), e3_bot( npiglo,npjglo))
   ALLOCATE (bathy( npiglo,npjglo), bathyin(npiglo,npjglo))
 
-  ! we use bathy as variable name but it can be any field from cf_in
-  bathy(:,:) = getvar(cwkc, cv_in, iklev, npiglo, npjglo, ktime=itime)
-  bathy(:,:) = bathy(:,:)/scale_factor
-  bathyin    = bathy  ! save original 
+  DO jt = 1,nnpt
+     IF (ll_time ) itime=jt
+     DO jk =1,nnpk
+        IF ( ll_lev ) iklev = jk
 
-  IF (lfullstep ) THEN ;CALL zgr_read ; CALL zgr_zps(iimin, iimax, ijmin, ijmax) ; ENDIF
-  IF (lfill     )       CALL fillzone  (iimin, iimax, ijmin, ijmax)
-  IF (lraz      )       CALL raz_zone  (iimin, iimax, ijmin, ijmax)
-  IF (lrazb     )       CALL raz_below (iimin, iimax, ijmin, ijmax, rdepfill)
-  IF (lsetb     )       CALL set_below (iimin, iimax, ijmin, ijmax, rdepfill)
-  IF (ldump     )       CALL dumpzone     (cf_dump, iimin, iimax, ijmin, ijmax)
-  IF (ldumpn    )       CALL nicedumpzone (cf_dump, iimin, iimax, ijmin, ijmax)
-  IF (lreplace  )       CALL replacezone  (cf_replace)
-  IF (lfillpool )       CALL fillpool  (icrit, iimin, iimax, ijmin, ijmax)
+        ! we use bathy as variable name but it can be any field from cf_in
+        bathy(:,:) = getvar(cwkc, cv_in, iklev, npiglo, npjglo, ktime=itime)
+        bathy(:,:) = bathy(:,:)/scale_factor
+        bathyin    = bathy  ! save original 
 
-  IF (lmodif ) THEN   ! save log 
-     CALL prlog(bathyin, bathy, npiglo, npjglo, lappend)
-     ierr = putvar(cwkc, cv_in, iklev, iimax-iimin+1, ijmax-ijmin+1, kimin=iimin, kjmin=ijmin, &
-      &            ptab=bathy(iimin:iimax,ijmin:ijmax)*scale_factor, ktime=itime)
-  ENDIF
+        IF (lfullstep ) THEN  ;  CALL zgr_read ; CALL zgr_zps(iimin, iimax, ijmin, ijmax); ENDIF
+        IF (lfill     )       CALL fillzone  (iimin, iimax, ijmin, ijmax)
+        IF (lraz      )       CALL raz_zone  (iimin, iimax, ijmin, ijmax)
+        IF (lrazb     )       CALL raz_below (iimin, iimax, ijmin, ijmax, rdepfill)
+        IF (lsetb     )       CALL set_below (iimin, iimax, ijmin, ijmax, rdepfill)
+        IF (lseta     )       CALL set_above (iimin, iimax, ijmin, ijmax, rdepfill)
+        IF (lsetz     )       CALL set_zone  (iimin, iimax, ijmin, ijmax, rdepfill)
+
+        IF (lrazb_ij  )       CALL raz_below (iimin, iimax, ijmin, ijmax, bathy(iiref,ijref))
+        IF (lsetb_ij  )       CALL set_below (iimin, iimax, ijmin, ijmax, bathy(iiref,ijref))
+        IF (lseta_ij  )       CALL set_above (iimin, iimax, ijmin, ijmax, bathy(iiref,ijref))
+        IF (lsetz_ij  )       CALL set_zone  (iimin, iimax, ijmin, ijmax, bathy(iiref,ijref))
+        
+        IF (ldump     )       CALL dumpzone     (cf_dump, iimin, iimax, ijmin, ijmax)
+        IF (ldumpn    )       CALL nicedumpzone (cf_dump, iimin, iimax, ijmin, ijmax)
+        IF (lreplace  )       CALL replacezone  (cf_replace)
+        IF (lfillpool )       CALL fillpool  (icrit, iimin, iimax, ijmin, ijmax)
+
+        IF (lmodif ) THEN   ! save log 
+           CALL prlog(bathyin, bathy, npiglo, npjglo, lappend)
+           ierr = putvar(cwkc, cv_in, iklev, iimax-iimin+1, ijmax-ijmin+1, kimin=iimin, kjmin=ijmin, &
+                &            ptab=bathy(iimin:iimax,ijmin:ijmax)*scale_factor, ktime=itime)
+        ENDIF
+     ENDDO
+  ENDDO
 
 CONTAINS 
 
-
   SUBROUTINE zgr_zps ( kimin, kimax ,kjmin, kjmax )
     !!---------------------------------------------------------------------
     !!                  ***  ROUTINE zgr_zps  ***
@@ -341,7 +379,7 @@ CONTAINS
     ! is larger than the minimum of e3zps_min and e3zps_rat * e3t (where
     ! e3t is the reference level thickness
     DO jk = npk-1, 1, -1
-!      zdepth = gdepw(jk) + MIN( e3zps_min, e3t(jk)*e3zps_rat )
+       !      zdepth = gdepw(jk) + MIN( e3zps_min, e3t(jk)*e3zps_rat )
        zdepth = gdept(jk)
        WHERE ( bathy(kimin:kimax,kjmin:kjmax) > 0. .AND. bathy (kimin:kimax,kjmin:kjmax) <= zdepth )  
           mbathy(kimin:kimax,kjmin:kjmax)=jk-1
@@ -350,12 +388,12 @@ CONTAINS
     END DO
 
     DO ji=kimin,kimax
-      DO jj=kjmin,kjmax
-        jk=mbathy(ji,jj)
-        IF (jk /= 0 ) THEN 
-           IF (gdepw(jk+1) > rdepmin ) bathy(ji,jj)=gdepw(jk+1)-0.1
-        ENDIF
-      ENDDO
+       DO jj=kjmin,kjmax
+          jk=mbathy(ji,jj)
+          IF (jk /= 0 ) THEN 
+             IF (gdepw(jk+1) > rdepmin ) bathy(ji,jj)=gdepw(jk+1)-0.1
+          ENDIF
+       ENDDO
     END DO
   END SUBROUTINE zgr_zps
 
@@ -430,7 +468,7 @@ CONTAINS
        DO jj=1,kpj
           IF ( ABS( ptabold(ji,jj) - ptab(ji,jj)) > 0.02  ) THEN    ! allow a 2 cm tolerance for rounding purposes
              WRITE(inumlog,'(a,i4,a,i4,a,f8.2,a,f8.2)') ' bathy(',ji,',',jj,')=',ptab(ji,jj)*scale_factor,  &
-                    & ' ! instead of ',ptabold(ji,jj)*scale_factor
+                  & ' ! instead of ',ptabold(ji,jj)*scale_factor
           END IF
        END DO
     END DO
@@ -490,7 +528,7 @@ CONTAINS
     INTEGER(KIND=4), INTENT(in) :: kimin, kimax, kjmin, kjmax ! position of the data windows
     REAL(KIND=4),    INTENT(in) :: pdepmin                    ! threshold bathy value
     !!----------------------------------------------------------------------
-   WHERE ( bathy(kimin:kimax, kjmin:kjmax) <= pdepmin)  bathy(kimin:kimax, kjmin:kjmax) = 0.
+    WHERE ( bathy(kimin:kimax, kjmin:kjmax) <= pdepmin)  bathy(kimin:kimax, kjmin:kjmax) = 0.
 
   END SUBROUTINE raz_below
 
@@ -506,11 +544,43 @@ CONTAINS
     INTEGER(KIND=4), INTENT(in) :: kimin, kimax, kjmin, kjmax ! position of the data windows
     REAL(KIND=4),    INTENT(in) :: pdepmin                    ! threshold bathy value
     !!----------------------------------------------------------------------
-   WHERE ( bathy(kimin:kimax, kjmin:kjmax) <= pdepmin .AND. bathy(kimin:kimax, kjmin:kjmax) > 0 ) &
-        &                 bathy(kimin:kimax, kjmin:kjmax) = pdepmin
+    WHERE ( bathy(kimin:kimax, kjmin:kjmax) <= pdepmin .AND. bathy(kimin:kimax, kjmin:kjmax) > 0 ) &
+         &                 bathy(kimin:kimax, kjmin:kjmax) = pdepmin
 
   END SUBROUTINE set_below
 
+  SUBROUTINE set_above(kimin, kimax, kjmin, kjmax, pdepmax)
+    !!---------------------------------------------------------------------
+    !!                  ***  ROUTINE set_above  ***
+    !!
+    !! ** Purpose : Set bathy points pdepmax if larger than pdepmax
+    !!              original bathy
+    !!
+    !!----------------------------------------------------------------------
+    INTEGER(KIND=4), INTENT(in) :: kimin, kimax, kjmin, kjmax ! position of the data windows
+    REAL(KIND=4),    INTENT(in) :: pdepmax                    ! threshold bathy value
+    !!----------------------------------------------------------------------
+    WHERE ( bathy(kimin:kimax, kjmin:kjmax) >= pdepmax ) &
+         &                 bathy(kimin:kimax, kjmin:kjmax) = pdepmax
+
+  END SUBROUTINE set_above
+
+
+  SUBROUTINE set_zone(kimin, kimax, kjmin, kjmax, pdepmin)
+    !!---------------------------------------------------------------------
+    !!                  ***  ROUTINE set_zone  ***
+    !!
+    !! ** Purpose : Set bathy points to pdepmin 
+    !!
+    !!----------------------------------------------------------------------
+    INTEGER(KIND=4), INTENT(in) :: kimin, kimax, kjmin, kjmax ! position of the data windows
+    REAL(KIND=4),    INTENT(in) :: pdepmin                    ! threshold bathy value
+    !!----------------------------------------------------------------------
+    bathy(kimin:kimax, kjmin:kjmax) =pdepmin
+
+  END SUBROUTINE set_zone
+
+
 
   SUBROUTINE dumpzone(cdumpf, kimin, kimax, kjmin, kjmax)
     !!---------------------------------------------------------------------
@@ -599,7 +669,7 @@ CONTAINS
     CLOSE(inumrep)
 
   END SUBROUTINE replacezone
-  
+
 
   SUBROUTINE fillpool(kcrit, kimin, kimax, kjmin, kjmax) 
     !!---------------------------------------------------------------------
@@ -635,7 +705,7 @@ CONTAINS
     PRINT *, 'Filling area in progress ... (it can take a while)'    
 
     DO ji=kimin,kimax
-       IF (mod(ji,100) == 0) PRINT *, ji,'/',npiglo
+       IF (MOD(ji,100) == 0) PRINT *, ji,'/',npiglo
        DO jj=kjmin,kjmax
           ! modify something only if seed point is a non 0 cell
           IF (ioptm(ji,jj) == 1) THEN
@@ -644,44 +714,44 @@ CONTAINS
              ipile(:,:)=0
              ipile(1,:)=[ji,jj]
              ip=1; ik=0
-         
+
              ! loop until the pile size is 0 or if the pool is larger than the critical size
              DO WHILE ( ip /= 0 .AND. ik < kcrit);
                 ik=ik+1 
                 ii=ipile(ip,1); ij=ipile(ip,2)
-               
+
                 ! update bathy and update pile size
                 zbathy(ii,ij)=0.0 
                 ipile(ip,:)  =[0,0]; ip=ip-1
-                
+
                 ! check neighbour cells and update pile
                 iip1=ii+1; IF ( iip1 == npiglo+1 ) iip1=2
                 iim1=ii-1; IF ( iim1 == 0        ) iim1=npiglo-1
                 IF (zbathy(ii, ij+1) /=  0.0) THEN
-                    ip=ip+1; ipile(ip,:)=[ii  ,ij+1] 
-                    ioptm (ii, ij+1) = 0
+                   ip=ip+1; ipile(ip,:)=[ii  ,ij+1] 
+                   ioptm (ii, ij+1) = 0
                 END IF
                 IF (zbathy(ii, ij-1) /= 0.0) THEN
-                    ip=ip+1; ipile(ip,:)=[ii  ,ij-1]
-                    ioptm(ii, ij-1) = 0
+                   ip=ip+1; ipile(ip,:)=[ii  ,ij-1]
+                   ioptm(ii, ij-1) = 0
                 END IF
                 IF (zbathy(iip1, ij) /=  0.0) THEN
-                    ip=ip+1; ipile(ip,:)=[iip1,ij  ]
-                    ioptm(iip1, ij) = 0
+                   ip=ip+1; ipile(ip,:)=[iip1,ij  ]
+                   ioptm(iip1, ij) = 0
                 END IF
                 IF (zbathy(iim1, ij) /=  0.0) THEN
-                    ip=ip+1; ipile(ip,:)=[iim1,ij  ]
-                    ioptm(iim1, ij) = 0
+                   ip=ip+1; ipile(ip,:)=[iim1,ij  ]
+                   ioptm(iim1, ij) = 0
                 END IF
              END DO
              IF (ik < kcrit) bathy=zbathy;
           END IF
        END DO
     END DO
- 
+
     DEALLOCATE(ipile); DEALLOCATE(zbathy, ioptm)
 
-  END SUBROUTINE
+  END SUBROUTINE fillpool
 
 
 END PROGRAM cdfbathy
