@@ -39,15 +39,17 @@ PROGRAM cdfbuoyflx
   !!           3.0  : 09/2015  : J.M. Molines : add nc4 capabilities, optional output file
   !!                                            short output,
   !!                                            different management of read fluxes (XIOS ...)
+  !!         : 4.0  : 03/2017  : J.M. Molines  
   !!----------------------------------------------------------------------
   USE cdfio
   USE eos
   USE modcdfnames
   !!----------------------------------------------------------------------
-  !! CDFTOOLS_3.0 , MEOM 2011
+  !! CDFTOOLS_4.0 , MEOM 2017 
   !! $Id$
-  !! Copyright (c) 2010, J.-M. Molines
+  !! Copyright (c) 2017, J.-M. Molines 
   !! Software governed by the CeCILL licence (Licence/CDFTOOLSCeCILL.txt)
+  !! @class forcing
   !!----------------------------------------------------------------------
   IMPLICIT NONE
 
@@ -65,7 +67,7 @@ PROGRAM cdfbuoyflx
   REAL(KIND=4)                              :: Grav = 9.81                       ! Gravity
 
   REAL(KIND=4)                              :: zsps                              ! Missing value for salinity
-  REAL(KIND=4), DIMENSION(:),   ALLOCATABLE :: tim, zdep                         ! time counter, deptht
+  REAL(KIND=4), DIMENSION(:),   ALLOCATABLE :: zdep                         ! deptht
   REAL(KIND=4), DIMENSION(:,:), ALLOCATABLE :: zmask, zcoefq, zcoefw             ! work array
   REAL(KIND=4), DIMENSION(:,:), ALLOCATABLE :: zalbet, zbeta                     ! work array
   REAL(KIND=4), DIMENSION(:,:), ALLOCATABLE :: evap, precip, runoff, wdmp, wnet  ! water flux components
@@ -77,7 +79,10 @@ PROGRAM cdfbuoyflx
   REAL(KIND=4), DIMENSION(:,:), ALLOCATABLE :: b_qsw , bh_net                    ! BF heat flux components
   REAL(KIND=4), DIMENSION(:,:), ALLOCATABLE :: zsst, zsss, buoyancy_fl           ! Total buoyancy flux
 
+  REAL(KIND=8), DIMENSION(:),   ALLOCATABLE :: dtim                              ! time counter
+
   CHARACTER(LEN=256)                        :: cf_tfil ,cf_flxfil, cf_rnfil      ! input file gridT, flx and runoff
+  CHARACTER(LEN=256)                        :: cf_sfil                           ! input salinity file (option)
   CHARACTER(LEN=256)                        :: cf_out='buoyflx.nc'               ! output file
   CHARACTER(LEN=256)                        :: cldum                             ! dummy character variable
   CHARACTER(LEN=256)                        :: cv_sss                            ! Actual name for SSS
@@ -85,9 +90,9 @@ PROGRAM cdfbuoyflx
 
   TYPE(variable), ALLOCATABLE, DIMENSION(:) :: stypvar                           ! structure for attributes
 
-  LOGICAL                                   :: lchk =.false.                     ! flag for missing files
-  LOGICAL                                   :: lnc4 =.false.                     ! flag for netcdf4 output
-  LOGICAL                                   :: lsho =.false.                     ! flag for short output
+  LOGICAL                                   :: lchk =.FALSE.                     ! flag for missing files
+  LOGICAL                                   :: lnc4 =.FALSE.                     ! flag for netcdf4 output
+  LOGICAL                                   :: lsho =.FALSE.                     ! flag for short output
   !!----------------------------------------------------------------------
   CALL ReadCdfNames()
 
@@ -95,7 +100,7 @@ PROGRAM cdfbuoyflx
   narg= iargc()
   IF ( narg == 0 ) THEN
      PRINT *,' usage : cdfbuoyflx  -t T-file [-r RNF-file] [-f FLX-file ] [-sss SSS-name]'
-     PRINT *,'     ... [-sst SST-name] [-nc4] [-o output_file]  [-short ]'
+     PRINT *,'     ... [-s S-file] [-sst SST-name] [-nc4] [-o OUT-file] [-short ]'
      PRINT *,'      '
      PRINT *,'     PURPOSE :'
      PRINT *,'       Compute (or read) the heat and water fluxes components.'
@@ -106,21 +111,23 @@ PROGRAM cdfbuoyflx
      PRINT *,'      '
      PRINT *,'     ARGUMENTS :'
      PRINT *,'       -t T-file   : netcdf file with temperature and salinity '
+     PRINT *,'                 If salinity not in T-file, use -s option.'
      PRINT *,'      '
      PRINT *,'      '
      PRINT *,'     OPTIONS :'
-     PRINT *,'       [ -r RNF-file ] : Specify a run-off file if runoff not in T-file '
+     PRINT *,'       [-s S-file   ] : Specify salinity file if not T-file.' 
+     PRINT *,'       [-r RNF-file ] : Specify a run-off file if runoff not in T-file '
      PRINT *,'                         nor in FLX-file'
-     PRINT *,'       [ -f FLX-file ] : Use this option if fluxes are not saved in gridT files'
-     PRINT *,'       [ -sss SSS-name ] : Use this option if SSS variable name in T-file '
+     PRINT *,'       [-f FLX-file ] : Use this option if fluxes are not saved in gridT files'
+     PRINT *,'       [-sss SSS-name ] : Use this option if SSS variable name in T-file '
      PRINT *,'                          differ from ',TRIM(cn_vosaline)
-     PRINT *,'       [ -sst SST-name ] : Use this option if SST variable name in T-file '
+     PRINT *,'       [-sst SST-name ] : Use this option if SST variable name in T-file '
      PRINT *,'                          differ from ',TRIM(cn_votemper)
-     PRINT *,'       [ -nc4 ] Use netcdf4 output with chunking and deflation level 1'
+     PRINT *,'       [-nc4 ] Use netcdf4 output with chunking and deflation level 1.'
      PRINT *,'               This option is effective only if cdftools are compiled with'
      PRINT *,'               a netcdf library supporting chunking and deflation.'
-     PRINT *,'       [ -o output_file ] Default is ', TRIM(cf_out)
-     PRINT *,'       [ -short ] With this option only save the buoyancy flux without '
+     PRINT *,'       [-o OUT-file ] Default is ', TRIM(cf_out)
+     PRINT *,'       [-short ] With this option only save the buoyancy flux without '
      PRINT *,'                  all the components of the flux.'
      PRINT *,'      '
      PRINT *,'     REQUIRED FILES :'
@@ -130,21 +137,12 @@ PROGRAM cdfbuoyflx
      PRINT *,'       netcdf file : ', TRIM(cf_out) 
      PRINT *,'         variables : 25 variables (2D) or 1 variable in case of -short option'
      PRINT *,'      '
-     PRINT *,'     WARNING : '
-     PRINT *,'         cdfbuoyflx is not working yet with CMIP6 output' 
      PRINT *,'     SEE ALSO :'
      PRINT *,'      '
-     PRINT *,'      '
-     STOP
+     STOP 
   ENDIF
-
-  IF (TRIM(cn_output) == 'CMIP6') THEN
-      PRINT *, 'cdfbuoyflx is not working yet with CMIP6 output'
-      STOP 99
-  END IF
-
-
   ijarg   = 1
+  cf_sfil = 'none'
   cf_flxfil='none'
   cf_rnfil='none'
   cv_sss=cn_vosaline
@@ -153,42 +151,36 @@ PROGRAM cdfbuoyflx
   DO   WHILE ( ijarg <= narg ) 
      CALL getarg (ijarg, cldum) ; ijarg = ijarg + 1
      SELECT CASE ( cldum )
-     CASE ( '-f' )     ! specify  flx files
-        CALL getarg (ijarg, cf_flxfil) ; ijarg = ijarg + 1
-        lchk = lchk .OR. chkfile (cf_flxfil)
-     CASE ( '-t' )     ! specify  T files
-        CALL getarg (ijarg, cf_tfil) ; ijarg = ijarg + 1
-        lchk = lchk .OR. chkfile (cf_tfil  )
-     CASE ( '-r' )     ! specify  runoff files
-        CALL getarg (ijarg, cf_rnfil) ; ijarg = ijarg + 1
-        lchk = lchk .OR. chkfile (cf_rnfil )
-     CASE ( '-sss' )     ! specify  runoff files
-        CALL getarg (ijarg, cv_sss) ; ijarg = ijarg + 1
-     CASE ( '-sst' )     ! specify  runoff files
-        CALL getarg (ijarg, cv_sst) ; ijarg = ijarg + 1
-     CASE ( '-nc4' )   !  allow chunking and deflation on output
-        lnc4 = .true.
-     CASE ( '-o' )     ! specify  output files
-        CALL getarg (ijarg, cf_out) ; ijarg = ijarg + 1
-     CASE ( '-short' ) ! use short output ( only buoyancy fluxes )
-        lsho = .true. ; np_varout = 1
-     CASE DEFAULT
-        PRINT *, " Option ", TRIM(cldum)," not supported "
-        STOP 99
+     CASE ( '-f'   ) ; CALL getarg (ijarg, cf_flxfil) ; ijarg = ijarg + 1
+        ;              lchk = lchk .OR. chkfile (cf_flxfil)
+     CASE ( '-t'   ) ; CALL getarg (ijarg, cf_tfil) ; ijarg = ijarg + 1
+        ;              lchk = lchk .OR. chkfile (cf_tfil  )
+     CASE ( '-s'   ) ; CALL getarg (ijarg, cf_sfil) ; ijarg = ijarg + 1
+        ;              lchk = lchk .OR. chkfile (cf_sfil  )
+     CASE ( '-r'   ) ; CALL getarg (ijarg, cf_rnfil) ; ijarg = ijarg + 1
+        ;              lchk = lchk .OR. chkfile (cf_rnfil )
+     CASE ( '-sss' ) ; CALL getarg (ijarg, cv_sss) ; ijarg = ijarg + 1
+     CASE ( '-sst' ) ; CALL getarg (ijarg, cv_sst) ; ijarg = ijarg + 1
+     CASE ( '-nc4' ) ; lnc4 = .TRUE.
+     CASE ( '-o'   ) ; CALL getarg (ijarg, cf_out) ; ijarg = ijarg + 1
+     CASE ('-short') ; lsho = .TRUE. ; np_varout = 1
+     CASE DEFAULT    ; PRINT *,' ERROR : ',TRIM(cldum),' : unknown option.' ; STOP 99
      END SELECT
   ENDDO
+
+  IF ( cf_sfil == 'none' ) cf_sfil=cf_tfil
   IF (lchk ) STOP 99 ! missing files
 
   IF ( cf_flxfil == 'none' ) THEN
-    cf_flxfil = cf_tfil
+     cf_flxfil = cf_tfil
   ENDIF
   ! If no runoff file specified, assume that run off are in flx file [ which must be read by the way ... ]
   IF ( cf_rnfil == 'none' ) THEN
-    cf_rnfil = cf_flxfil
+     cf_rnfil = cf_flxfil
   ENDIF
 
   ! Look for Missing value for salinity
-  zsps = getspval(cf_tfil, cn_vosaline)
+  zsps = getspval(cf_sfil, cn_vosaline)
 
   npiglo = getdim (cf_tfil,cn_x)
   npjglo = getdim (cf_tfil,cn_y)
@@ -208,11 +200,11 @@ PROGRAM cdfbuoyflx
 
   ! allocated only for full output
   IF ( .NOT. lsho ) THEN
-  ALLOCATE ( evap(npiglo,npjglo), precip(npiglo,npjglo), runoff(npiglo,npjglo), wdmp(npiglo,npjglo) )
-  ALLOCATE ( wice(npiglo,npjglo), precip_runoff(npiglo,npjglo) )
-  ALLOCATE ( qlat(npiglo,npjglo), qsb(npiglo,npjglo), qlw(npiglo,npjglo), qsw(npiglo,npjglo) )
-  ALLOCATE ( b_evap(npiglo,npjglo), b_precip(npiglo,npjglo), b_runoff(npiglo,npjglo), b_wdmp(npiglo,npjglo) ) 
-  ALLOCATE ( b_qlat(npiglo,npjglo), b_qsb(npiglo,npjglo),    b_qlw(npiglo,npjglo),    b_qsw(npiglo,npjglo)  )
+     ALLOCATE ( evap(npiglo,npjglo), precip(npiglo,npjglo), runoff(npiglo,npjglo), wdmp(npiglo,npjglo) )
+     ALLOCATE ( wice(npiglo,npjglo), precip_runoff(npiglo,npjglo) )
+     ALLOCATE ( qlat(npiglo,npjglo), qsb(npiglo,npjglo), qlw(npiglo,npjglo), qsw(npiglo,npjglo) )
+     ALLOCATE ( b_evap(npiglo,npjglo), b_precip(npiglo,npjglo), b_runoff(npiglo,npjglo), b_wdmp(npiglo,npjglo) ) 
+     ALLOCATE ( b_qlat(npiglo,npjglo), b_qsb(npiglo,npjglo),    b_qlw(npiglo,npjglo),    b_qsw(npiglo,npjglo)  )
   ENDIF
 
   DO jt = 1, npt
@@ -239,47 +231,47 @@ PROGRAM cdfbuoyflx
      END WHERE
 
      IF ( .NOT. lsho ) THEN
-     ! Evap : 
-     qlat(:,:)= getvar(cf_flxfil, cn_solhflup, 1, npiglo, npjglo, ktime=jt) *zmask(:,:)    ! W/m2 
-     evap(:,:)= -1.* qlat(:,:) /Lv*86400. *zmask(:,:)                                    ! mm/days
+        ! Evap : 
+        qlat(:,:)= getvar(cf_flxfil, cn_solhflup, 1, npiglo, npjglo, ktime=jt) *zmask(:,:)    ! W/m2 
+        evap(:,:)= -1.* qlat(:,:) /Lv*86400. *zmask(:,:)                                    ! mm/days
 
-     ! Wdmp
-     wdmp(:,:)= getvar(cf_flxfil, cn_sowafldp, 1, npiglo, npjglo, ktime=jt)*86400.*zmask(:,:) ! mm/days
+        ! Wdmp
+        wdmp(:,:)= getvar(cf_flxfil, cn_sowafldp, 1, npiglo, npjglo, ktime=jt)*86400.*zmask(:,:) ! mm/days
 
-     ! Runoff  ! take care : not a model output (time_counter may disagree ... jmm
-     runoff(:,:)= getvar(cf_rnfil, 'sorunoff', 1, npiglo, npjglo)*86400.*zmask(:,:)         ! mm/days
+        ! Runoff  ! take care : not a model output (time_counter may disagree ... jmm
+        runoff(:,:)= getvar(cf_rnfil, 'sorunoff', 1, npiglo, npjglo)*86400.*zmask(:,:)         ! mm/days
 
-     ! fsalt = contribution of ice freezing and melting to salinity ( + = freezing, - = melting )Q
-     wice(:,:) = getvar(cf_flxfil, cn_iowaflup, 1, npiglo, npjglo, ktime=jt )*86400.*zmask(:,:)          ! mm/days
+        ! fsalt = contribution of ice freezing and melting to salinity ( + = freezing, - = melting )Q
+        wice(:,:) = getvar(cf_flxfil, cn_iowaflup, 1, npiglo, npjglo, ktime=jt )*86400.*zmask(:,:)          ! mm/days
 
-     ! Precip:
-     precip(:,:)= evap(:,:)-runoff(:,:)+wdmp(:,:)-wnet(:,:)+wice(:,:)                     ! mm/day
+        ! Precip:
+        precip(:,:)= evap(:,:)-runoff(:,:)+wdmp(:,:)-wnet(:,:)+wice(:,:)                     ! mm/day
 
-     ! Precip+runoff : (as a whole ) (interpolated on line)
-     precip_runoff(:,:)= evap(:,:)+wdmp(:,:)-wnet(:,:)+wice(:,:)                          ! mm/day
+        ! Precip+runoff : (as a whole ) (interpolated on line)
+        precip_runoff(:,:)= evap(:,:)+wdmp(:,:)-wnet(:,:)+wice(:,:)                          ! mm/day
 
-     ! other heat fluxes
-     qsb(:,:)= getvar(cf_flxfil, cn_sosbhfup,  1, npiglo, npjglo, ktime = jt )*zmask(:,:)    ! W/m2 
-     qlw(:,:)= getvar(cf_flxfil, cn_solwfldo,  1, npiglo, npjglo, ktime = jt )*zmask(:,:)    ! W/m2 
-     qsw(:,:)= getvar(cf_flxfil, cn_soshfldo,  1, npiglo, npjglo, ktime = jt )*zmask(:,:)    ! W/m2 
+        ! other heat fluxes
+        qsb(:,:)= getvar(cf_flxfil, cn_sosbhfup,  1, npiglo, npjglo, ktime = jt )*zmask(:,:)    ! W/m2 
+        qlw(:,:)= getvar(cf_flxfil, cn_solwfldo,  1, npiglo, npjglo, ktime = jt )*zmask(:,:)    ! W/m2 
+        qsw(:,:)= getvar(cf_flxfil, cn_soshfldo,  1, npiglo, npjglo, ktime = jt )*zmask(:,:)    ! W/m2 
 
 
-     b_qlat=0.  ; b_qlw=0.    ; b_qsw=0.   ; b_qsb=0.
-     b_evap=0.  ; b_precip=0. ; b_wdmp=0.  ; b_runoff=0.
+        b_qlat=0.  ; b_qlw=0.    ; b_qsw=0.   ; b_qsb=0.
+        b_evap=0.  ; b_precip=0. ; b_wdmp=0.  ; b_runoff=0.
 
-     WHERE (zsss /= 0 ) 
-        b_qlat(:,:)= zcoefq * qlat
-        b_qlw (:,:)= zcoefq * qlw
-        b_qsw (:,:)= zcoefq * qsw
-        b_qsb (:,:)= zcoefq * qsb
+        WHERE (zsss /= 0 ) 
+           b_qlat(:,:)= zcoefq * qlat
+           b_qlw (:,:)= zcoefq * qlw
+           b_qsw (:,:)= zcoefq * qsw
+           b_qsb (:,:)= zcoefq * qsb
 
-        b_evap(:,:)= zcoefw * evap
-        b_precip(:,:)= -zcoefw * precip
-        b_runoff(:,:)= -zcoefw * runoff
-        b_wdmp(:,:)= zcoefw * wdmp
+           b_evap(:,:)= zcoefw * evap
+           b_precip(:,:)= -zcoefw * precip
+           b_runoff(:,:)= -zcoefw * runoff
+           b_wdmp(:,:)= zcoefw * wdmp
 
-        !    buoyancy_fl(:,:) = zcoefq * qnet +zcoefw * wnet
-     END WHERE
+           !    buoyancy_fl(:,:) = zcoefq * qnet +zcoefw * wnet
+        END WHERE
      ENDIF
 
      ! Write output file
@@ -319,8 +311,8 @@ PROGRAM cdfbuoyflx
      ENDIF
   END DO  ! time loop
 
-  tim  = getvar1d(cf_tfil, cn_vtimec, npt     )
-  ierr = putvar1d(ncout,   tim,       npt, 'T')
+  dtim = getvar1d(cf_tfil, cn_vtimec, npt     )
+  ierr = putvar1d(ncout,   dtim,      npt, 'T')
 
   ierr=closeout(ncout)
 
@@ -338,7 +330,7 @@ CONTAINS
     INTEGER(KIND=4) :: jv   ! dummy loop index
     !!----------------------------------------------------------------------
     ! prepare output variables
-    ALLOCATE (zdep(1), tim(npt) )
+    ALLOCATE (zdep(1), dtim(npt) )
     zdep(1) = 0.
     ALLOCATE (ipk(np_varout), id_varout(np_varout), stypvar(np_varout) )
     ipk(:)  = 1  ! all variables ( output are 2D)
