@@ -11,6 +11,7 @@ PROGRAM cdfcoloc
   !!
   !! History : 2.1  : 05/2007  : J.M. Molines : Original code
   !!           3.0  : 03/2011  : J.M. Molines : Doctor norm + Lic.
+  !!         : 4.0  : 03/2017  : J.M. Molines  
   !!----------------------------------------------------------------------
   !!----------------------------------------------------------------------
   !! subroutine  rotation    : perform vector rotation to get geographical
@@ -22,10 +23,11 @@ PROGRAM cdfcoloc
   USE cdfio
   USE modcdfnames
   !!----------------------------------------------------------------------
-  !! CDFTOOLS_3.0 , MEOM 2011
+  !! CDFTOOLS_4.0 , MEOM 2017 
   !! $Id$
-  !! Copyright (c) 2011, J.-M. Molines
+  !! Copyright (c) 2017, J.-M. Molines 
   !! Software governed by the CeCILL licence (Licence/CDFTOOLSCeCILL.txt)
+  !! @class data_transformation
   !!----------------------------------------------------------------------
   IMPLICIT NONE
 
@@ -53,7 +55,7 @@ PROGRAM cdfcoloc
   INTEGER(KIND=4)                                :: nSx, nSy      ! index of the Sx and Sy for rotation
   INTEGER(KIND=4)                                :: nU, nV        ! index of the U and V for rotation
   INTEGER(KIND=2), DIMENSION(:,:,:), ALLOCATABLE :: mask          ! 3D working mask
-  
+
   REAL(KIND=4)                                   :: xmin, ymin, rdep
   REAL(KIND=4)                                   :: vup, vdo, wup, wdo  ! Working variables
   REAL(KIND=4), DIMENSION(:,:,:),    ALLOCATABLE :: v3d           ! 3D  ! working variable (unavoidable)
@@ -71,6 +73,8 @@ PROGRAM cdfcoloc
   CHARACTER(LEN=256)                             :: cf_weight 
   CHARACTER(LEN=256)                             :: cf_weight_root
   CHARACTER(LEN=256)                             :: cf_gridt   = 'none'
+  CHARACTER(LEN=256)                             :: cf_grids   = 'none'
+  CHARACTER(LEN=256)                             :: cf_grid2d  = 'none'
   CHARACTER(LEN=256)                             :: cf_gridtrc = 'none'
   CHARACTER(LEN=256)                             :: cf_diag    = 'none'
   CHARACTER(LEN=256)                             :: cf_gridu   = 'none'
@@ -84,8 +88,8 @@ PROGRAM cdfcoloc
   CHARACTER(LEN=256)                             :: cldum   ! dummy char variable for line input
   CHARACTER(LEN=256)                             :: ctmplst0 ! current list of type: separated by ,
   CHARACTER(LEN=256)                             :: cformat  ! ASCII format adapted to ntyp
-  CHARACTER(LEN=12), DIMENSION(jptyp)            :: ctype  !  all possible type defined there
-  CHARACTER(LEN=12), DIMENSION(:),ALLOCATABLE    :: cltype !  actual type used given as argument
+  CHARACTER(LEN=15), DIMENSION(jptyp)            :: ctype  !  all possible type defined there
+  CHARACTER(LEN=15), DIMENSION(:),ALLOCATABLE    :: cltype !  actual type used given as argument
 
   LOGICAL                                        :: llchk
   !!----------------------------------------------------------------------
@@ -93,48 +97,91 @@ PROGRAM cdfcoloc
 
   ! exhaustive list of supported field     
   ctype    = (/'T       ','S       ','SSH     ','CFCINV  ','CFCCONC ','PENDEP  ',   &
-     &        'MXL     ','MXL01   ','MXLT02  ','ISOTHICK','U       ','V       ', &
-     &        'Sx      ','Sy      ','H       ','etopo   '/)
+       &        'MXL     ','MXL01   ','MXLT02  ','ISOTHICK','U       ','V       ', &
+       &        'Sx      ','Sy      ','H       ','etopo   '/)
   ctmplst0 = 'U,V,Sx,Sy,H'                 ! default list
-  !!  Read command line and output usage message if not compliant.
+
   narg= iargc()
-  IF ( narg <  1  ) THEN
-     PRINT *,' usage : cdfcoloc  -w root_weight -t gridT -trc TRC_file ...'
-     PRINT *,'          ...  -u gridU -v gridV [-l field list ] [-h]'
-     PRINT *,'       -w root_weight  : specify the root name of the weight files'
-     PRINT *,'                         _T.bin, _U.bin, or _V.bin will be appended '
-     PRINT *,'                         to name if necessary.'
-     PRINT *,'       -t gridT file   : name of gridT model file'
-     PRINT *,'       -trc TRC file   : name of gridT model file'
-     PRINT *,'       -d  diag file   : name of specific diagnostic file '
-     PRINT *,'       -u gridU file   : name of gridU model file'
-     PRINT *,'       -v gridV file   : name of gridV model file'
-     PRINT *,'       -b bathy file   : name of etopo like bathymetric file'
-     PRINT *,'       -l field list   : list of fields to be colocated, separated by '',''' 
-     PRINT *,'                         Default list is :',TRIM(ctmplst0)
-     PRINT *,'       -h              : Give the details of available field to colocate.'
-     PRINT *,'     Return a column ascii file id dep fields()'
-     PRINT *,  TRIM(cn_fmsk),' is required in local directory'
-     PRINT *,  TRIM(cn_fcoo),',',TRIM(cn_fzgr),' are also required for slope computation'
-     STOP
+  IF ( narg == 0 ) THEN
+     PRINT *,' usage : cdfcoloc -w ROOT-weight -t T-file -u U-file -v V-file [-s S-file]...'
+     PRINT *,'        ...  [--ssh-file SSH-file] [-h] [-l LST-fields] [-trc TRC-file] ...'
+     PRINT *,'        ...  [-d DIAG-file] [-b ETOPO-file]'
+     PRINT *,'      '
+     PRINT *,'     PURPOSE :'
+     PRINT *,'       This program produces 3D colocalized model values for selected fields.' 
+     PRINT *,'       It is the final pass in the colocalization process initialized by '
+     PRINT *,'       ''cdfweight'', in which the location of the points to be colocalized'
+     PRINT *,'       is set. The 2 steps of the process are separated because weight files'
+     PRINT *,'       are to be produced only once for a set of data-point and model config,'
+     PRINT *,'       whereas ''cdfcoloc'' is used for several model files corresponding to'
+     PRINT *,'       different times.'
+     PRINT *,'       This program was initially written to deal with G. Holloway topostrophy'
+     PRINT *,'       works.'
+     PRINT *,'      '
+     PRINT *,'     ARGUMENTS :'
+     PRINT *,'       -w ROOT-weight : specify the root-name of the weight files (binary '
+     PRINT *,'                files), to which the suffixes ''_T.bin'', ''_U.bin'' or ''_V.bin'''
+     PRINT *,'                are appended if necessary.'
+     PRINT *,'       -t T-file : name of gridT model file, used for default fields.'
+     PRINT *,'              If salinity not in T-file use -s option.'
+     PRINT *,'              If ssh not in T-file use --ssh-file option.'
+     PRINT *,'       -u U-file : name of gridU model file, used for default fields.'
+     PRINT *,'       -v V-file : name of gridV model file, used for default fields.'
+     PRINT *,'      '
+     PRINT *,'     OPTIONS :'
+     PRINT *,'       [-s S-file ] : Give salinity file if not T-file.'
+     PRINT *,'       [--ssh-file SSH-file] : specify the ssh file if not in T-file.'
+     PRINT *,'       [-h ] : Gives details on the available fields.'
+     PRINT *,'       [-l LST-fields ] : Gives a comma-separated list of selected fields to be'
+     PRINT *,'              colocalized, from a whole set of fields which are fully described'
+     PRINT *,'              with the ''-h'' option. The default list is: ',TRIM(ctmplst0)
+     PRINT *,'              According to the selected fields, specific model files are to be'
+     PRINT *,'              passed to the program with corresponding option.'
+     PRINT *,'       [-trc TRC-file]: name of ptrcT model file, used for when passive tracers'
+     PRINT *,'              related fields are selected (CFCINV, CFCCINC or PENDEP).'
+     PRINT *,'       [-d DIAG-file ] : name of specific diagnostic file. This file is used '
+     PRINT *,'              when ''PENDEP'' or ''ISOTHICK'' are selected. It must have the '
+     PRINT *,'              variables ',TRIM(cn_pendep),' or ',TRIM(cn_isothick),', respectively produced by'
+     PRINT *,'              ''cdfpendep'' and ''cdfsigintegr''.'
+     PRINT *,'       [-b ETOPO-file ] : name of ''etopo-like'' bathymetric file.'
+     PRINT *,'      '
+     PRINT *,'     REQUIRED FILES :'
+     PRINT *,'       ',TRIM(cn_fmsk),'. If bathymetric slopes are needed, then'
+     PRINT *,'       ',TRIM(cn_fcoo),' and ',TRIM(cn_fzgr),' files are also required.'
+     PRINT *,'      '
+     PRINT *,'     OUTPUT : '
+     PRINT *,'       Output is a multi columns ASCII file with first 2 columns giving'
+     PRINT *,'            ''ID'' and ''DEPTH''. Then the line is completed with colocated'
+     PRINT *,'            field values. The output file looks pretty much as the input file'
+     PRINT *,'            used in ''cdfweight'' for building the weight files.'
+     PRINT *,'      '
+     PRINT *,'     SEE ALSO :'
+     PRINT *,'      cdfweight '
+     PRINT *,'      '
+     STOP 
   ENDIF
 
   iarg = 1
   DO WHILE ( iarg <= narg ) 
      CALL getarg ( iarg, cldum ) ; iarg = iarg + 1
      SELECT CASE ( cldum )
-     CASE ('-w'   ) ; CALL getarg ( iarg, cf_weight_root ) ; iarg = iarg + 1
-     CASE ('-t'   ) ; CALL getarg ( iarg, cf_gridt       ) ; iarg = iarg + 1
-     CASE ('-trc' ) ; CALL getarg ( iarg, cf_gridtrc     ) ; iarg = iarg + 1
-     CASE ('-d'   ) ; CALL getarg ( iarg, cf_diag        ) ; iarg = iarg + 1
-     CASE ('-u'   ) ; CALL getarg ( iarg, cf_gridu       ) ; iarg = iarg + 1
-     CASE ('-v'   ) ; CALL getarg ( iarg, cf_gridv       ) ; iarg = iarg + 1
-     CASE ('-b'   ) ; CALL getarg ( iarg, cf_bathy       ) ; iarg = iarg + 1
-     CASE ('-l'   ) ; CALL getarg ( iarg, ctmplst0       ) ; iarg = iarg + 1
-     CASE ('-h'   ) ; CALL help_message 
+     CASE ('-w'        ) ; CALL getarg ( iarg, cf_weight_root ) ; iarg = iarg + 1
+     CASE ('-t'        ) ; CALL getarg ( iarg, cf_gridt       ) ; iarg = iarg + 1
+     CASE ('-u'        ) ; CALL getarg ( iarg, cf_gridu       ) ; iarg = iarg + 1
+     CASE ('-v'        ) ; CALL getarg ( iarg, cf_gridv       ) ; iarg = iarg + 1
+        ! options
+     CASE ('-s'        ) ; CALL getarg ( iarg, cf_grids       ) ; iarg = iarg + 1
+     CASE ('--ssh-file') ; CALL getarg ( iarg, cf_grid2d      ) ; iarg = iarg + 1
+     CASE ('-l'        ) ; CALL getarg ( iarg, ctmplst0       ) ; iarg = iarg + 1
+     CASE ('-trc'      ) ; CALL getarg ( iarg, cf_gridtrc     ) ; iarg = iarg + 1
+     CASE ('-d'        ) ; CALL getarg ( iarg, cf_diag        ) ; iarg = iarg + 1
+     CASE ('-b'        ) ; CALL getarg ( iarg, cf_bathy       ) ; iarg = iarg + 1
+     CASE ('-h'        ) ; CALL help_message 
      CASE DEFAULT   ; PRINT *,TRIM(cldum),' : option not available.' ; STOP 99
      END SELECT
   ENDDO
+  IF ( cf_grids  == 'none' ) cf_grids  = cf_gridt
+  IF ( cf_grid2d == 'none' ) cf_grid2d = cf_gridt
 
   ! intepret ctmplst0 to set up cltype list, ntype and build cf_out file name
   CALL getfld( ) 
@@ -153,17 +200,17 @@ PROGRAM cdfcoloc
   llchk = .FALSE.
 
   IF ( cf_bathy /= 'none' ) THEN ! dealing with special case of etopo file
-    llchk = llchk .OR. chkfile(cf_bathy    )
-    IF (llchk ) STOP 99 ! missing files
-    npiglo = getdim (cf_bathy,'lon')
-    npjglo = getdim (cf_bathy,'lat')
-    npk    = 1
+     llchk = llchk .OR. chkfile(cf_bathy    )
+     IF (llchk ) STOP 99 ! missing files
+     npiglo = getdim (cf_bathy,'lon')
+     npjglo = getdim (cf_bathy,'lat')
+     npk    = 1
   ELSE
-    llchk = llchk .OR. chkfile(cn_fmsk    )
-    IF (llchk ) STOP 99 ! missing files
-    npiglo = getdim (cn_fmsk,cn_x)
-    npjglo = getdim (cn_fmsk,cn_y)
-    npk    = getdim (cn_fmsk,cn_z)
+     llchk = llchk .OR. chkfile(cn_fmsk    )
+     IF (llchk ) STOP 99 ! missing files
+     npiglo = getdim (cn_fmsk,cn_x)
+     npjglo = getdim (cn_fmsk,cn_y)
+     npk    = getdim (cn_fmsk,cn_z)
   ENDIF
 
   ALLOCATE (v3d(npiglo, npjglo, npk), mask(npiglo, npjglo, npk) )
@@ -181,93 +228,93 @@ PROGRAM cdfcoloc
         cf_weight = cf_weight_t
         cf_in     = cf_gridt
         cvar      = cn_votemper 
-        cvmask    = 'tmask'
+        cvmask    = cn_tmask
         npkv      = npk
         dscale    = 1.d0
      CASE ('S')          ! salinity, not used for Greg Holloway output
         cf_weight = cf_weight_t
-        cf_in     = cf_gridt
+        cf_in     = cf_grids
         cvar      = cn_vosaline 
-        cvmask    = 'tmask'
+        cvmask    = cn_tmask
         npkv      = npk
         dscale    = 1.d0
      CASE ('SSH')        !  SSH, not used for Greg Holloway output
         cf_weight = cf_weight_t
-        cf_in     = cf_gridt
+        cf_in     = cf_grid2d
         cvar      = cn_sossheig 
-        cvmask    = 'tmask'
+        cvmask    = cn_tmask
         npkv      = 1
         dscale    = 100.d0
      CASE ('CFCINV')     !  CFC inventory, not used for Greg Holloway output
         cf_weight = cf_weight_t
         cf_in     = cf_gridtrc
         cvar      = cn_invcfc
-        cvmask    = 'tmask'
+        cvmask    = cn_tmask
         npkv      = 1
         dscale    = 1000000.d0
      CASE ('CFCCONC')     !  CFC inventory, not used for Greg Holloway output
         cf_weight = cf_weight_t
         cf_in     = cf_gridtrc
         cvar      = cn_cfc11
-        cvmask    = 'tmask'
+        cvmask    = cn_tmask
         npkv      = 1
         dscale    = 1.d0
      CASE ('PENDEP')     !  CFC penetration depth
         cf_weight = cf_weight_t
         cf_in     = cf_diag
         cvar      = cn_pendep
-        cvmask    = 'tmask'
+        cvmask    = cn_tmask
         npkv      = 1
         dscale    = 1.d0
      CASE ('MXL','MXL01' )  !  Mixed layer depth
         cf_weight = cf_weight_t
         cf_in     = cf_gridt
         cvar      = cn_somxl010
-        cvmask    = 'tmask'
+        cvmask    = cn_tmask
         npkv      = 1
         dscale    = 1.d0
      CASE ('MXLT02' )  !  Mixed layer depth
         cf_weight = cf_weight_t
         cf_in     = cf_gridt
         cvar      = cn_somxlt02
-        cvmask    = 'tmask'
+        cvmask    = cn_tmask
         npkv      = 1
         dscale    = 1.d0
-     CASE ('ISOTHICK' )  !  Mixed layer depth
+     CASE ('ISOTHICK' )  !  Isopycnal thickness
         cf_weight = cf_weight_t
         cf_in     = cf_diag
         cvar      = cn_isothick
-        cvmask    = 'tmask'
+        cvmask    = cn_tmask
         npkv      = 1
         dscale    = 1.d0
      CASE ('U')          ! Zonal component of velocity 
         cf_weight = cf_weight_u
         cf_in     = cf_gridu
         cvar      = cn_vozocrtx 
-        cvmask    = 'umask'
+        cvmask    = cn_umask
         npkv      = npk
         dscale    = 100.d0  ! to be cm/s in the output
      CASE ('V')             ! Meridional component of velocity
         cf_weight = cf_weight_v
         cf_in     = cf_gridv
         cvar      = cn_vomecrty 
-        cvmask    = 'vmask'
+        cvmask    = cn_vmask
         npkv      = npk
         dscale    = 100.d0  ! to be cm/s in the output
      CASE ('Sx')            ! Zonal component of bottom slope
         cf_weight = cf_weight_u
         cf_in     = 'none' 
         cvar      = 'none'
-        cvmask    = 'umask'
+        cvmask    = cn_umask
         npkv      = 1
         dscale    = 100.d0  ! to be in % in the output
         llchk = llchk .OR. chkfile(cn_fcoo    )
         llchk = llchk .OR. chkfile(cn_fzgr    )
-        IF ( llchk ) STOP 
+        IF ( llchk ) STOP 99 
         ! Sx is the i-slope of bottom topog: read adequate metric
         !   and compute it on v3d(:,:,1)
-        de(:,:)  = getvar(cn_fcoo, cn_ve1u, 1, npiglo, npjglo)     
-        d2d(:,:) = getvar(cn_fzgr, 'hdepw', 1, npiglo, npjglo)     
+        de(:,:)  = getvar(cn_fcoo, cn_ve1u,  1, npiglo, npjglo)     
+        d2d(:,:) = getvar(cn_fzgr, cn_hdepw, 1, npiglo, npjglo)     
         DO ji=2, npiglo-1
            v3d(ji,:,1) = (d2d(ji+1,:) - d2d(ji,:)) / de(ji,:)
         END DO
@@ -275,24 +322,24 @@ PROGRAM cdfcoloc
         cf_weight = cf_weight_v
         cf_in     = 'none' 
         cvar      = 'none'
-        cvmask    = 'vmask'
+        cvmask    = cn_vmask
         npkv      = 1
         dscale    = 100.d0  ! to be in % in the output
         llchk = llchk .OR. chkfile(cn_fcoo    )
         llchk = llchk .OR. chkfile(cn_fzgr    )
-        IF ( llchk ) STOP 
+        IF ( llchk ) STOP 99 
         ! Sy is the j-slope of bottom topog: read adequate metric
         !   and compute it on v3d(:,:,1)
-        de(:,:)  = getvar(cn_fcoo, cn_ve2v, 1, npiglo, npjglo)
-        d2d(:,:) = getvar(cn_fzgr, 'hdepw', 1, npiglo, npjglo)
+        de(:,:)  = getvar(cn_fcoo, cn_ve2v,  1, npiglo, npjglo)
+        d2d(:,:) = getvar(cn_fzgr, cn_hdepw, 1, npiglo, npjglo)
         DO jj=2, npjglo-1
            v3d(:,jj,1) = (d2d(:,jj+1) - d2d(:,jj)) / de(:,jj)
         END DO
      CASE ('H')          ! Bottom topography
         cf_weight = cf_weight_t
         cf_in     = cn_fzgr
-        cvar      = 'hdepw' 
-        cvmask    = 'tmask'
+        cvar      = cn_hdepw 
+        cvmask    = cn_tmask
         npkv      = 1
         dscale    = 1.d0
      CASE ('etopo')       ! Bottom topography from external file
@@ -334,16 +381,16 @@ PROGRAM cdfcoloc
 
      ! read corresponding mask
      IF ( cvmask == 'none' ) THEN   ! special case of etopo files ( valid values are < 0 ) 
-          mask = 1
-          WHERE ( v3d >= 0 ) mask = 0
+        mask = 1
+        WHERE ( v3d >= 0 ) mask = 0
      ELSE
         DO jk=1, npkv
-          mask(:,:,jk)=getvar(cn_fmsk,cvmask,jk, npiglo,npjglo)
+           mask(:,:,jk)=getvar(cn_fmsk,cvmask,jk, npiglo,npjglo)
         END DO
      ENDIF
 
      DO jid=1,nid
-!       READ(numbin) id, dymin, dxmin, idep ,nimin, njmin, nkmin, nquadran, dhN, dalpha, dbeta, dgama
+        !       READ(numbin) id, dymin, dxmin, idep ,nimin, njmin, nkmin, nquadran, dhN, dalpha, dbeta, dgama
         READ(numbin) id, ymin, xmin, rdep ,nimin, njmin, nkmin, nquadran, dhN, dalpha, dbeta, dgama
         dinterp(jid,jtyp)=interp()
         ! do not scale dummy values
@@ -367,10 +414,10 @@ PROGRAM cdfcoloc
      IF ( dlmin > -99990.d0 ) THEN
         ! apply vector rotation to have results on the geographic reference system (N-S, E-W )
         IF ( nSx > 0 ) THEN   ! (Sx, Sy pair )
-          CALL rotation( dinterp(jid,nSx), dinterp(jid,nSy), dhN)  
+           CALL rotation( dinterp(jid,nSx), dinterp(jid,nSy), dhN)  
         ENDIF
         IF ( nU > 0 ) THEN    ! (U, V pair)
-          CALL rotation( dinterp(jid,nU), dinterp(jid,nV), dhN)  
+           CALL rotation( dinterp(jid,nU), dinterp(jid,nV), dhN)  
         ENDIF
         WRITE(numout, cformat) id, rdep, (dinterp(jid,jtyp),jtyp=1,ntyp)
      ELSE
@@ -510,7 +557,7 @@ CONTAINS
     INTEGER(KIND=4)    :: jt
     CHARACTER(LEN=256) :: cltmplst
     !!----------------------------------------------------------------------
-    
+
     cltmplst = ctmplst0
     ntyp = 1
     idum = INDEX(cltmplst,',')
@@ -544,15 +591,15 @@ CONTAINS
           STOP 99
        ENDIF
     ENDDO
-    
+
     ! locate pairing for vector variables
     nSx = -1 ; nSy = -1
     nU  = -1 ; nV  = -1
     DO jtyp = 1, ntyp
-      IF (  cltype(jtyp) == 'Sx' ) nSx = jtyp
-      IF (  cltype(jtyp) == 'Sy' ) nSy = jtyp
-      IF (  cltype(jtyp) == 'U'  ) nU  = jtyp
-      IF (  cltype(jtyp) == 'V'  ) nV  = jtyp
+       IF (  cltype(jtyp) == 'Sx' ) nSx = jtyp
+       IF (  cltype(jtyp) == 'Sy' ) nSy = jtyp
+       IF (  cltype(jtyp) == 'U'  ) nU  = jtyp
+       IF (  cltype(jtyp) == 'V'  ) nV  = jtyp
     END DO
 
     IF ( nSx * nSy < 0 ) THEN 
@@ -572,7 +619,7 @@ CONTAINS
        cf_out=TRIM(cf_out)//'_'//TRIM(cltype(jtyp))
     ENDDO
     cf_out=TRIM(cf_out)//'.txt'
-    
+
     ! Build output format
     WRITE(cformat,'(a,i2,a)') '(I5,  I6,',ntyp,'e14.6)'
 
@@ -586,51 +633,58 @@ CONTAINS
     !!              corresponding required input files  
     !!
     !!----------------------------------------------------------------------
-    CHARACTER(LEN=24), DIMENSION(jptyp) :: comments
-    CHARACTER(LEN=10), DIMENSION(jptyp) :: crequired
+    CHARACTER(LEN=25), DIMENSION(jptyp) :: comments
+    CHARACTER(LEN=15), DIMENSION(jptyp) :: crequired
+    CHARACTER(LEN=15)  :: cf_zgr, cf_coo
     !!----------------------------------------------------------------------
     PRINT *,' List of available field to process:' 
-    PRINT *,'field name  comments                input files'
+    PRINT 9001,'field name   ','  comments  ','  input files  '
+    PRINT 9002
+    cf_zgr=cn_fzgr
+    cf_coo=cn_fcoo
 
-    
     ! ctype    = (/'T','S','SSH','CFCINV','CFCCONC','PENDEP','MXL','MXL01',
     !              'MXLT02','ISOTHICK','U ','V ','Sx','Sy','H ','etopo'/)
     comments = (/' Potential temperature  ', &
-          &      ' Salinity               ', &
-          &      ' Sea Surface height     ', &
-          &      ' CFC inventory          ', &
-          &      ' CFC concentration      ', &
-          &      ' Penetration depth      ', &
-          &      ' Mixed layer depth s0.01', &
-          &      ' Mixed layer depth s0.01', &
-          &      ' Mixed layer depth t0.2 ', &
-          &      ' Isopycnal thickness    ', &
-          &      ' Zonal velocity         ', &
-          &      ' Meridional velocity    ', &
-          &      ' Zonal bottom slope     ', &
-          &      ' Meridional bottom slope', &
-          &      ' Local model bathymetry ', &
-          &      ' etopo like bathymetry  ' /)
-    crequired = (/' -t gridT ',   &
-          &       ' -t gridT ',   &
-          &       ' -t gridT ',   &
-          &       ' -trc TRC ',   &
-          &       ' -trc TRC ',   &
-          &       ' -d  diag ',   &
-          &       ' -t gridT ',   &
-          &       ' -t gridT ',   &
-          &       ' -t gridT ',   &
-          &       ' -d  diag ',   &
-          &       ' -u gridU ',   &
-          &       ' -v gridV ',   &
-          &       ' zgr coord',   &
-          &       ' zgr coord',   &
-          &       ' zgr      ',   &
-          &       ' -b etopo ' /)
+         &      ' Salinity               ', &
+         &      ' Sea Surface height     ', &
+         &      ' CFC inventory          ', &
+         &      ' CFC concentration      ', &
+         &      ' Penetration depth      ', &
+         &      ' Mixed layer depth s0.01', &
+         &      ' Mixed layer depth s0.01', &
+         &      ' Mixed layer depth t0.2 ', &
+         &      ' Isopycnal thickness    ', &
+         &      ' Zonal velocity         ', &
+         &      ' Meridional velocity    ', &
+         &      ' Zonal bottom slope     ', &
+         &      ' Meridional bottom slope', &
+         &      ' Local model bathymetry ', &
+         &      ' etopo like bathymetry  ' /)
+    crequired = (/' -t T-file     ',   &
+         &        ' -t T-file     ',   &
+         &        ' -t T-file     ',   &
+         &        ' -trc TRC-file ',   &
+         &        ' -trc TRC-file ',   &
+         &        ' -d  DIAG-file ',   &
+         &        ' -t T-file     ',   &
+         &        ' -t T-file     ',   &
+         &        ' -t T-file     ',   &
+         &        ' -d  DIAG-file ',   &
+         &        ' -u U-file     ',   &
+         &        ' -v V-file     ',   &
+         &            cf_zgr,   &
+         &            cf_coo,   &
+         &            cf_zgr,   &
+         &        ' -b ETOPO-file ' /)
 
     DO jtyp=1, jptyp
-      PRINT '( 12a,x,24a,x,10a)', TRIM(ctype(jtyp)), comments(jtyp), crequired(jtyp)
+       PRINT 9001 , TRIM(ctype(jtyp)), comments(jtyp), crequired(jtyp)
     ENDDO
+    PRINT 9002
+    PRINT *,''
+9001 FORMAT (a15,x,a25,x,a15)
+9002 FORMAT (57("-") )
     STOP 99
 
   END SUBROUTINE help_message
