@@ -18,22 +18,24 @@ PROGRAM cdfmhst
   !!                  10/2012  : M.A. Balmaseda: opt separate V,T,S infiles
   !!                                             opt z dimension 
   !!                                             add inp0
+  !!         : 4.0  : 03/2017  : J.M. Molines  
   !!----------------------------------------------------------------------
   USE cdfio
   USE modcdfnames
   !!----------------------------------------------------------------------
-  !! CDFTOOLS_3.0 , MEOM 2011
+  !! CDFTOOLS_4.0 , MEOM 2017 
   !! $Id$
-  !! Copyright (c) 2011, J.-M. Molines
+  !! Copyright (c) 2017, J.-M. Molines 
   !! Software governed by the CeCILL licence (Licence/CDFTOOLSCeCILL.txt)
+  !! @class transport
   !!----------------------------------------------------------------------
   IMPLICIT NONE
 
   INTEGER(KIND=4)                              :: ji, jj, jk, jt   ! dummy loop index
   INTEGER(KIND=4)                              :: jbasins, jvar    ! dummy loop index
+  INTEGER(KIND=4)                              :: it               ! time index for vvl
   INTEGER(KIND=4)                              :: narg, iargc      ! command line 
   INTEGER(KIND=4)                              :: ik0              ! working integer
-  INTEGER(KIND=4)                              :: ifile            ! dummuy loop
   INTEGER(KIND=4)                              :: ijarg            ! argument counter
   INTEGER(KIND=4)                              :: npiglo, npjglo   ! size of the domain
   INTEGER(KIND=4)                              :: npk, npt, npko   ! size of the domain
@@ -53,7 +55,6 @@ PROGRAM cdfmhst
   REAL(KIND=4), PARAMETER                      :: ppspval= 9999.99 ! missing value
 
   REAL(KIND=4), DIMENSION(:),      ALLOCATABLE :: gdep             ! depth array
-  REAL(KIND=4), DIMENSION(:),      ALLOCATABLE :: tim              ! time counter
   REAL(KIND=4), DIMENSION(:),      ALLOCATABLE :: e31d             ! 1D e3t for full step
   REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: zmask            ! mask
   REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: e1v, e3v, gphiv  ! metrics and latitude
@@ -62,6 +63,7 @@ PROGRAM cdfmhst
   REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: rdumlon          ! dummy longitude = 0.
   REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: rdumlat          ! latitude for i = north pole
 
+  REAL(KIND=8), DIMENSION(:),      ALLOCATABLE :: dtim             ! time counter
   REAL(KIND=8), DIMENSION(:),      ALLOCATABLE :: dzonal_heat_glo  ! zonal integral
   REAL(KIND=8), DIMENSION(:),      ALLOCATABLE :: dzonal_heat_atl  ! zonal integral 
   REAL(KIND=8), DIMENSION(:),      ALLOCATABLE :: dzonal_heat_pac
@@ -96,36 +98,52 @@ PROGRAM cdfmhst
 
   LOGICAL                                      :: llglo = .FALSE.  ! flag for sub basin file
   LOGICAL                                      :: lchk  = .FALSE.  ! flag for missing files
-  LOGICAL                                      :: lfull = .FALSE.  ! flag for missing files
+  LOGICAL                                      :: lfull = .FALSE.  ! flag for full step
   LOGICAL                                      :: lsepf = .FALSE.  ! flag for separate files
-  LOGICAL                                      :: lzdim = .FALSE.  ! flag for separate files
+  LOGICAL                                      :: lzdim = .FALSE.  ! flag for saving cumulated MHT
   !!----------------------------------------------------------------------
   CALL ReadCdfNames()
 
   !!  Read command line and output usage message if not compliant.
   narg= iargc()
   IF ( narg == 0 ) THEN
-     PRINT *,' usage : cdfmhst  VT-file | (V-file T-file [S-file])  [MST] [-full] ...'
-     PRINT *,'              ...  [-Zdim] [-o OUT-file]'
+     PRINT *,' usage : cdfmhst  -vt VT-file | (-v V-file -t T-file [-s S-file]) ...'
+     PRINT *,'         ... [-vtvar VT-var VS-var] ...'
+     PRINT *,'         ... [-MST] [-b BASIN-mask] [-full] [-Zdim] [-o OUT-file] [-vvl] '
      PRINT *,'      '
      PRINT *,'     PURPOSE :'
-     PRINT *,'       Compute the meridional heat/salt transport as a function of '
-     PRINT *,'       latitude. If the file ',TRIM(cn_fbasins),' is provided, the meridional '
+     PRINT *,'       Compute the meridional heat/salt transport as a function of latitude.'
+     PRINT *,'       If the basin mask-file ',TRIM(cn_fbasins),' is provided, the meridional'
      PRINT *,'       heat/salt transport for each sub-basin is also computed.'
      PRINT *,'      '
+     PRINT *,'       This program is designed to read the time-average products V.T into'
+     PRINT *,'       the corresponding VT-file (produced by cdfvT). If VT-files are not'
+     PRINT *,'       available, or if working with model snapshot, V-file and T-file can'
+     PRINT *,'       be specified as an alternative.'
+     PRINT *,'      '
+     PRINT *,'       In the evaluation of the ''meridional'' component, a ''zonal'' integration'
+     PRINT *,'       is performed. Note that in CDFTOOLS, as far as ''zonal'' integrals'
+     PRINT *,'       are concerned, they in-fact correspond to ''along-model I'' integrals.'
+     PRINT *,'      '
      PRINT *,'     ARGUMENTS :'
-     PRINT *,'       VT-file  : netcdf file containing the mean value of the products' 
+     PRINT *,'       -vt VT-file  : netcdf file containing the mean value of the products' 
      PRINT *,'                  U.S, U.T, V.S and V.T (obtained with cdfvT).'
      PRINT *,'         or   '     
-     PRINT *,'       V-file T-file [S-file] : specify V, T S file as separate files. If'
-     PRINT *,'                 S-file is not specified, assume that salinity is in T-file.'
+     PRINT *,'       -v V-file  : specify V-file '
+     PRINT *,'       -t T-file  : specify T-file '
+     PRINT *,'       If not using ''-vt'' option, both V-file and T-file MUST be specified.'
      PRINT *,'      '
      PRINT *,'     OPTIONS :'
-     PRINT *,'       [MST ]   : output flag for meridional salt transport on netcdf files.'
-     PRINT *,'                  If not specified, only the MHT is output.' 
+     PRINT *,'       [-vtvar VT-var VS-var] : name of VT, VS variables in VT-file.'
+     PRINT *,'       [-s S-file] : specify S-file (salinity) if the salinity field is not in'
+     PRINT *,'              T-file.'
+     PRINT *,'       [-MST ]   : Indicates the the meridional Salt transport will also be '
+     PRINT *,'              computed.  If not specified, only the MHT is output.' 
      PRINT *,'       [-full ] : to be set for full step case.'
      PRINT *,'       [-Zdim ] : to be set to output vertical structure of Heat/salt transport'
+     PRINT *,'       [-b BASIN-mask ] : use BASIN-mask instead of default ', TRIM(cn_fbasins)
      PRINT *,'       [-o OUT-file ] : change name of the output file. Default:', TRIM(cf_outnc)
+     PRINT *,'       [-vvl ] : use time-varying vertical metrics.'
      PRINT *,'      '
      PRINT *,'     REQUIRED FILES :'
      PRINT *,'        ', TRIM(cn_fhgr),', ',TRIM(cn_fzgr),' and ',TRIM(cn_fmsk)
@@ -136,7 +154,7 @@ PROGRAM cdfmhst
      PRINT *,'       ASCII files : ', TRIM(cf_outh),' : Meridional Heat Transport'
      PRINT *,'                     ', TRIM(cf_outs),' : Meridional Salt Transport'
      PRINT *,'       netcdf file : ', TRIM(cf_outnc),' unless -o option is used.'
-     PRINT *,'           variables : ( [... ] : MST option ) '
+     PRINT *,'           variables : ( [... ] : -MST option ) '
      PRINT *,'                       ', TRIM(cv_zomht),cbasin(1),' : Meridional Heat Transport (global)'
      PRINT *,'                     [ ', TRIM(cv_zomst),cbasin(1),' : Meridional Salt Transport (global) ] '
      PRINT *,'       If ',TRIM(cn_fbasins),' is available, per basin meridional transport '
@@ -144,51 +162,64 @@ PROGRAM cdfmhst
      DO jbasins=2, 6
         PRINT *,'                       ', TRIM(cv_zomht),cbasin(jbasins),' : Meridional Heat Transport'
         PRINT *,'                     [ ', TRIM(cv_zomst),cbasin(jbasins),' : Meridional Salt Transport ]'
+     PRINT *,'      '
      END DO
-     STOP
+     STOP 
   ENDIF
 
+  cf_vtfil='none'
+  cf_vfil='none'
+  cf_tfil='none'
+  cf_sfil='none'
+  lsepf  = .TRUE.
   npvar   = 1    ! default value ( no MST output)
   ijarg   = 1
-  ifile   = 0
   ! browse command line and detect the file name as argument different from any option
   ! count the number of files. Assume VT if 1 only, V TS if 2 and V T S if 3
   DO WHILE ( ijarg <= narg ) 
      CALL getarg(ijarg, cldum) ; ijarg = ijarg+1
      SELECT CASE ( cldum)
-     CASE ( 'MST' )   ; npvar    = 2
+     CASE ( '-vt'   ) ; CALL getarg(ijarg, cf_vtfil  ) ; ijarg = ijarg+1
+     CASE ( '-vtvar') ; CALL getarg(ijarg, cn_vomevt ) ; ijarg = ijarg+1
+          ;           ; CALL getarg(ijarg, cn_vomevs ) ; ijarg = ijarg+1
+     CASE ( '-v'    ) ; CALL getarg(ijarg, cf_vfil   ) ; ijarg = ijarg+1
+     CASE ( '-t'    ) ; CALL getarg(ijarg, cf_tfil   ) ; ijarg = ijarg+1
+     CASE ( '-s'    ) ; CALL getarg(ijarg, cf_sfil   ) ; ijarg = ijarg+1
+     CASE ( '-MST'  ) ; npvar    = 2
      CASE ( '-full' ) ; lfull    = .TRUE.
      CASE ( '-Zdim' ) ; lzdim    = .TRUE.
-     CASE ( '-o'    ) ; CALL getarg(ijarg, cf_outnc) ; ijarg = ijarg+1
-     CASE DEFAULT     ; ifile = ifile + 1
-        SELECT CASE (ifile)
-        CASE ( 1) ; cf_vtfil = cldum
-        CASE ( 2) ; cf_tfil  = cldum
-        CASE ( 3) ; cf_sfil  = cldum
-        CASE DEFAULT ; PRINT *,' WARNING: more than 3 files in input : weird '
-        END SELECT
+     CASE ( '-b'    ) ; CALL getarg(ijarg, cn_fbasins) ; ijarg = ijarg+1
+     CASE ( '-o'    ) ; CALL getarg(ijarg, cf_outnc  ) ; ijarg = ijarg+1
+     CASE ( '-vvl'  ) ; lg_vvl   = .TRUE.
+     CASE DEFAULT     ; PRINT *,' ERROR : ',TRIM(cldum),' : unknown option.' ; STOP 99
      END SELECT
   END DO
 
   ! security check
-  SELECT CASE (ifile )
-  CASE ( 0 ) ; PRINT *, ' You must provide at least 1 file name (VT) ' ; STOP 99
-  CASE ( 1 ) ; lsepf = .false.; 
-  CASE ( 2 ) ; lsepf = .true. ; cf_vfil = cf_vtfil ; cf_sfil = cf_tfil
-  CASE ( 3 ) ; lsepf = .true. ; cf_vfil = cf_vtfil 
-  END SELECT
+  IF ( TRIM(cf_vtfil) /= 'none' ) lsepf = .FALSE.
+
+  IF ( lsepf ) THEN  ! need to have cf_vfil and cf_tfil at least
+     lchk = chkfile ( cf_vfil )
+     lchk = lchk .OR. chkfile( cf_tfil )
+     IF ( lchk ) STOP 99   ! Missing files
+     IF ( TRIM(cf_sfil) == 'none' ) cf_sfil = cf_tfil
+  ENDIF
 
   ! check for missing files
   lchk = lchk .OR. chkfile( cn_fhgr )
   lchk = lchk .OR. chkfile( cn_fzgr )
   lchk = lchk .OR. chkfile( cn_fmsk )
-  lchk = lchk .OR. chkfile( cf_vtfil)
+  lchk = lchk .OR. chkfile( cf_vtfil)   ! chkfile is returning F when input file name is 'none'
   IF ( lsepf ) THEN
      lchk = lchk .OR. chkfile( cf_tfil)
      lchk = lchk .OR. chkfile( cf_sfil)
   ENDIF
 
   IF ( lchk ) STOP 99 ! missing files
+  IF ( lg_vvl ) THEN
+     cn_fe3v = cf_vtfil  ! REM: in case of vvl e3v is either in VT file or in V file 
+     cn_ve3v = cn_ve3vvvl
+  ENDIF
 
   ! check for sub basin file and set appropriate variables
   IF ( .NOT. chkfile(cn_fbasins ) ) THEN
@@ -202,6 +233,7 @@ PROGRAM cdfmhst
      nbasinso = 1
   ENDIF
 
+  IF ( lsepf ) cf_vtfil=cf_vfil   ! needed only to read the dimension of the domain
   npiglo = getdim (cf_vtfil, cn_x)
   npjglo = getdim (cf_vtfil, cn_y)
   npk    = getdim (cf_vtfil, cn_z)
@@ -221,7 +253,7 @@ PROGRAM cdfmhst
   ENDIF
 
   ! Allocate arrays
-  ALLOCATE ( tim(npt) )
+  ALLOCATE ( dtim(npt) )
   ALLOCATE ( dwkh(npiglo,npjglo), zmask(npiglo,npjglo), zvt(npiglo,npjglo) )
   ALLOCATE ( dwks(npiglo,npjglo), zvs(npiglo,npjglo) )
   ALLOCATE ( e1v(npiglo,npjglo), e3v(npiglo,npjglo), gphiv(npiglo,npjglo))
@@ -261,48 +293,16 @@ PROGRAM cdfmhst
   ALLOCATE(stypvar(nbasinso*npvar),  cvarname(nbasinso*npvar) )
   ALLOCATE(    ipk(nbasinso*npvar), id_varout(nbasinso*npvar) )
 
-  ipk(:)=npko               ! all output variables either 1 or npko levels
-  DO jbasins = 1,nbasinso
-     cvarname(jbasins)                  = TRIM(cv_zomht)//TRIM(cbasin(jbasins))
-     stypvar(jbasins)%cname             = cvarname(jbasins)
-     stypvar(jbasins)%cunits            = 'PW'
-     stypvar(jbasins)%rmissing_value    = ppspval
-     stypvar(jbasins)%valid_min         = -10.
-     stypvar(jbasins)%valid_max         = 20
-     stypvar(jbasins)%clong_name        = 'Meridional Heat Transport '//TRIM(cbasin(jbasins))
-     stypvar(jbasins)%cshort_name       = cvarname(jbasins)
-     stypvar(jbasins)%conline_operation = 'N/A'
-     stypvar(jbasins)%caxis             = cldimension
-
-     IF ( npvar == 2 ) THEN
-        ! MST
-        ivar = nbasinso+jbasins
-        cvarname(ivar)                   = TRIM(cv_zomst)//TRIM(cbasin(jbasins))
-        stypvar(ivar )%cname             = cvarname(ivar)
-        stypvar(ivar )%cunits            = 'T/sec'
-        stypvar(ivar )%rmissing_value    = ppspval
-        stypvar(ivar )%valid_min         = -10.e9
-        stypvar(ivar )%valid_max         = 20.e9
-        stypvar(ivar )%clong_name        = 'Meridional Salt Transport '//TRIM(cbasin(jbasins))
-        stypvar(ivar )%cshort_name       = cvarname(ivar)
-        stypvar(ivar )%conline_operation = 'N/A'
-        stypvar(ivar )%caxis             = cldimension
-     ENDIF
-  END DO
-
-  ! create output fileset
-  ncout = create      (cf_outnc, cf_vtfil, 1,             npjglo, npko, cdep='depthv'                              )
-  ierr  = createvar   (ncout,    stypvar,  nbasinso*npvar, ipk,    id_varout                                       )
-  ierr  = putheadervar(ncout,    cf_vtfil, 1,             npjglo, npko, pnavlon=rdumlon, pnavlat=rdumlat, pdep=gdep)
-
-  tim   = getvar1d (cf_vtfil, cn_vtimec, npt     ) 
-  ierr  = putvar1d (ncout,    tim,       npt, 'T')
+  CALL CreateOutput
 
   OPEN(numouth,FILE=cf_outh,FORM='FORMATTED', RECL=256)  ! to avoid wrapped line with ifort
   OPEN(numouts,FILE=cf_outs,FORM='FORMATTED', RECL=256)  ! to avoid wrapped line with ifort
 
 
   DO jt=1, npt
+     IF ( lg_vvl ) THEN ; it=jt
+     ELSE               ; it=1
+     ENDIF
      dtrph(:,:) = 0.d0
      dtrps(:,:) = 0.d0
      DO jk = 1,npk
@@ -326,10 +326,8 @@ PROGRAM cdfmhst
            zvs(:,:)= getvar(cf_vtfil, cn_vomevs, jk, npiglo, npjglo, ktime=jt)
         ENDIF
         ! get e3v at level jk
-        IF ( lfull ) THEN
-           e3v(:,:) = e31d(jk)
-        ELSE
-           e3v(:,:)  = getvar(cn_fzgr, 'e3v_ps', jk, npiglo, npjglo, ldiom=.TRUE.)
+        IF ( lfull ) THEN ; e3v(:,:) = e31d(jk)
+        ELSE              ; e3v(:,:) = getvar(cn_fe3v, cn_ve3v, jk, npiglo, npjglo, ktime=it, ldiom=.NOT.lg_vvl )
         ENDIF
         dwkh(:,:) = zvt(:,:)*e1v(:,:)*e3v(:,:)*1.d0
         dwks(:,:) = zvs(:,:)*e1v(:,:)*e3v(:,:)*1.d0
@@ -338,9 +336,8 @@ PROGRAM cdfmhst
         dtrph(:,:) = dtrph(:,:) + dwkh(:,:) * pprau0 * pprcp
         dtrps(:,:) = dtrps(:,:) + dwks(:,:)  
 
-
         !global
-        zmask(:,:) = getvar(cn_fmsk, 'vmask', 1, npiglo, npjglo)
+        zmask(:,:) = getvar(cn_fmsk, cn_vmask, 1, npiglo, npjglo)
         DO jj=1,npjglo
            dzonal_heat_glo(jj) = SUM( dtrph(2:npiglo-1,jj)*zmask(2:npiglo-1,jj) )
            dzonal_salt_glo(jj) = SUM( dtrps(2:npiglo-1,jj)*zmask(2:npiglo-1,jj) )
@@ -349,21 +346,21 @@ PROGRAM cdfmhst
         IF ( llglo ) THEN
            ! Zonal mean with mask
            ! Atlantic 
-        zmask(:,:) = getvar(cn_fbasins, cn_tmaskatl, 1, npiglo, npjglo)
+           zmask(:,:) = getvar(cn_fbasins, cn_tmaskatl, 1, npiglo, npjglo)
            DO jj=1,npjglo
               dzonal_heat_atl(jj) = SUM( dtrph(:,jj)*zmask(:,jj) )
               dzonal_salt_atl(jj) = SUM( dtrps(:,jj)*zmask(:,jj) )
            END DO
 
            ! Pacific
-        zmask(:,:) = getvar(cn_fbasins, cn_tmaskpac, 1, npiglo, npjglo)
+           zmask(:,:) = getvar(cn_fbasins, cn_tmaskpac, 1, npiglo, npjglo)
            DO jj=1,npjglo
               dzonal_heat_pac(jj) = SUM( dtrph(:,jj)*zmask(:,jj) )
               dzonal_salt_pac(jj) = SUM( dtrps(:,jj)*zmask(:,jj) )
            END DO
 
            ! Indian
-        zmask(:,:) = getvar(cn_fbasins, cn_tmaskind, 1, npiglo, npjglo)
+           zmask(:,:) = getvar(cn_fbasins, cn_tmaskind, 1, npiglo, npjglo)
            DO jj=1,npjglo
               dzonal_heat_ind(jj) = SUM( dtrph(:,jj)*zmask(:,jj) )
               dzonal_salt_ind(jj) = SUM( dtrps(:,jj)*zmask(:,jj) )
@@ -372,7 +369,7 @@ PROGRAM cdfmhst
            ! Austral
            dzonal_heat_aus = 0.d0
            dzonal_salt_aus = 0.d0
-           !    zmask(:,:)=getvar(cn_fbasins,'tmaskant',1,npiglo,npjglo)
+           !    zmask(:,:)=getvar(cn_fbasins,cn_tmaskant,1,npiglo,npjglo)
            !    DO jj=1,npjglo
            !       dzonal_heat_aus(jj)= SUM( dtrph(:,jj)*zmask(:,jj))
            !       dzonal_salt_aus(jj)= SUM( dtrps(:,jj)*zmask(:,jj))
@@ -382,7 +379,7 @@ PROGRAM cdfmhst
            dzonal_heat_med = 0.d0
            dzonal_salt_med = 0.d0
 
-           !    zmask(:,:)=getvar(cn_fbasins,'tmaskmed',1,npiglo,npjglo)
+           !    zmask(:,:)=getvar(cn_fbasins,cn_tmaskmed,1,npiglo,npjglo)
            !    DO jj=1,npjglo
            !       dzonal_heat_med(jj)= SUM( dtrph(:,jj)*zmask(:,jj))
            !       dzonal_salt_med(jj)= SUM( dtrps(:,jj)*zmask(:,jj))
@@ -390,7 +387,9 @@ PROGRAM cdfmhst
         ENDIF
 
         IF (  lzdim  .OR. ( jk == npk ) ) THEN  !output this level
-           IF ( lzdim ) THEN ; ik0 = jk ;  ELSE ;  ik0 = 1  ; ENDIF
+           IF ( lzdim ) THEN ; ik0 = jk   
+           ELSE              ; ik0 = 1 
+           ENDIF
 
            DO jvar=1,npvar   !  MHT [ and MST ]  (1 or 2 )
               IF ( jvar == 1 ) THEN
@@ -398,59 +397,47 @@ PROGRAM cdfmhst
                  ivar=1
                  dmtrp(:) = dzonal_heat_glo(:)/1.d15                            ! GLO
                  WHERE ( dmtrp == 0 ) dmtrp = ppspval
-                 ierr = putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt)
-                 ivar=ivar+1
+                 ierr = putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt) ; ivar=ivar+1
                  IF ( nbasins == 5 ) THEN
                     dmtrp(:) = dzonal_heat_atl(:)/1.d15                         ! ATL
                     WHERE ( dmtrp == 0 ) dmtrp = ppspval         
-                    ierr = putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt)
-                    ivar=ivar+1
+                    ierr = putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt); ivar=ivar+1
                     dmtrp(:) = (dzonal_heat_ind(:) + dzonal_heat_pac(:))/1.d15  ! INP
                     WHERE ( dmtrp == 0 ) dmtrp = ppspval
-                    ierr = putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt)
-                    ivar=ivar+1
+                    ierr = putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt); ivar=ivar+1
                     dmtrp(:) = dzonal_heat_ind(:)/1.d15                         ! IND
                     WHERE ( dmtrp == 0 ) dmtrp = ppspval
-                    ierr = putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt)
-                    ivar=ivar+1
+                    ierr = putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt); ivar=ivar+1
                     dmtrp(:) = dzonal_heat_pac(:)/1.d15                         ! PAC
                     WHERE ( dmtrp == 0 ) dmtrp = ppspval
-                    ierr = putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt)
-                    ivar=ivar+1
+                    ierr = putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt); ivar=ivar+1
                     ! now inp0
                     dmtrp(:) = ( dzonal_heat_glo(:) - dzonal_heat_atl(:) )/1.d15 ! INP0
                     WHERE ( dmtrp == 0 ) dmtrp = ppspval
-                    ierr = putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt)
-                    ivar=ivar+1
+                    ierr = putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt); ivar=ivar+1
                  ENDIF
               ELSE
                  ! MST
                  dmtrp(:) = dzonal_salt_glo(:)/1.d6                              ! GLO
                  WHERE ( dmtrp == 0 ) dmtrp = ppspval
-                 ierr=putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt)
-                 ivar=ivar+1
+                 ierr = putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt); ivar=ivar+1
                  IF ( nbasins == 5 ) THEN
                     dmtrp(:) = dzonal_salt_atl(:)/1.d6                           ! ATL
                     WHERE ( dmtrp == 0 ) dmtrp = ppspval
-                    ierr = putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt)
-                    ivar=ivar+1
+                    ierr = putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt); ivar=ivar+1
                     dmtrp(:) = (dzonal_salt_ind(:) + dzonal_salt_pac(:))/1.d6    ! INP
                     WHERE ( dmtrp == 0 ) dmtrp = ppspval
-                    ierr = putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt)
-                    ivar=ivar+1
+                    ierr = putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt); ivar=ivar+1
                     dmtrp(:) = dzonal_salt_ind(:)/1.d6                           ! IND
                     WHERE ( dmtrp == 0 ) dmtrp = ppspval
-                    ierr = putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt)
-                    ivar=ivar+1
+                    ierr = putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt); ivar=ivar+1
                     dmtrp(:) = dzonal_salt_pac(:)/1.d6                           ! PAC
                     WHERE ( dmtrp == 0 ) dmtrp = ppspval
-                    ierr=putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt)
-                    ivar=ivar+1
+                    ierr = putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt); ivar=ivar+1
                     ! now inp0
                     dmtrp(:) = ( dzonal_salt_glo(:) - dzonal_salt_atl(:) )/1.d6  ! INP0  
                     WHERE ( dmtrp == 0 ) dmtrp = ppspval
-                    ierr = putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt)
-                    ivar=ivar+1
+                    ierr = putvar(ncout, id_varout(ivar), REAL(dmtrp), ik0, 1, npjglo, ktime=jt); ivar=ivar+1
                  ENDIF
               ENDIF
            END DO
@@ -512,4 +499,54 @@ PROGRAM cdfmhst
 9000 FORMAT(I4,6(1x,f9.3,1x,f8.4))
 9001 FORMAT(I4,6(1x,f9.2,1x,f9.3))
 
+CONTAINS
+
+  SUBROUTINE CreateOutput
+    !!---------------------------------------------------------------------
+    !!                  ***  ROUTINE CreateOutput  ***
+    !!
+    !! ** Purpose :  Create netcdf output file(s) 
+    !!
+    !! ** Method  :  Use stypvar global description of variables
+    !!
+    !!----------------------------------------------------------------------
+    ipk(:)=npko               ! all output variables either 1 or npko levels
+    DO jbasins = 1,nbasinso
+       cvarname(jbasins)                  = TRIM(cv_zomht)//TRIM(cbasin(jbasins))
+       stypvar(jbasins)%cname             = cvarname(jbasins)
+       stypvar(jbasins)%cunits            = 'PW'
+       stypvar(jbasins)%rmissing_value    = ppspval
+       stypvar(jbasins)%valid_min         = -10.
+       stypvar(jbasins)%valid_max         = 20
+       stypvar(jbasins)%clong_name        = 'Meridional Heat Transport '//TRIM(cbasin(jbasins))
+       stypvar(jbasins)%cshort_name       = cvarname(jbasins)
+       stypvar(jbasins)%conline_operation = 'N/A'
+       stypvar(jbasins)%caxis             = cldimension
+
+       IF ( npvar == 2 ) THEN
+          ! MST
+          ivar = nbasinso+jbasins
+          cvarname(ivar)                   = TRIM(cv_zomst)//TRIM(cbasin(jbasins))
+          stypvar(ivar )%cname             = cvarname(ivar)
+          stypvar(ivar )%cunits            = 'T/sec'
+          stypvar(ivar )%rmissing_value    = ppspval
+          stypvar(ivar )%valid_min         = -10.e9
+          stypvar(ivar )%valid_max         = 20.e9
+          stypvar(ivar )%clong_name        = 'Meridional Salt Transport '//TRIM(cbasin(jbasins))
+          stypvar(ivar )%cshort_name       = cvarname(ivar)
+          stypvar(ivar )%conline_operation = 'N/A'
+          stypvar(ivar )%caxis             = cldimension
+       ENDIF
+    END DO
+
+    ! create output fileset
+    ncout = create      (cf_outnc, cf_vtfil, 1,             npjglo, npko, cdep='depthv'                              )
+    ierr  = createvar   (ncout,    stypvar,  nbasinso*npvar, ipk,    id_varout                                       )
+    ierr  = putheadervar(ncout,    cf_vtfil, 1,             npjglo, npko, pnavlon=rdumlon, pnavlat=rdumlat, pdep=gdep)
+
+    dtim  = getvar1d (cf_vtfil, cn_vtimec, npt     ) 
+    ierr  = putvar1d (ncout,    dtim,      npt, 'T')
+
+  END SUBROUTINE CreateOutput
+
 END PROGRAM cdfmhst
