@@ -23,16 +23,18 @@ PROGRAM cdfmocsig
   !!           3.0  : 04/2011  : J.M. Molines  : Doctor norm + Lic.
   !!           3.0  : 06/2013  : J.M. Molines  : add neutral density
   !!           3.0  : 06/2013  : J.M. Molines  : add bin mean depth calculation and OpenMp directives
+  !!         : 4.0  : 03/2017  : J.M. Molines  
   !!----------------------------------------------------------------------
   USE cdfio
   USE eos
   USE modcdfnames
   USE modutils
   !!----------------------------------------------------------------------
-  !! CDFTOOLS_3.0 , MEOM 2011
+  !! CDFTOOLS_4.0 , MEOM 2017 
   !! $Id$
-  !! Copyright (c) 2011, J.-M. Molines
+  !! Copyright (c) 2017, J.-M. Molines 
   !! Software governed by the CeCILL licence (Licence/CDFTOOLSCeCILL.txt)
+  !! @class transport
   !!----------------------------------------------------------------------
   IMPLICIT NONE
   INTEGER(KIND=2), DIMENSION (:,:,:), ALLOCATABLE ::  ibmask              ! nbasins x npiglo x npjglo
@@ -40,6 +42,7 @@ PROGRAM cdfmocsig
 
   INTEGER(KIND=4)                                 :: jbasin, jj, jk       ! dummy loop index
   INTEGER(KIND=4)                                 :: ji, jt, jbin         ! dummy loop index
+  INTEGER(KIND=4)                                 :: it                   ! time index for vvl
   INTEGER(KIND=4)                                 :: nbins                ! number of  density  bins
   INTEGER(KIND=4)                                 :: npglo, npatl, npinp  ! basins index (mnemonics)
   INTEGER(KIND=4)                                 :: npind, nppac         !  "      "
@@ -57,6 +60,16 @@ PROGRAM cdfmocsig
   INTEGER(KIND=4), DIMENSION(:),      ALLOCATABLE :: ipk, id_varout       ! output variable levels and id
   INTEGER(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: ibin                 ! remaping density in bin number
 
+  REAL(KIND=4), PARAMETER                         :: rp_spval=99999.      !
+  REAL(KIND=4)                                    :: pref=0.              ! depth reference for pot. density 
+  REAL(KIND=4)                                    :: sigmin               ! minimum density for bining
+  REAL(KIND=4)                                    :: sigstp               ! density step for bining
+  REAL(KIND=4)                                    :: zsps                 ! Salinity Missing value
+  REAL(KIND=4)                                    :: zspt                 ! Temperature Missing value
+  REAL(KIND=4)                                    :: zspv                 ! Merid. Vel.  Missing value
+  REAL(KIND=4), DIMENSION (:),        ALLOCATABLE :: sigma                ! density coordinate 
+  REAL(KIND=4), DIMENSION (:),        ALLOCATABLE :: e31d                 ! vertical level (full step)
+  REAL(KIND=4), DIMENSION (:),        ALLOCATABLE :: gdep                 ! depth of T layers ( full step)
   REAL(KIND=4), DIMENSION (:,:),      ALLOCATABLE :: e1v, gphiv           ! horizontal metrics, latitude
   REAL(KIND=4), DIMENSION (:,:),      ALLOCATABLE :: zt, zs               ! temperature, salinity
   REAL(KIND=4), DIMENSION (:,:),      ALLOCATABLE :: zv, zveiv            ! velocity and bolus velocity
@@ -65,26 +78,19 @@ PROGRAM cdfmocsig
   REAL(KIND=4), DIMENSION (:,:),      ALLOCATABLE :: rdumlat              ! latitude for i = north pole
   REAL(KIND=4), DIMENSION (:,:),      ALLOCATABLE :: zttmp                ! arrays to call sigmai and mask it
   REAL(KIND=4), DIMENSION (:,:),      ALLOCATABLE :: zarea                ! product e1v * e3v
-  REAL(KIND=4), DIMENSION (:),        ALLOCATABLE :: sigma                ! density coordinate 
-  REAL(KIND=4), DIMENSION (:),        ALLOCATABLE :: e31d                 ! vertical level (full step)
-  REAL(KIND=4), DIMENSION (:),        ALLOCATABLE :: gdep                 ! depth of T layers ( full step)
-  REAL(KIND=4), DIMENSION (:),        ALLOCATABLE :: tim                  ! time counter
-  REAL(KIND=4), PARAMETER                         :: rp_spval=99999.      !
-  REAL(KIND=4)                                    :: pref=0.              ! depth reference for pot. density 
-  REAL(KIND=4)                                    :: sigmin               ! minimum density for bining
-  REAL(KIND=4)                                    :: sigstp               ! density step for bining
-  REAL(KIND=4)                                    :: zsps                 ! Salinity Missing value
 
-  REAL(KIND=8), DIMENSION(:,:,:),     ALLOCATABLE :: dmoc                 ! nbasins x npjglo x npk
-  REAL(KIND=8), DIMENSION(:,:,:),     ALLOCATABLE :: depi                 ! Zonal mean of depths of isopycnal
-  REAL(KIND=8), DIMENSION(:,:,:),     ALLOCATABLE :: wdep                 ! count array
+  REAL(KIND=8), DIMENSION (:),        ALLOCATABLE :: dtim                 ! time counter
   REAL(KIND=8), DIMENSION(:,:),       ALLOCATABLE :: dens                 ! density
   REAL(KIND=8), DIMENSION(:,:),       ALLOCATABLE :: dmoc_tmp             ! temporary transport array
   REAL(KIND=8), DIMENSION(:,:),       ALLOCATABLE :: depi_tmp             ! temporary cumulated depth array
   REAL(KIND=8), DIMENSION(:,:),       ALLOCATABLE :: wdep_tmp             ! temporary count array
+  REAL(KIND=8), DIMENSION(:,:,:),     ALLOCATABLE :: dmoc                 ! nbasins x npjglo x npk
+  REAL(KIND=8), DIMENSION(:,:,:),     ALLOCATABLE :: depi                 ! Zonal mean of depths of isopycnal
+  REAL(KIND=8), DIMENSION(:,:,:),     ALLOCATABLE :: wdep                 ! count array
 
   CHARACTER(LEN=256)                              :: cf_vfil              ! meridional velocity file
   CHARACTER(LEN=256)                              :: cf_tfil              ! temperature/salinity file
+  CHARACTER(LEN=256)                              :: cf_sfil              ! salinity file (option)
   CHARACTER(LEN=256)                              :: cf_moc='mocsig.nc'   ! output file
   CHARACTER(LEN=255)                              :: cglobal              ! Global attribute
   CHARACTER(LEN=256)                              :: cldum                ! dummy char variable
@@ -104,44 +110,54 @@ PROGRAM cdfmocsig
 
   narg= iargc()
   IF ( narg == 0 ) THEN
-     PRINT *,' usage : cdfmocsig  V_file T_file depth_ref [-eiv] [-full]  ... '
-     PRINT *,'         ...  [-sigmin sigmin] [-sigstp sigstp] [-nbins nbins] [-isodep] [-v]'
+     PRINT *,' usage : cdfmocsig  -v V-file -t T-file -r REF-depth | -ntr [-eiv] [-full] ...'
+     PRINT *,'        ... [-sigmin sigmin] [-sigstp sigstp] [-nbins nbins] [-isodep] ...'
+     PRINT *,'        ... [-s S-file ] [-o OUT-file] [-vvl] [-verbose]'
+     PRINT *,'      '
      PRINT *,'     PURPOSE : '
-     PRINT *,'       Computes the MOC in density-latitude coordinates. The global value'
-     PRINT *,'       is always computed. Values for oceanic sub-basins are calculated'
-     PRINT *,'       if the file ', TRIM(cn_fbasins), ' is provided.'
-     PRINT *,'       Last arguments is the reference depth for potential density, in m.'
-     PRINT *,'       Actually only 0 1000 or 2000 are available with standard values for'
-     PRINT *,'       density bins. If you specify another reference depth, you must also'
-     PRINT *,'       specify the minimum density, the bin size and the number of bins,'
-     PRINT *,'       with the options -sigmin, -sigstp, -nbins'
+     PRINT *,'       Compute the MOC in density-latitude coordinates. The global value is '
+     PRINT *,'       always computed. Values for oceanic sub-basins are calculated if the '
+     PRINT *,'       ', TRIM(cn_fbasins), ' file is provided.'
+     PRINT *,'      '
+     PRINT *,'       The reference depth for potential density is given with ''-D'' option.'
+     PRINT *,'       Density ranges and number of bins to use are pre-defined only for three'
+     PRINT *,'       reference depth (0, 1000 and 2000 m). For other reference depth, the '
+     PRINT *,'       density binning must be specified using the relevant options for setting'
+     PRINT *,'       the minimum density, the density step and the number of bins to use.'
      PRINT *,'      '
      PRINT *,'     ARGUMENTS :'
-     PRINT *,'        V_file  : Netcdf gridV file'
-     PRINT *,'        T_file  : Netcdf gridT file'
-     PRINT *,'        depth_ref : reference depth for density '
-     PRINT *,'               for depth values of 0 1000 or 2000, pre-defined limits for'
-     PRINT *,'               minimum density, number of density bins and width of density'
-     PRINT *,'               bins are provided. For other reference depth, you must use'
-     PRINT *,'               -sigmin, -sigstp and -nbins options (see below).'
-     PRINT *,'               Keyword ''ntr'' can also be used in place of depth_ref in '
-     PRINT *,'               order to use neutral density (no default bin defined so far).'
+     PRINT *,'        -v V-file  : Netcdf gridV file.'
+     PRINT *,'        -t T-file  : Netcdf gridT file with temperature and salinity.'
+     PRINT *,'             If salinity not in T-file use -s option.'
+     PRINT *,'        -r ref-depth : reference depth for density. '
+     PRINT *,'            For depth values of 0 1000 or 2000 m, pre-defined limits for '
+     PRINT *,'            minimum density, number of density bins and width of density '
+     PRINT *,'            bins are provided. For other reference depth, you must use the'
+     PRINT *,'            options ''-sigmin'', ''-sigstp'' and ''-nbins'' (see below).'
+     PRINT *,'        or '
+     PRINT *,'        -ntr : uses neutral density (no default bin defined so far), no ''-r'''
      PRINT *,'      '
      PRINT *,'     OPTIONS :'
-     PRINT *,'       [-eiv ] : takes into account VEIV Meridional eddy induced velocity'
+     PRINT *,'       [-s S-file   ] : Specify salinity file if not T-file.'
+     PRINT *,'       [-eiv ] : takes into account VEIV Meridional eddy induced velocity.'
      PRINT *,'                 -> To be used only if Gent and McWilliams parameterization '
-     PRINT *,'                    has been used '
-     PRINT *,'       [ -full ] : Works with full step instead of standard partial steps'
-     PRINT *,'       [ -sigmin ] : Specify minimum of density for bining'
-     PRINT *,'       [ -sigstp ] : Specify density step for bining'
-     PRINT *,'       [ -nbins ]  : Specify the number of density bins you want'
-     PRINT *,'       [ -isodep]  : Compute the zonal mean of isopycnal depths used for mocsig'
-     PRINT *,'       [ -v  ]     : Verbose option for more info during execution'
+     PRINT *,'                    has been used. '
+     PRINT *,'       [-full       ] : Works with full step instead of standard partial steps.'
+     PRINT *,'       [-sigmin     ] : Specify minimum of density for bining.'
+     PRINT *,'       [-sigstp     ] : Specify density step for bining.'
+     PRINT *,'       [-nbins      ] : Specify the number of density bins you want.'
+     PRINT *,'       [-isodep     ] : Compute the zonal mean of isopycnal depths used for '
+     PRINT *,'                        mocsig.'
+     PRINT *,'       [-o OUT-file ] : Specify output file name instead of ', TRIM(cf_moc)
+     PRINT *,'       [-vvl        ] : Use time-varying vertical metrics.'
+     PRINT *,'       [-verbose    ] : Verbose option for more info during execution.'
      PRINT *,'      '
      PRINT *,'     REQUIRED FILES :'
      PRINT *,'        Files ', TRIM(cn_fzgr),', ',TRIM(cn_fhgr),', ', TRIM(cn_fmsk)
      PRINT *,'        File ', TRIM(cn_fbasins),' is optional [sub basins masks]'
      PRINT *,'      '
+     PRINT *,'     OPENMP SUPPORT : yes '
+     PRINT *,'      '
      PRINT *,'     OUTPUT : '
      PRINT *,'       netcdf file : ', TRIM(cf_moc) 
      PRINT *,'       variables ',TRIM( cn_zomsfglo),' : Global ocean '
@@ -149,64 +165,66 @@ PROGRAM cdfmocsig
      PRINT *,'       variables ',TRIM( cn_zomsfinp),' : Indo Pacific '
      PRINT *,'       variables ',TRIM( cn_zomsfind),' : Indian Ocean alone'
      PRINT *,'       variables ',TRIM( cn_zomsfpac),' : Pacific Ocean alone'
-     PRINT *,'       If file ',TRIM(cn_fbasins),' is not present, ',TRIM(cn_fmsk),' file'
-     PRINT *,'       is used and only ',TRIM( cn_zomsfglo),' is produced.'
+     PRINT *,'       If file ',TRIM(cn_fbasins),' is not present, ',TRIM(cn_fmsk),' file is used and'
+     PRINT *,'       only ',TRIM( cn_zomsfglo),' is produced.'
      PRINT *,'       If option -isodep is used, each MOC variable is complemented by a iso'
      PRINT *,'       variable, giving the zonal mean of ispycnal depth (e.g.',TRIM(cn_zoisoglo),').'
-     STOP
+     PRINT *,'      '
+     PRINT *,'     SEE ALSO :'
+     PRINT *,'       cdfmoc '
+     PRINT *,'      '
+     STOP 
   ENDIF
 
   cglobal = 'Partial step computation'
   lbin=(/.TRUE.,.TRUE.,.TRUE./)
-  ijarg = 1 ; ii = 0
+  ijarg = 1 ; ii = 0 ! ii is used to count mandatory arguments
+  cf_sfil = 'none'
   DO WHILE ( ijarg <= narg )
      CALL getarg (ijarg, cldum) ; ijarg=ijarg+1
      SELECT CASE ( cldum )
-     CASE ('-full')
-        lfull   = .TRUE.
-        cglobal = 'Full step computation'
-     CASE ('-eiv')
-        leiv    = .TRUE.
-     CASE ('-sigmin')
-        CALL getarg (ijarg, cldum) ; ijarg=ijarg+1 ; READ(cldum,*) sigmin
-        lbin(1) = .FALSE.
-     CASE ('-nbins')
-        CALL getarg (ijarg, cldum) ; ijarg=ijarg+1 ; READ(cldum,*) nbins
-        lbin(2) = .FALSE.
-     CASE ('-sigstp')
-        CALL getarg (ijarg, cldum) ; ijarg=ijarg+1 ; READ(cldum,*) sigstp
-        lbin(3) = .FALSE.
-     CASE ('-isodep')
-        lisodep = .TRUE.
-     CASE ('-v')
-        lprint = .TRUE.
-     CASE DEFAULT
-        ii=ii+1
-        SELECT CASE (ii)
-        CASE ( 1 ) ; cf_vfil = cldum
-        CASE ( 2 ) ; cf_tfil = cldum
-        CASE ( 3 ) 
-             SELECT CASE ( cldum )
-             CASE ( 'ntr', 'NTR', 'Ntr' ) ; lntr = .TRUE.
-             CASE DEFAULT                 ; READ(cldum,*) pref
-             END SELECT
-        CASE DEFAULT
-           PRINT *, 'ERROR : Too many arguments ...'
-           STOP 99
-        END SELECT
+     CASE ( '-v'     ) ;  CALL getarg (ijarg, cf_vfil) ; ijarg=ijarg+1 ; ii=ii+1
+     CASE ( '-t'     ) ;  CALL getarg (ijarg, cf_tfil) ; ijarg=ijarg+1 ; ii=ii+1
+     CASE ( '-r'     ) ;  CALL getarg (ijarg, cldum  ) ; ijarg=ijarg+1 ; ii=ii+1 ; READ(cldum,*) pref
+     CASE ( '-ntr'   ) ;  lntr = .TRUE. ; ii=ii+1
+        ! options
+     CASE ( '-s'     ) ;  CALL getarg (ijarg, cf_sfil) ; ijarg=ijarg+1 
+     CASE ('-full'   ) ; lfull   = .TRUE. ; cglobal = 'Full step computation'
+     CASE ('-eiv'    ) ; leiv    = .TRUE.
+     CASE ('-sigmin' ) ; CALL getarg (ijarg, cldum ) ; ijarg=ijarg+1 ; READ(cldum,*) sigmin ; lbin(1) = .FALSE.
+     CASE ('-nbins'  ) ; CALL getarg (ijarg, cldum ) ; ijarg=ijarg+1 ; READ(cldum,*) nbins  ; lbin(2) = .FALSE.
+     CASE ('-sigstp' ) ; CALL getarg (ijarg, cldum ) ; ijarg=ijarg+1 ; READ(cldum,*) sigstp ; lbin(3) = .FALSE.
+     CASE ('-o'      ) ; CALL getarg (ijarg, cf_moc) ; ijarg=ijarg+1 
+     CASE ('-vvl'    ) ; lg_vvl  = .TRUE.
+     CASE ('-isodep' ) ; lisodep = .TRUE.
+     CASE ('-verbose') ; lprint  = .TRUE.
+     CASE DEFAULT     ; PRINT *,' ERROR : ',TRIM(cldum), ' : unknown option.'  ; STOP 99
      END SELECT
   END DO
 
+  IF ( ii /= 3  ) THEN ; PRINT *,' ERROR : mandatory arguments missing, see usage please !'  ; STOP 99
+  ENDIF
+
+  IF ( cf_sfil == 'none' ) cf_sfil=cf_tfil
+
   ! check file existence
   lchk = lchk .OR. chkfile ( cn_fhgr )
   lchk = lchk .OR. chkfile ( cn_fzgr )
   lchk = lchk .OR. chkfile ( cn_fmsk )
   lchk = lchk .OR. chkfile ( cf_vfil )
   lchk = lchk .OR. chkfile ( cf_tfil )
+  lchk = lchk .OR. chkfile ( cf_sfil )
   IF ( lchk ) STOP 99  ! missing file(s)
 
   ! Look for salinity spval
-  zsps = getspval(cf_tfil, cn_vosaline)
+  zsps = getspval(cf_sfil, cn_vosaline)
+  zspt = getspval(cf_tfil, cn_votemper)
+  zspv = getspval(cf_vfil, cn_vomecrty)
+
+  IF ( lg_vvl )  THEN
+     cn_fe3v = cf_vfil
+     cn_ve3v = cn_ve3vvvl
+  ENDIF
 
   ! re-use lchk for binning control : TRUE if no particular binning specified
   lchk = lbin(1) .OR. lbin(2) .OR. lbin(3) 
@@ -227,16 +245,12 @@ PROGRAM cdfmocsig
   !  Detects newmaskglo file
   lbas = .NOT. chkfile (cn_fbasins )
 
-  IF (lbas) THEN
-     nbasins = 5
-  ELSE
-     nbasins = 1
+  IF (lbas) THEN ; nbasins = 5
+  ELSE           ; nbasins = 1
   ENDIF
 
-  IF ( lisodep ) THEN
-     nvaro = 2 * nbasins
-  ELSE
-     nvaro =     nbasins
+  IF ( lisodep ) THEN ; nvaro = 2 * nbasins
+  ELSE                ; nvaro =     nbasins
   ENDIF
 
   ALLOCATE ( stypvar(nvaro), ipk(nvaro), id_varout(nvaro) )
@@ -244,36 +258,34 @@ PROGRAM cdfmocsig
   IF ( lchk )  THEN  ! use default bins definition according to pref 
      ! Define density parameters
      IF ( lntr) THEN   ! to be confirmed ( note that sigmantr returns values > 1000 kg/m3)
-         nbins  = 52
-         sigmin = 1023.
-         sigstp = 0.1
+        nbins  = 52
+        sigmin = 1023.
+        sigstp = 0.1
      ELSE
-       SELECT CASE ( INT(pref) )
-       CASE ( 0 )
-          nbins  = 52
-          sigmin = 23.
-          sigstp = 0.1
-       CASE ( 1000 )
-          nbins  = 88
-          sigmin = 24.
-          sigstp = 0.1
-       CASE ( 2000)
-          nbins  = 158
-          sigmin = 30.
-          sigstp = 0.05
-       CASE DEFAULT
-          PRINT *,' This value of depth_ref (',pref,') is not implemented as standard'
-          PRINT *,' You must use the -sigmin, -sigstp and -nbins options to precise'
-          PRINT *,' the density bining you want to use.'
-          STOP 99
-       END SELECT
+        SELECT CASE ( INT(pref) )
+        CASE ( 0 )
+           nbins  = 52
+           sigmin = 23.
+           sigstp = 0.1
+        CASE ( 1000 )
+           nbins  = 88
+           sigmin = 24.
+           sigstp = 0.1
+        CASE ( 2000)
+           nbins  = 158
+           sigmin = 30.
+           sigstp = 0.05
+        CASE DEFAULT
+           PRINT *,' This value of depth_ref (',pref,') is not implemented as standard'
+           PRINT *,' You must use the -sigmin, -sigstp and -nbins options to precise'
+           PRINT *,' the density bining you want to use.'
+           STOP 99
+        END SELECT
      ENDIF
   ENDIF
 
-  IF (lntr ) THEN
-     PRINT '(a,f6.1,a)',           '  For Neutral density MOC'
-  ELSE
-     PRINT '(a,f6.1,a)',           '  For reference depth ', pref, ' m, '
+  IF (lntr ) THEN  ; PRINT '(a       )',           '  For Neutral density MOC'
+  ELSE             ; PRINT '(a,f6.1,a)',           '  For reference depth ', pref, ' m, '
   ENDIF
   PRINT '(a,f5.2,a,f5.2,a,i3)', '  You are using -sigmin ', sigmin,' -sigstp ', sigstp,' -nbins ', nbins
 
@@ -295,11 +307,11 @@ PROGRAM cdfmocsig
   ALLOCATE ( rdumlon(1,npjglo) , rdumlat(1,npjglo))
   ALLOCATE ( dens(npiglo,npjglo))
   ALLOCATE ( itmask(npiglo,npjglo), zttmp(npiglo,npjglo))
-  ALLOCATE ( tim(npt), e31d(npk)  )
+  ALLOCATE ( dtim(npt), e31d(npk)  )
 
   IF ( lisodep) THEN 
-                ALLOCATE ( depi(nvaro, nbins, npjglo), gdep(npk))
-                ALLOCATE ( wdep(nvaro, nbins, npjglo)           )
+     ALLOCATE ( depi(nvaro, nbins, npjglo), gdep(npk))
+     ALLOCATE ( wdep(nvaro, nbins, npjglo)           )
   ENDIF
   IF ( leiv   ) ALLOCATE ( zveiv (npiglo,npjglo))
 
@@ -307,7 +319,7 @@ PROGRAM cdfmocsig
 
   IF ( lfull  ) e31d(:) = getvare3(cn_fzgr, cn_ve3t1d,  npk )
   IF ( lisodep) gdep(:) = -getvare3(cn_fzgr, cn_gdept, npk )  ! take negative value
-                                                              ! to be compliant with zonal mean
+  ! to be compliant with zonal mean
 
   IF ( npjglo > 1 ) THEN 
      gphiv(:,:)   = getvar(cn_fhgr, cn_gphiv, 1, npiglo, npjglo)
@@ -322,7 +334,7 @@ PROGRAM cdfmocsig
   !global   ; Atlantic  ; Indo-Pacif ; Indian  ; Pacif
   npglo= 1  ; npatl=2   ;  npinp=3   ; npind=4 ; nppac=5
 
-  CALL CreateOutputFiles
+  CALL CreateOutputFile
 
   ! reading the masks
   ibmask(npglo,:,:) = getvar(cn_fmsk, cn_vmask, 1, npiglo, npjglo)
@@ -340,10 +352,12 @@ PROGRAM cdfmocsig
   ENDIF
 
   DO jt=1, npt
+     IF ( lg_vvl ) THEN ; it=jt
+     ELSE               ; it=1
+     ENDIF
      ! initialize moc to 0
      dmoc(:,:,:) = 0.d0 
-     IF ( lisodep ) THEN 
-       depi(:,:,:) = 0.d0 ; wdep(:,:,:) = 0.d0
+     IF ( lisodep ) THEN ; depi(:,:,:) = 0.d0 ; wdep(:,:,:) = 0.d0
      ENDIF
 
      DO jk=1,npk-1
@@ -351,29 +365,28 @@ PROGRAM cdfmocsig
         IF (lprint) PRINT *,' working at depth ',jk
         ! Get velocities v at jj
         zv(:,:) = getvar(cf_vfil, cn_vomecrty, jk, npiglo, npjglo, ktime = jt)
+        WHERE( zv == zspv ) zv = 0.
         IF ( leiv ) THEN
            zveiv(:,:) = getvar(cf_vfil, cn_vomeeivv, jk, npiglo,npjglo, ktime = jt)
            zv(:,:)    = zv(:,:) + zveiv(:,:)
         END IF
         ! JMM remark : should be more correct to use t and s a V point ?
         zt(:,:) = getvar(cf_tfil, cn_votemper, jk, npiglo, npjglo, ktime = jt)
-        zs(:,:) = getvar(cf_tfil, cn_vosaline, jk, npiglo, npjglo, ktime = jt)
+        zs(:,:) = getvar(cf_sfil, cn_vosaline, jk, npiglo, npjglo, ktime = jt)
+        WHERE( zt == zspt ) zt = 0.
+        WHERE( zs == zsps ) zs = 0.
 
         ! get e3v at latitude jj
-        IF ( lfull ) THEN
-           e3v(:,:) = e31d(jk)
-        ELSE
-           e3v(:,:) = getvar(cn_fzgr, 'e3v_ps', jk, npiglo, npjglo, ldiom=.TRUE.)
+        IF ( lfull ) THEN ; e3v(:,:) = e31d(jk)
+        ELSE              ; e3v(:,:) = getvar(cn_fe3v, cn_ve3v, jk, npiglo, npjglo, ktime=it, ldiom=.NOT.lg_vvl )
         ENDIF
         zarea(:,:) = e1v(:,:) * e3v(:,:)
         !
         !  finds density 
         itmask =  1
         WHERE ( zs == zsps ) itmask = 0
-        IF ( lntr ) THEN
-           dens  = sigmantr(zt, zs,     npiglo, npjglo)
-        ELSE
-           dens  = sigmai(zt, zs, pref, npiglo, npjglo)
+        IF ( lntr ) THEN ; dens  = sigmantr(zt, zs,       npiglo, npjglo)
+        ELSE             ; dens  = sigmai  (zt, zs, pref, npiglo, npjglo)
         ENDIF
 
         zttmp = dens* itmask ! convert to single precision 
@@ -382,13 +395,11 @@ PROGRAM cdfmocsig
         ibin(:,:) = MAX( ibin(:,:), 1    )
         ibin(:,:) = MIN( ibin(:,:), nbins)
 
-        IF ( npjglo > 1 ) THEN
-           ij1 = 2 ; ij2 = npjglo-1
-        ELSE
-           ij1 = 1 ; ij2 = 1 ! input file has only one j ( case of extracted broken lines) 
+        IF ( npjglo > 1 ) THEN ; ij1 = 2 ; ij2 = npjglo-1
+        ELSE                   ; ij1 = 1 ; ij2 = 1        ! input file has only one j ( case of extracted broken lines) 
         ENDIF
-!$OMP PARALLEL  PRIVATE(dmoc_tmp,depi_tmp,wdep_tmp, ib)
-                       ALLOCATE ( dmoc_tmp(nbins,npiglo) )
+        !$OMP PARALLEL  PRIVATE(dmoc_tmp,depi_tmp,wdep_tmp, ib)
+        ALLOCATE ( dmoc_tmp(nbins,npiglo) )
         IF ( lisodep ) ALLOCATE ( depi_tmp(nbins,npiglo) )
         IF ( lisodep ) ALLOCATE ( wdep_tmp(nbins,npiglo) )
         IF ( lprint ) PRINT *, ' Entering main J loop '
@@ -412,8 +423,8 @@ PROGRAM cdfmocsig
            ENDIF
            ! integrates 'zonally' (along i-coordinate) 
            ! add to dmoc the contributions from level jk  at all densities jbin
-           
-!          IF ( lprint ) PRINT *, ' Entering main bin loop ', jj,ij2
+
+           !          IF ( lprint ) PRINT *, ' Entering main bin loop ', jj,ij2
            DO jbasin= 1, nbasins
               DO jbin =1,nbins  
                  DO ji=2,npiglo-1
@@ -424,29 +435,29 @@ PROGRAM cdfmocsig
            END DO
 
            IF ( lisodep) THEN
-           DO jbasin= 1, nbasins
-              DO jbin =1,nbins
-                 DO ji=2,npiglo-1
+              DO jbasin= 1, nbasins
+                 DO jbin =1,nbins
+                    DO ji=2,npiglo-1
                        depi(jbasin,jbin,jj)=depi(jbasin,jbin,jj) + depi_tmp(jbin,ji) * ibmask(jbasin,ji,jj)
                        wdep(jbasin,jbin,jj)=wdep(jbasin,jbin,jj) + wdep_tmp(jbin,ji) * ibmask(jbasin,ji,jj)
-                 ENDDO
+                    ENDDO
+                 END DO
               END DO
-           END DO
 
            ENDIF
         END DO  ! end of loop on latitude for filling dmoc
         !$OMP END DO
-                       DEALLOCATE (dmoc_tmp)
+        DEALLOCATE (dmoc_tmp)
         IF ( lisodep ) DEALLOCATE (depi_tmp)
         IF ( lisodep ) DEALLOCATE (wdep_tmp)
-!$OMP END PARALLEL
+        !$OMP END PARALLEL
      END DO     ! end of loop on depths for calculating transports     
 
      IF ( lisodep ) THEN
         WHERE ( wdep(:,:,:) /= 0.d0 ) 
-         depi(:,:,:) = depi(:,:,:) / wdep (:,:,:)
+           depi(:,:,:) = depi(:,:,:) / wdep (:,:,:)
         ELSEWHERE
-         depi(:,:,:) = rp_spval
+           depi(:,:,:) = rp_spval
         END WHERE
      ENDIF
 
@@ -459,7 +470,7 @@ PROGRAM cdfmocsig
      ! netcdf output  
      DO jbasin = 1, nbasins
         DO jbin = 1, nbins
-                         ierr = putvar (ncout, id_varout(jbasin        ), REAL(dmoc(jbasin,jbin,:)), jbin, 1, npjglo, ktime = jt)
+           ierr = putvar (ncout, id_varout(jbasin        ), REAL(dmoc(jbasin,jbin,:)), jbin, 1, npjglo, ktime = jt)
            IF (lisodep ) ierr = putvar (ncout, id_varout(jbasin+nbasins), REAL(depi(jbasin,jbin,:)), jbin, 1, npjglo, ktime = jt)
         END DO
      END DO
@@ -468,10 +479,11 @@ PROGRAM cdfmocsig
 
   ierr = closeout(ncout)
 
-  CONTAINS
-  SUBROUTINE CreateOutputFiles
+CONTAINS
+
+  SUBROUTINE CreateOutputFile
     !!---------------------------------------------------------------------
-    !!                  ***  ROUTINE CreateOutputFiles ***
+    !!                  ***  ROUTINE CreateOutputFile ***
     !!
     !! ** Purpose :  Initialize and create output files 
     !!
@@ -479,89 +491,89 @@ PROGRAM cdfmocsig
     !!
     !!----------------------------------------------------------------------
 
-  ! Common to all variables :
-  stypvar%cunits            = 'Sverdrup'
-  stypvar%rmissing_value    = rp_spval
-  stypvar%valid_min         = -1000.
-  stypvar%valid_max         =  1000.
-  stypvar%scale_factor      = 1.
-  stypvar%add_offset        = 0.
-  stypvar%savelog10         = 0.
-  stypvar%conline_operation = 'N/A'
-  stypvar%caxis             = 'TZY'
+    ! Common to all variables :
+    stypvar%cunits            = 'Sverdrup'
+    stypvar%rmissing_value    = rp_spval
+    stypvar%valid_min         = -1000.
+    stypvar%valid_max         =  1000.
+    stypvar%scale_factor      = 1.
+    stypvar%add_offset        = 0.
+    stypvar%savelog10         = 0.
+    stypvar%conline_operation = 'N/A'
+    stypvar%caxis             = 'TZY'
 
-  ipk(:) = nbins
+    ipk(:) = nbins
 
-  ! Global basin
-  stypvar(npglo)%cname       = cn_zomsfglo
-  stypvar(npglo)%clong_name  = 'Meridional_Overt.Cell_Global'
-  stypvar(npglo)%cshort_name = cn_zomsfglo
+    ! Global basin
+    stypvar(npglo)%cname       = cn_zomsfglo
+    stypvar(npglo)%clong_name  = 'Meridional_Overt.Cell_Global'
+    stypvar(npglo)%cshort_name = cn_zomsfglo
 
-  IF (lbas) THEN
-     stypvar(npatl)%cname       = cn_zomsfatl
-     stypvar(npatl)%clong_name  = 'Meridional_Overt.Cell_Atlantic'
-     stypvar(npatl)%cshort_name = cn_zomsfatl
+    IF (lbas) THEN
+       stypvar(npatl)%cname       = cn_zomsfatl
+       stypvar(npatl)%clong_name  = 'Meridional_Overt.Cell_Atlantic'
+       stypvar(npatl)%cshort_name = cn_zomsfatl
 
-     stypvar(npinp)%cname       = cn_zomsfinp
-     stypvar(npinp)%clong_name  = 'Meridional_Overt.Cell_IndoPacif'
-     stypvar(npinp)%cshort_name = cn_zomsfinp
+       stypvar(npinp)%cname       = cn_zomsfinp
+       stypvar(npinp)%clong_name  = 'Meridional_Overt.Cell_IndoPacif'
+       stypvar(npinp)%cshort_name = cn_zomsfinp
 
-     stypvar(npind)%cname       = cn_zomsfind
-     stypvar(npind)%clong_name  = 'Meridional_Overt.Cell_Indian'
-     stypvar(npind)%cshort_name = cn_zomsfind
+       stypvar(npind)%cname       = cn_zomsfind
+       stypvar(npind)%clong_name  = 'Meridional_Overt.Cell_Indian'
+       stypvar(npind)%cshort_name = cn_zomsfind
 
-     stypvar(nppac)%cname       = cn_zomsfpac
-     stypvar(nppac)%clong_name  = 'Meridional_Overt.Cell_pacif'
-     stypvar(nppac)%cshort_name = cn_zomsfpac
-  ENDIF
+       stypvar(nppac)%cname       = cn_zomsfpac
+       stypvar(nppac)%clong_name  = 'Meridional_Overt.Cell_pacif'
+       stypvar(nppac)%cshort_name = cn_zomsfpac
+    ENDIF
 
-  IF ( lisodep ) THEN
-     ! Global basin
-     stypvar(npglo+nbasins)%cunits      = 'm'
-     stypvar(npglo+nbasins)%cname       = cn_zoisoglo
-     stypvar(npglo+nbasins)%clong_name  = 'Zonal_mean_isopycnal_depth_Global'
-     stypvar(npglo+nbasins)%cshort_name = cn_zoisoglo
-     stypvar(npglo+nbasins)%valid_min   = 0.
-     stypvar(npglo+nbasins)%valid_max   = 8000.
-     IF ( lbas ) THEN
-        stypvar(npatl+nbasins)%cunits      = 'm'
-        stypvar(npatl+nbasins)%cname       = cn_zoisoatl
-        stypvar(npatl+nbasins)%clong_name  = 'Zonal_mean_isopycnal_depth_Atlantic'
-        stypvar(npatl+nbasins)%cshort_name = cn_zoisoatl
-        stypvar(npatl+nbasins)%valid_min   = 0.
-        stypvar(npatl+nbasins)%valid_max   = 8000.
+    IF ( lisodep ) THEN
+       ! Global basin
+       stypvar(npglo+nbasins)%cunits      = 'm'
+       stypvar(npglo+nbasins)%cname       = cn_zoisoglo
+       stypvar(npglo+nbasins)%clong_name  = 'Zonal_mean_isopycnal_depth_Global'
+       stypvar(npglo+nbasins)%cshort_name = cn_zoisoglo
+       stypvar(npglo+nbasins)%valid_min   = 0.
+       stypvar(npglo+nbasins)%valid_max   = 8000.
+       IF ( lbas ) THEN
+          stypvar(npatl+nbasins)%cunits      = 'm'
+          stypvar(npatl+nbasins)%cname       = cn_zoisoatl
+          stypvar(npatl+nbasins)%clong_name  = 'Zonal_mean_isopycnal_depth_Atlantic'
+          stypvar(npatl+nbasins)%cshort_name = cn_zoisoatl
+          stypvar(npatl+nbasins)%valid_min   = 0.
+          stypvar(npatl+nbasins)%valid_max   = 8000.
 
-        stypvar(npinp+nbasins)%cunits      = 'm'
-        stypvar(npinp+nbasins)%cname       = cn_zoisoinp
-        stypvar(npinp+nbasins)%clong_name  = 'Zonal_mean_isopycnal_depth_IndoPacif'
-        stypvar(npinp+nbasins)%cshort_name = cn_zoisoinp
-        stypvar(npinp+nbasins)%valid_min   = 0.
-        stypvar(npinp+nbasins)%valid_max   = 8000.
+          stypvar(npinp+nbasins)%cunits      = 'm'
+          stypvar(npinp+nbasins)%cname       = cn_zoisoinp
+          stypvar(npinp+nbasins)%clong_name  = 'Zonal_mean_isopycnal_depth_IndoPacif'
+          stypvar(npinp+nbasins)%cshort_name = cn_zoisoinp
+          stypvar(npinp+nbasins)%valid_min   = 0.
+          stypvar(npinp+nbasins)%valid_max   = 8000.
 
-        stypvar(npind+nbasins)%cunits      = 'm'
-        stypvar(npind+nbasins)%cname       = cn_zoisoind
-        stypvar(npind+nbasins)%clong_name  = 'Zonal_mean_isopycnal_depth_Indian'
-        stypvar(npind+nbasins)%cshort_name = cn_zoisoind
-        stypvar(npind+nbasins)%valid_min   = 0.
-        stypvar(npind+nbasins)%valid_max   = 8000.
+          stypvar(npind+nbasins)%cunits      = 'm'
+          stypvar(npind+nbasins)%cname       = cn_zoisoind
+          stypvar(npind+nbasins)%clong_name  = 'Zonal_mean_isopycnal_depth_Indian'
+          stypvar(npind+nbasins)%cshort_name = cn_zoisoind
+          stypvar(npind+nbasins)%valid_min   = 0.
+          stypvar(npind+nbasins)%valid_max   = 8000.
 
-        stypvar(nppac+nbasins)%cunits      = 'm'
-        stypvar(nppac+nbasins)%cname       = cn_zoisopac
-        stypvar(nppac+nbasins)%clong_name  = 'Zonal_mean_isopycnal_depth_pacif'
-        stypvar(nppac+nbasins)%cshort_name = cn_zoisopac
-        stypvar(nppac+nbasins)%valid_min   = 0.
-        stypvar(nppac+nbasins)%valid_max   = 8000.
-     ENDIF
-  ENDIF
+          stypvar(nppac+nbasins)%cunits      = 'm'
+          stypvar(nppac+nbasins)%cname       = cn_zoisopac
+          stypvar(nppac+nbasins)%clong_name  = 'Zonal_mean_isopycnal_depth_pacif'
+          stypvar(nppac+nbasins)%cshort_name = cn_zoisopac
+          stypvar(nppac+nbasins)%valid_min   = 0.
+          stypvar(nppac+nbasins)%valid_max   = 8000.
+       ENDIF
+    ENDIF
 
-  ncout = create      (cf_moc, 'none', 1,      npjglo, nbins,  cdep='sigma')
-  ierr  = createvar   (ncout,  stypvar, nvaro, ipk ,id_varout, cdglobal=cglobal)
-  ierr  = putheadervar(ncout,  cf_vfil, 1,     npjglo, nbins,  pnavlon=rdumlon, pnavlat=rdumlat, pdep=sigma)
+    ncout = create      (cf_moc, 'none', 1,      npjglo, nbins,  cdep='sigma')
+    ierr  = createvar   (ncout,  stypvar, nvaro, ipk ,id_varout, cdglobal=cglobal)
+    ierr  = putheadervar(ncout,  cf_vfil, 1,     npjglo, nbins,  pnavlon=rdumlon, pnavlat=rdumlat, pdep=sigma)
 
-  tim  = getvar1d(cf_vfil, cn_vtimec, npt     )
-  ierr = putvar1d(ncout,   tim,       npt, 'T')
+    dtim = getvar1d(cf_vfil, cn_vtimec, npt     )
+    ierr = putvar1d(ncout,  dtim,       npt, 'T')
 
-  END SUBROUTINE CreateOutputFiles
+  END SUBROUTINE CreateOutputFile
 
 END PROGRAM cdfmocsig
    
