@@ -14,15 +14,17 @@ PROGRAM cdfmxl
   !!           3.0  : 01/2011  : J.M. Molines : Doctor norm + Lic.
   !!           3.0  : 07/2012  : F. Hernandez: Optional S-FILE input
   !!           3.0  : 07/2012  : F. Hernandez: Add new MLD computation for GSOP/GODAE
+  !!         : 4.0  : 03/2017  : J.M. Molines  
   !!----------------------------------------------------------------------
   USE cdfio
   USE eos
   USE modcdfnames
   !!----------------------------------------------------------------------
-  !! CDFTOOLS_3.0 , MEOM 2011
+  !! CDFTOOLS_4.0 , MEOM 2017 
   !! $Id$
-  !! Copyright (c) 2011, J.-M. Molines
+  !! Copyright (c) 2017, J.-M. Molines 
   !! Software governed by the CeCILL licence (Licence/CDFTOOLSCeCILL.txt)
+  !! @class mixed_layer
   !!----------------------------------------------------------------------
   IMPLICIT NONE
 
@@ -47,11 +49,11 @@ PROGRAM cdfmxl
 
   REAL(KIND=4)                                 :: rmisval=32767. ! Missing value of Mercator fields 
   REAL(KIND=4)                                 :: rr1,rr2        ! Coef for T(z=10m) interp. 
-  REAL(KIND=4)                                 :: rho_c1=0.01    ! 1rst density criterium
-  REAL(KIND=4)                                 :: rho_c2=0.03    ! 2nd density criterium
-  REAL(KIND=4)                                 :: rho_c3=0.125   ! 3rd density criterium 
-  REAL(KIND=4)                                 :: temp_c=-0.2    ! temperature criterium
-  REAL(KIND=4)                                 :: temp_c2=-0.5   ! 2nd temperature criterium 
+  REAL(KIND=4)                                 :: rho_c1=0.01    ! 1rst density criterion
+  REAL(KIND=4)                                 :: rho_c2=0.03    ! 2nd density criterion
+  REAL(KIND=4)                                 :: rho_c3=0.125   ! 3rd density criterion 
+  REAL(KIND=4)                                 :: temp_c=-0.2    ! temperature criterion
+  REAL(KIND=4)                                 :: temp_c2=-0.5   ! 2nd temperature criterion 
   REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: rtem           ! temperature
   REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: rtem10         ! 10m depth temperature 
   REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: rsal           ! salinity
@@ -63,18 +65,19 @@ PROGRAM cdfmxl
   REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: tmask_surf     ! surface tmask
   REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: tmask_10       ! 10m-depth tmask 
   REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: tmask          ! level tmask
-  REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: hmlp1          ! mxl depth based on density criterium 1
-  REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: hmlp2          ! mxl depth based on density criterium 2
-  REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: hmlp3          ! mxl depth based on density criterium 2 and 10m
-  REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: hmlp4          ! mxl depth based on density criterium 3 and 10m
-  REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: hmlt           ! mxl depth based on temperature criterium
-  REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: hmlt2          ! mxl depth based on temperature criterium and 10m
-  REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: hmlt3          ! mxl depth based on temperature criterium 2 and 10m
+  REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: hmlp1          ! mxl depth based on density criterion 1
+  REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: hmlp2          ! mxl depth based on density criterion 2
+  REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: hmlp3          ! mxl depth based on density criterion 2 and 10m
+  REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: hmlp4          ! mxl depth based on density criterion 3 and 10m
+  REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: hmlt           ! mxl depth based on temperature criterion
+  REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: hmlt2          ! mxl depth based on temperature criterion and 10m
+  REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: hmlt3          ! mxl depth based on temperature criterion 2 and 10m
   REAL(KIND=4), DIMENSION(:),      ALLOCATABLE :: gdepw          ! depth of w levels
   REAL(KIND=4), DIMENSION(:),      ALLOCATABLE :: gdept          ! depth of T levels 
-  REAL(KIND=4), DIMENSION(:),      ALLOCATABLE :: tim            ! time counter
   REAL(KIND=4), DIMENSION(1)                   :: rdep           ! dummy depth for output
 
+  REAL(KIND=8), DIMENSION(:),      ALLOCATABLE :: dtim           ! time counter
+
   CHARACTER(LEN=256)                           :: cf_tfil        ! input T file
   CHARACTER(LEN=256)                           :: cf_sfil        ! input S file (F.Hernandez)
   CHARACTER(LEN=256)                           :: cf_out='mxl.nc'! output file name
@@ -89,30 +92,29 @@ PROGRAM cdfmxl
 
   narg = iargc()
   IF ( narg == 0 ) THEN
-     PRINT *,' usage : cdfmxl T-file [S-file] [-nc4] [-o output file]'
+     PRINT *,' usage : cdfmxl -t T-file [-s S-file] [-o OUT-file] [-nc4]'
      PRINT *,'      '
      PRINT *,'     PURPOSE :'
-     PRINT *,'       Compute 7 estimates of the mixed layer depth from temperature'
-     PRINT *,'       and salinity given in the input file, based on 3 different criteria:'
-     PRINT *,'       1- Density criterium (0.01 kg/m3 difference between surface and MLD)' 
-     PRINT *,'       2- Density criterium (0.03 kg/m3 difference between surface and MLD)' 
-     PRINT *,'       3- Temperature criterium (0.2 C absolute difference between surface '
+     PRINT *,'       Compute 7 estimates of the mixed layer depth from temperature and '
+     PRINT *,'       salinity given in the input file, based on 7 different criteria:'
+     PRINT *,'       1- Density criterion (0.01 kg/m3 difference between surface and MLD)' 
+     PRINT *,'       2- Density criterion (0.03 kg/m3 difference between surface and MLD)' 
+     PRINT *,'       3- Temperature criterion (0.2 C absolute difference between surface '
      PRINT *,'          and MLD)'
-     PRINT *,'       4- Temperature criterium (0.2 C absolute difference between T at 10m '
+     PRINT *,'       4- Temperature criterion (0.2 C absolute difference between T at 10m '
      PRINT *,'          and MLD)'
-     PRINT *,'       5- Temperature criterium (0.5 C absolute difference between T at 10m '
+     PRINT *,'       5- Temperature criterion (0.5 C absolute difference between T at 10m '
      PRINT *,'          and MLD)'
-     PRINT *,'       6- Density criterium (0.03 kg/m3 difference between rho at 10m and MLD) '
-     PRINT *,'       7- Density criterium (0.125 kg/m3 difference between rho at 10m and MLD) '
+     PRINT *,'       6- Density criterion (0.03 kg/m3 difference between rho at 10m and MLD) '
+     PRINT *,'       7- Density criterion (0.125 kg/m3 difference between rho at 10m and MLD) '
      PRINT *,'      '
      PRINT *,'     ARGUMENTS :'
-     PRINT *,'       T-file   : input netcdf file (gridT)' 
-     PRINT *,'       [S-file] : input netcdf file (gridS) Optional if vosaline not in T-file' 
+     PRINT *,'       -t T-file   : input netcdf file (gridT)' 
      PRINT *,'      '
      PRINT *,'     OPTIONS :'
+     PRINT *,'       [-s S-file] : input netcdf file (gridS), if vosaline not in T-file' 
+     PRINT *,'       [-o OUT-file] : specify the name of output file instead of ', TRIM(cf_out)
      PRINT *,'       [-nc4] : use netcdf4 chunking and deflation on output '
-     PRINT *,'       [-o output file] : specify the name of output file instead of '
-     PRINT *,'                default name ',TRIM(cf_out)
      PRINT *,'      '
      PRINT *,'     REQUIRED FILES :'
      PRINT *,'        ',TRIM(cn_fzgr)
@@ -120,34 +122,33 @@ PROGRAM cdfmxl
      PRINT *,'      '
      PRINT *,'     OUTPUT : '
      PRINT *,'       netcdf file : ', TRIM(cf_out) 
-     PRINT *,'         variables : somxl010    = mld on density criterium 0.01 ref. surf.'
-     PRINT *,'                     somxl030    = mld on density criterium 0.03 ref. surf.'
-     PRINT *,'                     somxlt02    = mld on temperature criterium -0.2 ref. surf.'
-     PRINT *,'                     somxlt02z10 = mld on temperature criterium -0.2 ref. 10m'
-     PRINT *,'                     somxlt05z10 = mld on temperature criterium -0.5 ref. 10m'
-     PRINT *,'                     somxl030z10 = mld on density criterium 0.03 ref. 10m'
-     PRINT *,'                     somxl125z10 = mld on density criterium 0.125 ref. 10m'
-     STOP
+     PRINT *,'         variables : somxl010    = mld on density criterion 0.01 ref. surf.'
+     PRINT *,'                     somxl030    = mld on density criterion 0.03 ref. surf.'
+     PRINT *,'                     somxlt02    = mld on temperature criterion -0.2 ref. surf.'
+     PRINT *,'                     somxlt02z10 = mld on temperature criterion -0.2 ref. 10m'
+     PRINT *,'                     somxlt05z10 = mld on temperature criterion -0.5 ref. 10m'
+     PRINT *,'                     somxl030z10 = mld on density criterion 0.03 ref. 10m'
+     PRINT *,'                     somxl125z10 = mld on density criterion 0.125 ref. 10m'
+     PRINT *,'      '
+     STOP 
   ENDIF
 
-  ijarg = 1 ; ixtra = 0
+  ijarg = 1  ; cf_sfil='none'
+
   DO WHILE ( ijarg <= narg )
-    CALL getarg (ijarg, cldum) ; ijarg = ijarg + 1
-    SELECT CASE (cldum )
-    CASE ( '-nc4' ) ; lnc4 = .TRUE.
-    CASE ( '-o'   ) ; CALL getarg (ijarg,cf_out )   ; ijarg = ijarg + 1
-    CASE DEFAULT
-      ixtra = ixtra + 1
-      SELECT CASE (ixtra )
-      CASE ( 1 ) ; cf_tfil = cldum ; cf_sfil = cf_tfil  ! first free name is a gridT file name
-      CASE ( 2 ) ; cf_sfil = cldum                      ! second free name ( if any) is a gridS file name
-      CASE DEFAULT 
-        PRINT *, ' +++ ERROR : Too many files in input !'
-        STOP 99
-      END SELECT
-    END SELECT
+     CALL getarg (ijarg, cldum) ; ijarg = ijarg + 1
+     SELECT CASE (cldum )
+     CASE ( '-t'   ) ; CALL getarg (ijarg,cf_tfil ) ; ijarg = ijarg + 1
+     CASE ( '-s'   ) ; CALL getarg (ijarg,cf_sfil ) ; ijarg = ijarg + 1
+        ! options
+     CASE ( '-nc4' ) ; lnc4 = .TRUE.
+     CASE ( '-o'   ) ; CALL getarg (ijarg,cf_out  ) ; ijarg = ijarg + 1
+     CASE DEFAULT    ; PRINT *,' ERROR : ',TRIM(cldum) ,' :  unknown option.' ; STOP 99
+     END SELECT
   ENDDO
 
+  IF ( cf_sfil == 'none' ) cf_sfil=cf_tfil
+
   IF ( chkfile(cf_tfil) .OR. chkfile(cn_fzgr) .OR. chkfile(cf_sfil)  ) STOP 99 ! missing file
 
   ! read dimensions 
@@ -156,41 +157,6 @@ PROGRAM cdfmxl
   npk    = getdim (cf_tfil,cn_z)
   npt    = getdim (cf_tfil,cn_t)
 
-  rdep(1) = 0.
-
-  DO ji = 1, jp_varout
-     stypvar(ji)%ichunk = (/npiglo, MAX(1,npjglo/30), 1, 1 /)
-  ENDDO
-
-  ipk(:)                    = 1
-  stypvar(1)%cname          = 'somxl010'
-  stypvar(2)%cname          = 'somxl030'
-  stypvar(3)%cname          = 'somxlt02'
-  stypvar(4)%cname          = 'somxlt02z10' 
-  stypvar(5)%cname          = 'somxlt05z10'
-  stypvar(6)%cname          = 'somxl030z10'
-  stypvar(7)%cname          = 'somxl125z10'
-  stypvar%cunits            = 'm'
-  stypvar%rmissing_value    = rmisval  ! to be compliant with Mercator standards
-  stypvar%valid_min         = 0.
-  stypvar%valid_max         = 7000.
-  stypvar(1)%clong_name     = 'Mixed_Layer_Depth_on_0.01_rho_crit'
-  stypvar(2)%clong_name     = 'Mixed_Layer_Depth_on_0.03_rho_crit'
-  stypvar(3)%clong_name     = 'Mixed_Layer_Depth_on_-0.2_temp_crit'
-  stypvar(4)%clong_name     = 'Mixed_Layer_Depth_on_-0.2_temp_crit ref. 10m'
-  stypvar(5)%clong_name     = 'Mixed_Layer_Depth_on_-0.5_temp_crit ref. 10m'
-  stypvar(6)%clong_name     = 'Mixed_Layer_Depth_on_0.03_rho_crit ref. 10m'
-  stypvar(7)%clong_name     = 'Mixed_Layer_Depth_on_0.125_rho_crit ref. 10m'
-  stypvar(1)%cshort_name    = 'somxl010'
-  stypvar(2)%cshort_name    = 'somxl030'
-  stypvar(3)%cshort_name    = 'somxlt02'
-  stypvar(4)%cshort_name    = 'ILD02z10'
-  stypvar(5)%cshort_name    = 'ILD05z10'
-  stypvar(6)%cshort_name    = 'MLD030z10'
-  stypvar(7)%cshort_name    = 'MLD125z10'
-  stypvar%conline_operation = 'N/A'
-  stypvar%caxis             = 'TYX'
-
   PRINT *, 'npiglo = ', npiglo
   PRINT *, 'npjglo = ', npjglo
   PRINT *, 'npk    = ', npk
@@ -210,14 +176,14 @@ PROGRAM cdfmxl
   ALLOCATE (tmask(npiglo,npjglo), tmask_surf(npiglo,npjglo), tmask_10(npiglo,npjglo))
   ALLOCATE (rho_surf(npiglo,npjglo), tem_surf(npiglo,npjglo)                )
   ALLOCATE (mbathy(npiglo,npjglo)                                           )
-
-  ALLOCATE (gdepw(0:npk), gdept(npk), tim(npt)                              )
+  ALLOCATE (gdepw(0:npk), gdept(npk), dtim(npt)                              )
 
   ! read mbathy and gdepw use real rtem(:,:) as template (getvar is used for real only)
-  IF ( chkfile( cn_fbathylev)  ) THEN
+  IF ( chkfile( cn_fbathylev,ld_verbose=.FALSE.)  ) THEN
      PRINT *, 'Read mbathy in ', TRIM(cn_fzgr),' ...'
      rtem(:,:) = getvar(cn_fzgr,      'mbathy',    1, npiglo, npjglo)
   ELSE
+     PRINT *, 'Read mbathy in ', TRIM(cn_bathylev),' ...'
      rtem(:,:) = getvar(cn_fbathylev, cn_bathylev, 1, npiglo, npjglo)
   ENDIF
 
@@ -236,12 +202,7 @@ PROGRAM cdfmxl
   ! find W levels for later computation
   nkref10 = MINLOC(gdepw(1:npk),gdepw(1:npk)>=10)-1 ;  IF ( nkref10(1) < 1 ) nkref10(1)=1
 
-  ncout = create      (cf_out, cf_tfil, npiglo, npjglo, 1             , ld_nc4=lnc4 )
-  ierr  = createvar   (ncout,  stypvar, jp_varout,      ipk, id_varout, ld_nc4=lnc4 )
-  ierr  = putheadervar(ncout,  cf_tfil, npiglo, npjglo, 1, pdep=rdep)
-
-  tim  = getvar1d(cf_tfil, cn_vtimec, npt     )
-  ierr = putvar1d(ncout,   tim,       npt, 'T')
+  CALL CreateOutput
 
   DO jt=1,npt
 
@@ -356,4 +317,59 @@ PROGRAM cdfmxl
 
   ierr = closeout(ncout)
 
+CONTAINS
+
+  SUBROUTINE CreateOutput
+    !!---------------------------------------------------------------------
+    !!                  ***  ROUTINE CreateOutput  ***
+    !!
+    !! ** Purpose :  Create netcdf output file(s) 
+    !!
+    !! ** Method  :  Use stypvar global description of variables
+    !!
+    !!----------------------------------------------------------------------
+    rdep(1) = 0.
+
+    DO ji = 1, jp_varout
+       stypvar(ji)%ichunk = (/npiglo, MAX(1,npjglo/30), 1, 1 /)
+    ENDDO
+
+    ipk(:)                    = 1
+    stypvar(1)%cname          = 'somxl010'
+    stypvar(2)%cname          = 'somxl030'
+    stypvar(3)%cname          = 'somxlt02'
+    stypvar(4)%cname          = 'somxlt02z10' 
+    stypvar(5)%cname          = 'somxlt05z10'
+    stypvar(6)%cname          = 'somxl030z10'
+    stypvar(7)%cname          = 'somxl125z10'
+    stypvar%cunits            = 'm'
+    stypvar%rmissing_value    = rmisval  ! to be compliant with Mercator standards
+    stypvar%valid_min         = 0.
+    stypvar%valid_max         = 7000.
+    stypvar(1)%clong_name     = 'Mixed_Layer_Depth_on_0.01_rho_crit'
+    stypvar(2)%clong_name     = 'Mixed_Layer_Depth_on_0.03_rho_crit'
+    stypvar(3)%clong_name     = 'Mixed_Layer_Depth_on_-0.2_temp_crit'
+    stypvar(4)%clong_name     = 'Mixed_Layer_Depth_on_-0.2_temp_crit ref. 10m'
+    stypvar(5)%clong_name     = 'Mixed_Layer_Depth_on_-0.5_temp_crit ref. 10m'
+    stypvar(6)%clong_name     = 'Mixed_Layer_Depth_on_0.03_rho_crit ref. 10m'
+    stypvar(7)%clong_name     = 'Mixed_Layer_Depth_on_0.125_rho_crit ref. 10m'
+    stypvar(1)%cshort_name    = 'somxl010'
+    stypvar(2)%cshort_name    = 'somxl030'
+    stypvar(3)%cshort_name    = 'somxlt02'
+    stypvar(4)%cshort_name    = 'ILD02z10'
+    stypvar(5)%cshort_name    = 'ILD05z10'
+    stypvar(6)%cshort_name    = 'MLD030z10'
+    stypvar(7)%cshort_name    = 'MLD125z10'
+    stypvar%conline_operation = 'N/A'
+    stypvar%caxis             = 'TYX'
+
+    ncout = create      (cf_out, cf_tfil, npiglo, npjglo, 1             , ld_nc4=lnc4 )
+    ierr  = createvar   (ncout,  stypvar, jp_varout,      ipk, id_varout, ld_nc4=lnc4 )
+    ierr  = putheadervar(ncout,  cf_tfil, npiglo, npjglo, 1, pdep=rdep)
+
+    dtim = getvar1d(cf_tfil, cn_vtimec, npt     )
+    ierr = putvar1d(ncout,  dtim,       npt, 'T')
+
+  END SUBROUTINE CreateOutput
+
 END PROGRAM cdfmxl
