@@ -17,22 +17,25 @@ PROGRAM cdfpsi
   !!
   !! History : 2.1  : 05/2005  : J.M. Molines : Original code
   !!           3.0  : 05/2011  : J.M. Molines : Doctor norm + Lic.
+  !!         : 4.0  : 03/2017  : J.M. Molines  
   !!----------------------------------------------------------------------
   USE cdfio
   USE modcdfnames
   USE modutils
   !!----------------------------------------------------------------------
-  !! CDFTOOLS_3.0 , MEOM 2011
+  !! CDFTOOLS_4.0 , MEOM 2017 
   !! $Id$
-  !! Copyright (c) 2011, J.-M. Molines
+  !! Copyright (c) 2017, J.-M. Molines 
   !! Software governed by the CeCILL licence (Licence/CDFTOOLSCeCILL.txt)
+  !! @class transport
   !!----------------------------------------------------------------------
   IMPLICIT NONE
 
   INTEGER(KIND=4)                           :: ji, jj, jk, jt  ! dummy loop index
+  INTEGER(KIND=4)                           :: it              ! time idex for vvl
   INTEGER(KIND=4)                           :: ierr            ! working integer
   INTEGER(KIND=4)                           :: narg, iargc     ! command line 
-  INTEGER(KIND=4)                           :: ijarg, ireq     ! command line
+  INTEGER(KIND=4)                           :: ijarg           ! command line
   INTEGER(KIND=4)                           :: npiglo, npjglo  ! size of the domain
   INTEGER(KIND=4)                           :: npk, npt        ! size of the domain
   INTEGER(KIND=4)                           :: ncout           ! ncid of output file
@@ -47,9 +50,10 @@ PROGRAM cdfpsi
   REAL(KIND=4), DIMENSION(:,:), ALLOCATABLE :: glamf, gphif    ! longitude/latitude
   REAL(KIND=4), DIMENSION(:,:), ALLOCATABLE :: zsshu, zsshv    ! ssh at u and v point
   REAL(KIND=4), DIMENSION(:,:), ALLOCATABLE :: zssh            ! temporary array for ssh
-  REAL(KIND=4), DIMENSION(:),   ALLOCATABLE :: tim             ! time counter
   REAL(KIND=4), DIMENSION(:),   ALLOCATABLE :: e31d            ! 1d vertical metrics, full step case
 
+  REAL(KIND=8), DIMENSION(:),   ALLOCATABLE :: dtim            ! time counter
+  REAL(KIND=8), DIMENSION(:,:), ALLOCATABLE :: dztrpu, dztrpv  ! transport working arrays
   REAL(KIND=8), DIMENSION(:,:), ALLOCATABLE :: dtrpu, dtrpv    ! transport working arrays
   REAL(KIND=8), DIMENSION(:,:), ALLOCATABLE :: dtrpsshu        ! transport working arrays
   REAL(KIND=8), DIMENSION(:,:), ALLOCATABLE :: dtrpsshv        ! transport working arrays
@@ -62,7 +66,7 @@ PROGRAM cdfpsi
 
   CHARACTER(LEN=256)                        :: cf_ufil         ! gridU netcdf file name
   CHARACTER(LEN=256)                        :: cf_vfil         ! gridV netcdf file name
-  CHARACTER(LEN=256)                        :: cf_tfil         ! gridT netcdf file name (-ssh option)
+  CHARACTER(LEN=255)                        :: cf_sshfil       ! input SSH file (option)
   CHARACTER(LEN=256)                        :: cf_out='psi.nc' ! output file name
   CHARACTER(LEN=256)                        :: cv_out='sobarstf' ! output variable name
   CHARACTER(LEN=256)                        :: cv_outssh='sobarstfssh'    ! output variable name
@@ -80,38 +84,47 @@ PROGRAM cdfpsi
   LOGICAL                                   :: lmean = .FALSE. ! flag for mean U,V calculation
   LOGICAL                                   :: lopen = .FALSE. ! flag for open calculation
   LOGICAL                                   :: lssh  = .FALSE. ! flag for ssh computation
-  LOGICAL                                   :: lnc4  = .FALSE. ! flag for netcdf4 cchunking and deflation
+  LOGICAL                                   :: lnc4  = .FALSE. ! flag for netcdf4 chunking and deflation
+  LOGICAL                                   :: llev  = .FALSE. ! flag for level computation (cdfpsi_level like)
   !!----------------------------------------------------------------------
   CALL ReadCdfNames()
 
   narg= iargc()
   IF ( narg == 0 ) THEN
-     PRINT *,' usage : cdfpsi U-file V-file [V] [-full ] [-mask ] [-mean] [-nc4 ] ...'
-     PRINT *,'          ... [-ssh T-file ] [-open ] [-ref iref jref ] [-o OUT-file]'
+     PRINT *,' usage : cdfpsi -u U-file -v V-file [-V] [-full] [-mask] [-mean] [-nc4]    ...'
+     PRINT *,'          ... [--ssh-file SSH-file] [-open] [-ref iref jref] [-o OUT-file] ...'
+     PRINT *,'          ... [-vvl] [-lev]'
      PRINT *,'      '
      PRINT *,'     PURPOSE :'
-     PRINT *,'       Computes the barotropic stream function (a proxy ) as the integral of '
-     PRINT *,'       the transport.' 
+     PRINT *,'       Compute the barotropic stream function (a proxy) as the integral of '
+     PRINT *,'       the transport. It is a proxy, as far as the flow in not strictly non-'
+     PRINT *,'       divergent. '
      PRINT *,'      '
      PRINT *,'     ARGUMENTS :'
-     PRINT *,'       U-file  : netcdf file of zonal velocity.' 
-     PRINT *,'       V-file  : netcdf file of meridional velocity.' 
+     PRINT *,'       -u U-file  : netcdf file of zonal velocity.' 
+     PRINT *,'       -v V-file  : netcdf file of meridional velocity.' 
      PRINT *,'      '
      PRINT *,'     OPTIONS :'
-     PRINT *,'       [V] : use V field instead of U field for integration.' 
-     PRINT *,'       [ -full ] : indicates a full step case. Default is partial steps.'
-     PRINT *,'       [ -mask ] : mask output fields. Note that the land value is significant.'
-     PRINT *,'                   It correspond to the potential on this continent.'
-     PRINT *,'       [ -mean ] : save the average of the computations done with U and V.'
-     PRINT *,'       [ -nc4  ] : use netcdf4 output files with chunking and deflation'
-     PRINT *,'       [ -ssh T-file ] : compute the transport in the ''ssh'' layer, using '
-     PRINT *,'                  surface velocities. Take the ssh from T-file specified in '
-     PRINT *,'                  this option. This is a experimental option, not certified ...'
-     PRINT *,'       [ -open ] : for open domain configuration. See also -ref to set '
-     PRINT *,'                   reference point.'
-     PRINT *,'       [ -ref iref jref ] : Set the reference point in i,j coordinates.'
-     PRINT *,'                   BSF at reference point is arbitrarly set to zero.'
-     PRINT *,'       [ -o  OUT-file ] : specify output file name instead of default ',TRIM(cf_out)
+     PRINT *,'       [-V ] : use V field instead of U field for integration.' 
+     PRINT *,'       [-full ] : indicates a full step case. Default is partial steps.'
+     PRINT *,'       [-mask ] : mask output fields. Note that the land value is significant.'
+     PRINT *,'              It correspond to the potential on this continent.'
+     PRINT *,'       [-mean ] : save the average of the computations done with U and V.'
+     PRINT *,'       [--ssh-file SSH-file] : compute the transport in the ''ssh'' layer, using '
+     PRINT *,'              surface velocities. Take the ssh from T-file specified in this'
+     PRINT *,'              option. This is an experimental option, not certified ...'
+     PRINT *,'       [-open ] : for open domain configuration. See also -ref to set  '
+     PRINT *,'              reference point.'
+     PRINT *,'       [-ref iref jref ] : Set the reference point in i,j coordinates. BSF at'
+     PRINT *,'              reference point is arbitrarly set to zero.'
+     PRINT *,'       [-o  OUT-file ] : specify output file name instead of default ',TRIM(cf_out)
+     PRINT *,'       [-nc4  ] : use netcdf4 output files with chunking and deflation'
+     PRINT *,'       [-vvl  ] : use time-varying vertical metrics'
+     PRINT *,'       [-lev] : This option is still experimental. It has been added to cdfpsi'
+     PRINT *,'            in order to mimic former ''cdfpsi_level'', now removed. '
+     PRINT *,'            With this option, integral of transports is saved at every level,'
+     PRINT *,'            giving a ''pseudo'' stream-function. It works only for global'
+     PRINT *,'            configurations, so far.'
      PRINT *,'      '
      PRINT *,'     REQUIRED FILES :'
      PRINT *,'       ', TRIM(cn_fhgr),' and ', TRIM(cn_fzgr),'.'
@@ -124,48 +137,63 @@ PROGRAM cdfpsi
      PRINT *,'                     ', TRIM(cv_outssh),' (m3/s ) : contribution of SSH'
      PRINT *,'                     ', TRIM(cv_outotal),' (m3/s ) : total BSF'
      PRINT *,'      '
-     STOP
+     STOP 
   ENDIF
 
   CALL SetGlobalAtt (cglobal)
   iiref = -1 ; ijref= -1
 
-  ijarg = 1 ; ireq = 0
+  ijarg = 1
   DO WHILE ( ijarg <= narg )
-     CALL getarg( ijarg, cldum ) ; ijarg=ijarg + 1
+     CALL getarg( ijarg, cldum ) ; ijarg=ijarg+1
      SELECT CASE ( cldum )
-     CASE ('-full') ; lfull = .TRUE.
-     CASE ('-mask') ; lmask = .TRUE.
-     CASE ('-mean') ; lmean = .TRUE.  ; ll_v=.TRUE. ; ll_u=.TRUE.
-     CASE ('-ssh' ) ; lssh  = .TRUE.  ; nvout=3
-        CALL getarg( ijarg, cf_tfil ) ; ijarg=ijarg + 1 
-     CASE ('-nc4' ) ; lnc4  = .TRUE. 
-     CASE ('-open') ; lopen = .TRUE.  ; ll_v=.TRUE. ; ll_u=.TRUE.
-     CASE ('-o'   ) ; CALL getarg( ijarg, cf_out )   ; ijarg=ijarg + 1 
-     CASE ('-ref') 
-        CALL getarg( ijarg, cldum )   ; ijarg=ijarg + 1 ; READ(cldum,*) iiref
-        CALL getarg( ijarg, cldum )   ; ijarg=ijarg + 1 ; READ(cldum,*) ijref
-
-     CASE DEFAULT
-        ireq = ireq + 1
-        SELECT CASE ( ireq)
-        CASE ( 1 ) ; cf_ufil = cldum
-        CASE ( 2 ) ; cf_vfil = cldum
-        CASE ( 3 ) ; ll_v = .TRUE. ; ll_u = .FALSE.
-        CASE DEFAULT
-           PRINT *, ' Too many arguments !' ; STOP 99
-        END SELECT
+     CASE ('-u'        ) ; CALL getarg( ijarg, cf_ufil ) ; ijarg=ijarg+1 
+     CASE ('-v'        ) ; CALL getarg( ijarg, cf_vfil ) ; ijarg=ijarg+1 
+        ! options 
+     CASE ('-V'        ) ; ll_v  = .TRUE. ;  ll_u = .FALSE.
+     CASE ('-full'     ) ; lfull = .TRUE.
+     CASE ('-mask'     ) ; lmask = .TRUE.
+     CASE ('-mean'     ) ; lmean = .TRUE.  ; ll_v = .TRUE. ; ll_u = .TRUE.
+     CASE ('--ssh-file') ; lssh  = .TRUE.  ; nvout=3
+        ;                  CALL getarg( ijarg, cf_sshfil ) ; ijarg=ijarg+1 
+     CASE ('-open'     ) ; lopen = .TRUE.  ; ll_v = .TRUE. ; ll_u = .TRUE.
+     CASE ('-o'        ) ; CALL getarg( ijarg, cf_out ) ; ijarg=ijarg + 1 
+     CASE ('-nc4'      ) ; lnc4  = .TRUE. 
+     CASE ('-ref'      ) ; CALL getarg( ijarg, cldum  ) ; ijarg=ijarg+1 ; READ(cldum,*) iiref
+        ;                  CALL getarg( ijarg, cldum  ) ; ijarg=ijarg+1 ; READ(cldum,*) ijref
+     CASE ('-vvl'      ) ; lg_vvl = .TRUE.
+     CASE ('-lev'      ) ; llev   = .TRUE. ; cv_out='vobarstf'
+     CASE DEFAULT        ; PRINT *,' ERROR : ',TRIM(cldum), ' : unknown option.' ; STOP 99
      END SELECT
   ENDDO
 
   lchk = lchk .OR. chkfile( cn_fhgr )
   lchk = lchk .OR. chkfile( cn_fzgr )
   IF ( lmask) lchk = lchk .OR. chkfile( cn_fmsk )
-  IF ( lssh ) lchk = lchk .OR. chkfile( cf_tfil )
+  IF ( lssh ) lchk = lchk .OR. chkfile( cf_sshfil )
   lchk = lchk .OR. chkfile( cf_ufil )
   lchk = lchk .OR. chkfile( cf_vfil )
 
   IF ( lchk ) STOP 99 ! missing file
+  IF ( lg_vvl ) THEN 
+     cn_fe3u = cf_ufil
+     cn_fe3v = cf_vfil
+     cn_ve3u = cn_ve3uvvl
+     cn_ve3v = cn_ve3vvvl
+  ENDIF
+  ! check option compatibility
+  IF ( llev .AND. lopen) THEN
+     PRINT *,' +++ error : cannot use both -lev and -open options.' ; STOP 99 
+  ENDIF
+  IF ( llev .AND. lssh) THEN
+     PRINT *,' +++ error : cannot use both -lev and -ssh options.' ; STOP 99 
+  ENDIF
+  IF ( llev .AND. lmean) THEN
+     PRINT *,' +++ error : cannot use both -lev and -mean options.' ; STOP 99 
+  ENDIF
+  IF ( llev .AND. lmask) THEN
+     PRINT *,' +++ error : cannot use both -lev and -mask options.' ; STOP 99 
+  ENDIF
 
   npiglo = getdim (cf_ufil, cn_x)
   npjglo = getdim (cf_ufil, cn_y)
@@ -178,56 +206,33 @@ PROGRAM cdfpsi
   ENDIF
 
   ALLOCATE (stypvar(nvout), ipk(nvout), id_varout(nvout))
-  ! define new variables for output ( must update att.txt)
-  ipk(:) = 1  !  2D ( X, Y , T )
-  stypvar(:)%cunits            = 'm3/s'
-  stypvar(:)%valid_min         = -300.e6
-  stypvar(:)%valid_max         = 300.e6
-  stypvar(:)%conline_operation = 'N/A'
-  stypvar(:)%caxis             = 'TYX'
-  DO ji=1,nvout
-    stypvar(ji)%ichunk = (/npiglo,MAX(1,npjglo/30),1,1 /)
-  ENDDO
-
-  stypvar(1)%cname             = cv_out
-  stypvar(1)%rmissing_value    = 0.
-  stypvar(1)%clong_name        = 'Barotropic_Stream_Function'
-  stypvar(1)%cshort_name       = cv_out
-
-  IF ( lssh ) THEN
-     stypvar(2)%cname             = cv_outssh
-     stypvar(2)%rmissing_value    = 0.
-     stypvar(2)%clong_name        = 'Barotropic_Stream_Function SSH contribution'
-     stypvar(2)%cshort_name       = cv_outssh
-
-     stypvar(3)%cname             = cv_outotal
-     stypvar(3)%rmissing_value    = 0.
-     stypvar(3)%clong_name        = 'Barotropic_Stream_Function SSH total'
-     stypvar(3)%cshort_name       = cv_outotal
-  ENDIF
 
   PRINT *, 'npiglo = ', npiglo
   PRINT *, 'npjglo = ', npjglo
   PRINT *, 'npk    = ', npk
   PRINT *, 'npt    = ', npt
   PRINT *, ' Option is use :'
-  PRINT *, '    -full :', lfull
-  PRINT *, '    -mask :', lmask
-  PRINT *, '    -mean :', lmean
-  PRINT *, '    -ssh  :', lssh
-  PRINT *, '    -open :', lopen
-  PRINT *, '    -ref  :', iiref, ijref
-  PRINT *, '  U-comp  :', ll_u
-  PRINT *, '  V-comp  :', ll_v
+  PRINT *, '    -full : ', lfull
+  PRINT *, '    -mask : ', lmask
+  PRINT *, '    -mean : ', lmean
+  PRINT *, '    -ssh  : ', lssh
+  PRINT *, '    -open : ', lopen
+  PRINT *, '    -ref  : ', iiref, ijref
+  PRINT *, '  U-comp  : ', ll_u
+  PRINT *, '  V-comp  : ', ll_v
+  PRINT *, '      -o  : ', TRIM(cf_out)
+  PRINT *, '    -lnc4 : ', lnc4
+  PRINT *, '    -vvl  : ', lg_vvl
 
   ! Allocate arrays
   ALLOCATE ( zmask(npiglo,npjglo)                 )
   ALLOCATE ( e1v(npiglo,npjglo),e3v(npiglo,npjglo))
   ALLOCATE ( e2u(npiglo,npjglo),e3u(npiglo,npjglo))
-  ALLOCATE ( zu(npiglo,npjglo),dtrpu(npiglo,npjglo), dpsiu(npiglo,npjglo) )
-  ALLOCATE ( zv(npiglo,npjglo),dtrpv(npiglo,npjglo), dpsiv(npiglo,npjglo) )
+  ALLOCATE ( zu(npiglo,npjglo),dtrpu(npiglo,npjglo), dpsiu(npiglo,npjglo), dztrpu(npiglo,npjglo) )
+  ALLOCATE ( zv(npiglo,npjglo),dtrpv(npiglo,npjglo), dpsiv(npiglo,npjglo), dztrpv(npiglo,npjglo) )
   ALLOCATE ( glamf(npiglo,npjglo), gphif(npiglo,npjglo))
-  ALLOCATE ( tim(npt))
+  ALLOCATE ( dtim(npt))
+
   IF ( lfull ) ALLOCATE ( e31d(npk))
   IF ( lssh  ) ALLOCATE ( zssh(npiglo,npjglo), zsshu(npiglo,npjglo), zsshv(npiglo,npjglo))
   IF ( lssh  ) ALLOCATE ( dpsisshu(npiglo,npjglo), dpsisshv(npiglo,npjglo) )
@@ -236,18 +241,12 @@ PROGRAM cdfpsi
   glamf(:,:) = getvar(cn_fhgr, cn_glamf, 1, npiglo, npjglo)
   gphif(:,:) = getvar(cn_fhgr, cn_gphif, 1, npiglo, npjglo)
 
-  ! create output fileset
-  ncout = create      (cf_out, cf_ufil, npiglo, npjglo, 1                                , ld_nc4=lnc4 )
-  ierr  = createvar   (ncout,  stypvar, nvout,  ipk,    id_varout, cdglobal=TRIM(cglobal), ld_nc4=lnc4 )
-  ierr  = putheadervar(ncout,  cf_ufil, npiglo, npjglo, 1, glamf, gphif)
-
-  tim  = getvar1d(cf_ufil, cn_vtimec, npt     )
-  ierr = putvar1d(ncout,   tim,       npt, 'T')
+  CALL CreateOutput
 
   e1v(:,:)   = getvar(cn_fhgr, cn_ve1v, 1, npiglo, npjglo)
   e2u(:,:)   = getvar(cn_fhgr, cn_ve2u, 1, npiglo, npjglo)
   IF ( lmask) THEN
-     zmask(:,:) = getvar(cn_fmsk, 'fmask', 1, npiglo, npjglo)
+     zmask(:,:) = getvar(cn_fmsk, cn_fmask, 1, npiglo, npjglo)
      WHERE ( zmask >= 2 ) zmask = 1
   ENDIF
 
@@ -255,20 +254,17 @@ PROGRAM cdfpsi
   ! get rid of the free-slip/no-slip condition
 
   DO jt = 1, npt
-     dtrpu(:,:)= 0.d0
-     dtrpv(:,:)= 0.d0
-     dpsiu(:,:)= 0.d0
-     dpsiv(:,:)= 0.d0
+     IF ( lg_vvl ) THEN ; it=jt
+     ELSE               ; it=1
+     ENDIF
+     dtrpu(:,:)= 0.d0 ; dtrpv(:,:)= 0.d0 ; dpsiu(:,:)= 0.d0 ; dpsiv(:,:)= 0.d0
      IF ( lssh ) THEN
-       zsshu(:,:) = 0.0
-       zsshv(:,:) = 0.0
-       dpsisshu(:,:) = 0.d0
-       dpsisshv(:,:) = 0.d0
-       dtrpsshu(:,:) = 0.d0
-       dtrpsshv(:,:) = 0.d0
-       zssh(:,:) = getvar(cf_tfil, cn_sossheig, 1, npiglo, npjglo, ktime=jt)
-       zsshu(1:npiglo-1,    :     ) = 0.5*( zssh(2:npiglo,:       ) + zssh(1:npiglo-1,:         ))
-       zsshv(   :      ,1:npjglo-1) = 0.5*( zssh(:       ,2:npjglo) + zssh(:         ,1:npjglo-1))
+        zsshu(:,:) = 0.0 ; zsshv(:,:) = 0.0
+        dpsisshu(:,:) = 0.d0 ; dpsisshv(:,:) = 0.d0 ; dtrpsshu(:,:) = 0.d0 ; dtrpsshv(:,:) = 0.d0
+
+        zssh(:          ,    :     ) = getvar(cf_sshfil, cn_sossheig, 1, npiglo, npjglo, ktime=jt )
+        zsshu(1:npiglo-1,    :     ) = 0.5*( zssh(2:npiglo,:       ) + zssh(1:npiglo-1,:         ))
+        zsshv(   :      ,1:npjglo-1) = 0.5*( zssh(:       ,2:npjglo) + zssh(:         ,1:npjglo-1))
      ENDIF
 
 
@@ -276,42 +272,64 @@ PROGRAM cdfpsi
         IF ( ll_v ) THEN
            zv(:,:) = getvar(cf_vfil, cn_vomecrty, jk, npiglo, npjglo, ktime=jt )
            IF ( lfull ) THEN ; e3v(:,:) = e31d(jk)
-           ELSE              ; e3v(:,:) = getvar(cn_fzgr, 'e3v_ps', jk, npiglo, npjglo, ldiom=.TRUE.)
+           ELSE              ; e3v(:,:) = getvar(cn_fe3v, cn_ve3v, jk, npiglo, npjglo, ktime=it, ldiom=.NOT.lg_vvl)
            ENDIF
-           dtrpv(:,:) = dtrpv(:,:) + zv(:,:)*e1v(:,:)*e3v(:,:)*1.d0  ! meridional transport of each grid cell
+           dztrpv(:,:) = zv(:,:)*e1v(:,:)*e3v(:,:)*1.d0 ! meridional transport of each grid cell
+           dtrpv(:,:) = dtrpv(:,:) + dztrpv(:,:)        ! cumul over the depth
            IF ( lssh .AND. (jk == 1 ) ) THEN
-             dtrpsshv(:,:) = dtrpsshv(:,:) + zv(:,:)*e1v(:,:)*zsshv(:,:)*1.d0  ! meridional transport of each grid cell
+              dtrpsshv(:,:) = dtrpsshv(:,:) + zv(:,:)*e1v(:,:)*zsshv(:,:)*1.d0  ! meridional transport of each grid cell
+           ENDIF
+           IF ( llev ) THEN  ! compute zonal integral
+              dpsiv(npiglo,:)= 0.d0
+              DO ji=npiglo-1,1,-1
+                 dpsiv(ji,:) = dpsiv(ji+1,:) - dztrpv(ji,:)  ! psi at f point
+              END DO
+              ierr = putvar(ncout, id_varout(1), SNGL(dpsiv), jk, npiglo, npjglo, ktime=jt)
            ENDIF
         ENDIF
 
         IF ( ll_u) THEN
            zu(:,:) = getvar(cf_ufil, cn_vozocrtx, jk, npiglo, npjglo, ktime=jt )
            IF ( lfull ) THEN ; e3u(:,:) = e31d(jk)
-           ELSE              ; e3u(:,:) = getvar(cn_fzgr, 'e3u_ps', jk, npiglo, npjglo, ldiom=.TRUE.)
+           ELSE              ; e3u(:,:) = getvar(cn_fe3u, cn_ve3u, jk, npiglo, npjglo, ktime=it, ldiom=.NOT.lg_vvl)
            ENDIF
-           dtrpu(:,:) = dtrpu(:,:) + zu(:,:)*e2u(:,:)*e3u(:,:)*1.d0  ! zonal transport of each grid cell
+           dztrpu(:,:)=  zu(:,:)*e2u(:,:)*e3u(:,:)*1.d0  !  zonal transport of each grid cell
+           dtrpu(:,:) = dtrpu(:,:) + dztrpu(:,:)         ! cumul over the depth
            IF ( lssh .AND. (jk == 1 ) ) THEN
-             dtrpsshu(:,:) = dtrpsshu(:,:) + zv(:,:)*e2u(:,:)*zsshu(:,:)*1.d0  ! meridional transport of each grid cell
+              dtrpsshu(:,:) = dtrpsshu(:,:) + zv(:,:)*e2u(:,:)*zsshu(:,:)*1.d0  ! meridional transport of each grid cell
+           ENDIF
+           IF ( llev ) THEN
+              dpsiu(:,:)= 0.d0
+              DO jj = 2, npjglo
+                 dpsiu(:,jj) = dpsiu(:,jj-1) - dztrpu(:,jj)   ! psi at f point
+              END DO
+              ierr = putvar(ncout, id_varout(1), SNGL(dpsiu), jk, npiglo, npjglo, ktime=jt)
            ENDIF
         ENDIF
      END DO  ! loop to next level
 
+     IF ( llev ) THEN  ! stop here !
+        ierr = closeout (ncout)
+        PRINT *,' cdfpsi_level completed.'
+        STOP 0
+     ENDIF
+
      IF ( lopen ) THEN
         ! This case corresponds to arbitrary configuration: we chose to compute the transport
         ! across a first line ( eg, ji= 2 or jj= npjglo-1 ), assuming that this starting line is
         ! in the true ocean. If it is on true land, it is not a problem. But it cannot be on
         ! arbitrary masked points....
         IF ( lssh ) THEN
-          dpsisshu(1,npjglo-2) = dtrpsshv(1, npjglo-2)
-          DO ji = 2, npiglo
-             dpsisshu(ji,npjglo-2) = dpsisshu(ji-1,npjglo-2) + dtrpsshv(ji,npjglo-2)
-          END DO
-          ! Then compute the transport with along U starting from this line
-          DO jj= npjglo-3,1,-1
-             DO ji = 1, npiglo
-                dpsisshu(ji,jj) = dpsisshu(ji,jj+1) + dtrpsshu(ji,jj+1)
-             END DO
-          END DO
+           dpsisshu(1,npjglo-2) = dtrpsshv(1, npjglo-2)
+           DO ji = 2, npiglo
+              dpsisshu(ji,npjglo-2) = dpsisshu(ji-1,npjglo-2) + dtrpsshv(ji,npjglo-2)
+           END DO
+           ! Then compute the transport with along U starting from this line
+           DO jj= npjglo-3,1,-1
+              DO ji = 1, npiglo
+                 dpsisshu(ji,jj) = dpsisshu(ji,jj+1) + dtrpsshu(ji,jj+1)
+              END DO
+           END DO
         ENDIF
 
         dpsiu(1,npjglo-2) = dtrpv(1, npjglo-2)
@@ -387,7 +405,7 @@ PROGRAM cdfpsi
               DO jj = 2, npjglo
                  dpsisshu(:,jj) = dpsisshu(:,jj-1) - dtrpsshu(:,jj)   ! psissh at f point
               END DO
-                 dpsissh => dpsisshu
+              dpsissh => dpsisshu
            ENDIF
         ENDIF
 
@@ -421,5 +439,69 @@ PROGRAM cdfpsi
   ENDDO
 
   ierr = closeout (ncout)
+CONTAINS
+
+  SUBROUTINE CreateOutput
+    !!---------------------------------------------------------------------
+    !!                  ***  ROUTINE CreateOutput  ***
+    !!
+    !! ** Purpose :  Create netcdf output file(s) 
+    !!
+    !! ** Method  :  Use stypvar global description of variables
+    !!
+    !!----------------------------------------------------------------------
+    IF ( llev ) THEN
+       ! define new variables for output ( must update att.txt)
+       ipk(1) = npk  !  2D ( X, Y , T )
+       stypvar(1)%cname             = cv_out
+       stypvar(1)%rmissing_value    = 0.
+       stypvar(1)%clong_name        = 'Barotropic_Stream_Function_per_level'
+       stypvar(1)%cshort_name       = cv_out
+       stypvar(1)%cunits            = 'm3/s'
+       stypvar(1)%valid_min         = -300.e6
+       stypvar(1)%valid_max         = 300.e6
+       stypvar(1)%conline_operation = 'N/A'
+       stypvar(1)%caxis             = 'TYX'
+       stypvar(1)%ichunk = (/npiglo,MAX(1,npjglo/30),1,1 /)
+
+
+    ELSE
+       ! define new variables for output ( must update att.txt)
+       ipk(:) = 1  !  2D ( X, Y , T )
+       stypvar(:)%cunits            = 'm3/s'
+       stypvar(:)%valid_min         = -300.e6
+       stypvar(:)%valid_max         = 300.e6
+       stypvar(:)%conline_operation = 'N/A'
+       stypvar(:)%caxis             = 'TYX'
+       DO ji=1,nvout
+          stypvar(ji)%ichunk = (/npiglo,MAX(1,npjglo/30),1,1 /)
+       ENDDO
+
+       stypvar(1)%cname             = cv_out
+       stypvar(1)%rmissing_value    = 0.
+       stypvar(1)%clong_name        = 'Barotropic_Stream_Function'
+       stypvar(1)%cshort_name       = cv_out
+
+       IF ( lssh ) THEN
+          stypvar(2)%cname             = cv_outssh
+          stypvar(2)%rmissing_value    = 0.
+          stypvar(2)%clong_name        = 'Barotropic_Stream_Function SSH contribution'
+          stypvar(2)%cshort_name       = cv_outssh
+
+          stypvar(3)%cname             = cv_outotal
+          stypvar(3)%rmissing_value    = 0.
+          stypvar(3)%clong_name        = 'Barotropic_Stream_Function SSH total'
+          stypvar(3)%cshort_name       = cv_outotal
+       ENDIF
+    ENDIF
+    ! create output fileset
+    ncout = create      (cf_out, cf_ufil, npiglo, npjglo, 1                                , ld_nc4=lnc4 )
+    ierr  = createvar   (ncout,  stypvar, nvout,  ipk,    id_varout, cdglobal=TRIM(cglobal), ld_nc4=lnc4 )
+    ierr  = putheadervar(ncout,  cf_ufil, npiglo, npjglo, 1, glamf, gphif)
+
+    dtim = getvar1d(cf_ufil, cn_vtimec, npt     )
+    ierr = putvar1d(ncout,   dtim,      npt, 'T')
+
+  END SUBROUTINE CreateOutput
 
 END PROGRAM cdfpsi
