@@ -28,22 +28,25 @@ PROGRAM cdfpvor
   !!
   !! History : 2.1  : 12/2005  : A.M. Treguier : Original code
   !!           3.0  : 05/2011  : J.M. Molines  : Doctor norm + Lic., merge with cdfpv
+  !!         : 4.0  : 03/2017  : J.M. Molines  
   !!-------------------------------------------------------------------
   USE cdfio
   USE eos
   USE modcdfnames
   !!----------------------------------------------------------------------
-  !! CDFTOOLS_3.0 , MEOM 2011
+  !! CDFTOOLS_4.0 , MEOM 2017 
   !! $Id$
-  !! Copyright (c) 2011, J.-M. Molines
+  !! Copyright (c) 2017, J.-M. Molines 
   !! Software governed by the CeCILL licence (Licence/CDFTOOLSCeCILL.txt)
+  !! @class derived_fields
   !!----------------------------------------------------------------------
   IMPLICIT NONE
 
   INTEGER(KIND=4)                             :: jk, jj, ji, jt       ! dummy loop index
+  INTEGER(KIND=4)                             :: it                   ! time index for vvl
   INTEGER(KIND=4)                             :: ierr                 ! working integer
   INTEGER(KIND=4)                             :: narg, iargc          ! command line
-  INTEGER(KIND=4)                             :: ijarg, ireq          ! command line
+  INTEGER(KIND=4)                             :: ijarg                ! command line
   INTEGER(KIND=4)                             :: npiglo, npjglo       ! size of the domain
   INTEGER(KIND=4)                             :: npk, npt             ! size of the domain
   INTEGER(KIND=4)                             :: iup=1, idown=2, itmp ! working interger
@@ -51,19 +54,19 @@ PROGRAM cdfpvor
   INTEGER(KIND=4)                             :: nvar=3               ! number of output variable
   INTEGER(KIND=4), DIMENSION(:), ALLOCATABLE  :: ipk, id_varout       ! output variable id's
 
-  REAL(KIND=4), DIMENSION(:,:,:), ALLOCATABLE :: ztemp, zsal, zwk     ! array to ead 2 layer of data
+  REAL(KIND=4)                                :: zpi, zomega, rau0sg  ! physical constant
+  REAL(KIND=4)                                :: zsps                 ! Missing value for salinity
+  REAL(KIND=4), DIMENSION(:),     ALLOCATABLE :: gdepw                ! deptht
+  REAL(KIND=4), DIMENSION(:),     ALLOCATABLE :: e31d                 ! metric for full step
   REAL(KIND=4), DIMENSION(:,:),   ALLOCATABLE :: zn2                  ! Brunt Vaissala frequency
   REAL(KIND=4), DIMENSION(:,:),   ALLOCATABLE :: tmask                ! tmask from salinity
   REAL(KIND=4), DIMENSION(:,:),   ALLOCATABLE :: e3w                  ! vertical metric
   REAL(KIND=4), DIMENSION(:,:),   ALLOCATABLE :: e1u, e2v             ! horizontal metric
   REAL(KIND=4), DIMENSION(:,:),   ALLOCATABLE :: e1t, e2t             ! horizontal metric at T point
   REAL(KIND=4), DIMENSION(:,:),   ALLOCATABLE :: gphit                ! latitude of t point
-  REAL(KIND=4), DIMENSION(:),     ALLOCATABLE :: gdepw                ! deptht
-  REAL(KIND=4), DIMENSION(:),     ALLOCATABLE :: tim                  ! time counter
-  REAL(KIND=4), DIMENSION(:),     ALLOCATABLE :: e31d                 ! metric for full step
-  REAL(KIND=4)                                :: zpi, zomega, rau0sg  ! physical constant
-  REAL(KIND=4)                                :: zsps                 ! Missing value for salinity
+  REAL(KIND=4), DIMENSION(:,:,:), ALLOCATABLE :: ztemp, zsal, zwk     ! array to ead 2 layer of data
 
+  REAL(KIND=8), DIMENSION(:),     ALLOCATABLE :: dtim                 ! time counter
   REAL(KIND=8), DIMENSION(:,:),   ALLOCATABLE :: dun, dvn             ! velocity component and flx
   REAL(KIND=8), DIMENSION(:,:),   ALLOCATABLE :: drotn                ! curl of the velocity
   REAL(KIND=8), DIMENSION(:,:),   ALLOCATABLE :: d2fcor               ! coriolis term at T point
@@ -71,23 +74,27 @@ PROGRAM cdfpvor
   REAL(KIND=8), DIMENSION(:,:),   ALLOCATABLE :: dareat               ! area of T cells
 
   CHARACTER(LEN=256)                          :: cf_tfil              ! input T file
+  CHARACTER(LEN=256)                          :: cf_sfil              ! salinity file (option)
   CHARACTER(LEN=256)                          :: cf_ufil              ! input U file
   CHARACTER(LEN=256)                          :: cf_vfil              ! input V file
   CHARACTER(LEN=256)                          :: cf_out='pvor.nc'     ! output file
+  CHARACTER(LEN=256)                          :: cf_e3w               ! file holding e3w in the case of vvl
   CHARACTER(LEN=256)                          :: cldum                ! dummy character variable
 
   TYPE(variable), DIMENSION(:), ALLOCATABLE   :: stypvar              ! structure for attribute
 
-  LOGICAL                                     :: lfull  = .FALSE.     ! flag for full step
-  LOGICAL                                     :: lertel = .TRUE.      ! flag for large scale pv
-  LOGICAL                                     :: lchk   = .FALSE.     ! flag for missing files
-  LOGICAL                                     :: lnc4   = .FALSE.     ! flag for netcdf4 chunking and deflation
+  LOGICAL                                     :: lfull   = .FALSE.    ! flag for full step
+  LOGICAL                                     :: lertel  = .TRUE.     ! flag for large scale pv
+  LOGICAL                                     :: lchk    = .FALSE.    ! flag for missing files
+  LOGICAL                                     :: lnc4    = .FALSE.    ! flag for netcdf4 chunking and deflation
+  LOGICAL                                     :: l_metric= .TRUE.     ! flag for netcdf4 chunking and deflation
   !!----------------------------------------------------------------------
   CALL ReadCdfNames()
 
   narg= iargc()
-  IF ( narg < 2 ) THEN
-     PRINT *,' usage : cdfpvor T-file  U-file V-file [-full] [-lspv ] [-nc4] [-o output file]'
+  IF ( narg == 0 ) THEN
+     PRINT *,' usage : cdfpvor -t T-file  -u U-file -v V-file [-s S-file] [-full] [-lspv] ...'
+     PRINT *,'           ... [-nometric] [-o OUT-file] [-nc4] [-vvl W-file]'
      PRINT *,'      '
      PRINT *,'     PURPOSE :'
      PRINT *,'       Compute the Ertel potential vorticity and save the relative  ' 
@@ -96,17 +103,23 @@ PROGRAM cdfpvor
      PRINT *,'       With -lspv option, compute only Qstretch or Large Scale P V '
      PRINT *,'      '
      PRINT *,'     ARGUMENTS :'
-     PRINT *,'       T-file : netcdf file for temperature and salinity.           ' 
-     PRINT *,'       U-file : netcdf file for zonal component of the velocity.    '
-     PRINT *,'       V-file : netcdf file for meridional component of the velocity.'
+     PRINT *,'       -t T-file : netcdf file for temperature and salinity.           ' 
+     PRINT *,'            If salinity not in T-file use -s option.'
+     PRINT *,'       -u U-file : netcdf file for zonal component of the velocity.    '
+     PRINT *,'       -v V-file : netcdf file for meridional component of the velocity.'
      PRINT *,'      '
      PRINT *,'     OPTIONS :'
+     PRINT *,'       [-s S-file] : specify salinity file if not T-file.'
      PRINT *,'       [-full ] : indicate a full step configuration.                ' 
      PRINT *,'       [-lspv ] : calculate only the large scale potential vorticity.'
      PRINT *,'                  ( replace the old cdflspv tool).'
      PRINT *,'                  If used only T-file is required, no need for velocities.'
      PRINT *,'       [-nc4 ] :  use netcdf4 with chunking and deflation '
-     PRINT *,'       [-o output file ] : use output file instead of default ',TRIM(cf_out)
+     PRINT *,'       [-o OUT-file] : use output file instead of default ',TRIM(cf_out)
+     PRINT *,'       [-vvl W-file] : use time-varying vertical metrics. W-file holds the'
+     PRINT *,'                  time-varying e3w vertical metrics.'
+     PRINT *,'       [-nometric] : Do not use metrics for computing the vorticity.'
+     PRINT *,'                 Assume all metrics to be 1, arbitrary.'
      PRINT *,'      '
      PRINT *,'     REQUIRED FILES :'
      PRINT *,'       ', TRIM(cn_fhgr),' and ',TRIM(cn_fzgr)
@@ -120,47 +133,62 @@ PROGRAM cdfpvor
      PRINT *,'           '
      PRINT *,'       With option -lspv :'
      PRINT *,'       netcdf file : lspv.nc'
-     PRINT *,'         variables :  volspv  (1.e-7 kg.m-4.s-1 ) large scale potential vorticity'
-     PRINT *,'                  LSPV is  located at W points.'
+     PRINT *,'         variables :  volspv  (1.e-7 kg.m-4.s-1 ) large scale potential '
+     PRINT *,'              vorticity. LSPV is located at W points.'
      PRINT *,'      '
      PRINT *,'     SEE ALSO :'
      PRINT *,'       cdfcurl ( compute only the curl on 1 level)'
      PRINT *,'      '
-     STOP
+     STOP 
   ENDIF
 
-  ijarg = 1  ; ireq = 0
+  ijarg = 1 
+  cf_sfil = 'none'
   DO WHILE ( ijarg <= narg )
      CALL getarg( ijarg, cldum ) ; ijarg = ijarg + 1
-
      SELECT CASE ( cldum )
-     CASE ( '-full' ) ; lfull  = .TRUE.
-     CASE ( '-lspv' ) ; lertel = .FALSE. ; nvar = 1 ; cf_out = 'lspv.nc'
-     CASE ( '-nc4'  ) ; lnc4 = .TRUE.
-     CASE ( '-o'    ) ; CALL getarg( ijarg, cf_out ) ; ijarg = ijarg + 1
-     CASE DEFAULT
-        ireq=ireq+1
-        SELECT CASE ( ireq )
-        CASE ( 1 ) ; cf_tfil = cldum
-        CASE ( 2 ) ; cf_ufil = cldum
-        CASE ( 3 ) ; cf_vfil = cldum
-        CASE DEFAULT
-           PRINT *,' Too many arguments '; STOP 99
-        END SELECT
+     CASE ( '-t'      ) ; CALL getarg( ijarg, cf_tfil) ; ijarg = ijarg + 1
+     CASE ( '-u'      ) ; CALL getarg( ijarg, cf_ufil) ; ijarg = ijarg + 1
+     CASE ( '-v'      ) ; CALL getarg( ijarg, cf_vfil) ; ijarg = ijarg + 1
+      ! options
+     CASE ( '-s'      ) ; CALL getarg( ijarg, cf_sfil) ; ijarg = ijarg + 1
+     CASE ( '-full'   ) ; lfull  = .TRUE.
+     CASE ( '-lspv'   ) ; lertel = .FALSE. ; nvar = 1  ; cf_out = 'lspv.nc'
+     CASE ( '-nc4'    ) ; lnc4   = .TRUE.
+     CASE ( '-o'      ) ; CALL getarg( ijarg, cf_out ) ; ijarg = ijarg + 1
+     CASE ( '-vvl'    ) ; lg_vvl = .TRUE.
+        ;                 CALL getarg( ijarg, cf_e3w ) ; ijarg = ijarg + 1
+     CASE ('-nometric') ; l_metric   = .FALSE.
+        ;               ; cf_out = 'pvor_grid.nc'
+     CASE DEFAULT       ;  PRINT *,' ERROR : ', TRIM(cldum),' : unknown option.' ; STOP 99
      END SELECT
   END DO
 
-  lchk = lchk .OR. chkfile( cn_fzgr)
-  lchk = lchk .OR. chkfile( cn_fhgr)
+  IF ( cf_sfil == 'none' ) cf_sfil=cf_tfil
+
+  IF ( l_metric ) THEN
+    lchk = lchk .OR. chkfile( cn_fzgr)
+    lchk = lchk .OR. chkfile( cn_fhgr)
+  ENDIF
+
   lchk = lchk .OR. chkfile( cf_tfil)
+  lchk = lchk .OR. chkfile( cf_sfil)
   IF ( lertel ) THEN
      lchk = lchk .OR. chkfile( cf_ufil)
      lchk = lchk .OR. chkfile( cf_vfil)
   ENDIF
-  IF ( lchk ) STOP 99 ! missing file
+  IF ( lg_vvl ) THEN
+     lchk = lchk .OR. chkfile( cf_e3w )
+  ENDIF
+  IF ( lchk   ) STOP 99 ! missing file
 
   ! Look for MissingValue for salinity
-  zsps = getspval(cf_tfil, cn_vosaline)
+  zsps = getspval(cf_sfil, cn_vosaline)
+
+  IF ( lg_vvl ) THEN
+     cn_fe3w = cf_e3w
+     cn_ve3w = cn_ve3wvvl
+  ENDIF
 
   npiglo = getdim (cf_tfil, cn_x)
   npjglo = getdim (cf_tfil, cn_y)
@@ -176,7 +204,7 @@ PROGRAM cdfpvor
   ALLOCATE ( e2v(npiglo,npjglo),   e2t(npiglo,npjglo)     )
   ALLOCATE ( gphit(npiglo,npjglo), d2fcor(npiglo,npjglo)  )
   ALLOCATE ( tmask(npiglo,npjglo)                         )
-  ALLOCATE ( gdepw(npk), tim(npt)                         )
+  ALLOCATE ( gdepw(npk), dtim(npt)                        )
 
   ALLOCATE ( dstretch(npiglo,npjglo)                      )
   IF ( lertel ) THEN
@@ -186,11 +214,19 @@ PROGRAM cdfpvor
   ENDIF
   IF ( lfull ) ALLOCATE ( e31d(npk)                       )
 
-  e1u   = getvar(cn_fhgr, cn_ve1u,  1, npiglo, npjglo)
-  e1t   = getvar(cn_fhgr, cn_ve1t,  1, npiglo, npjglo)
-  e2v   = getvar(cn_fhgr, cn_ve2v,  1, npiglo, npjglo)
-  e2t   = getvar(cn_fhgr, cn_ve2t,  1, npiglo, npjglo)
-  gphit = getvar(cn_fhgr, cn_gphit, 1, npiglo, npjglo)
+  IF ( l_metric ) THEN
+    e1u   = getvar(cn_fhgr, cn_ve1u,  1, npiglo, npjglo)
+    e1t   = getvar(cn_fhgr, cn_ve1t,  1, npiglo, npjglo)
+    e2v   = getvar(cn_fhgr, cn_ve2v,  1, npiglo, npjglo)
+    e2t   = getvar(cn_fhgr, cn_ve2t,  1, npiglo, npjglo)
+    gphit = getvar(cn_fhgr, cn_gphit, 1, npiglo, npjglo)
+  ELSE
+    e1u   = 1.
+    e1t   = 1.
+    e2v   = 1.
+    e2t   = 1.
+    gphit = 10.  ! will give a constant value for f (10 deg N, arbitrary)
+  ENDIF
 
   rau0sg      = 1020./9.81
   zpi         = ACOS(-1.)
@@ -198,7 +234,7 @@ PROGRAM cdfpvor
   d2fcor(:,:) = 2.d0 * zomega * SIN(gphit(:,:)*zpi/180.0)
 
   IF ( lertel ) THEN
-    dareat(:,:) = 4.d0 * e1t(:,:) * e2t(:,:)  ! factor of 4 to normalize relative vorticity
+     dareat(:,:) = 4.d0 * e1t(:,:) * e2t(:,:)  ! factor of 4 to normalize relative vorticity
   ENDIF
 
   gdepw(:) = getvare3(cn_fzgr, cn_gdepw, npk)
@@ -209,51 +245,25 @@ PROGRAM cdfpvor
   ALLOCATE ( stypvar(nvar), ipk(nvar), id_varout(nvar)   )
 
   ! create output fileset
+  CALL CreateOutput
 
-  ipk(:)= npk                   ! Those three variables are  3D
-  stypvar%cunits            = '1.e-7 kg.m-4.s-1'
-  stypvar%rmissing_value    = 0.
-  stypvar%valid_min         = -10000.
-  stypvar%valid_max         = 10000.
-  stypvar%conline_operation = 'N/A'
-  stypvar%caxis             = 'TZYX'
-
-  DO ji = 1, nvar
-   stypvar(ji)%ichunk = (/npiglo,MAX(1,npjglo/30), 1, 1 /)
-  ENDDO
-
-  IF (lertel ) THEN
-     ! define variable name and attribute
-     stypvar(1)%cname = 'vorelvor' ; stypvar(1)%clong_name = 'Relative_component_of_Ertel_PV'
-     stypvar(2)%cname = 'vostrvor' ; stypvar(2)%clong_name = 'Stretching_component_of_Ertel_PV'
-     stypvar(3)%cname = 'vototvor' ; stypvar(3)%clong_name = 'Ertel_potential_vorticity'
-
-     stypvar(1)%cshort_name = 'vorelvor'
-     stypvar(2)%cshort_name = 'vostrvor'
-     stypvar(3)%cshort_name = 'vototvor'
-
-     ncout = create      (cf_out, cf_tfil, npiglo, npjglo, npk       )
-     ierr  = createvar   (ncout,  stypvar, nvar,   ipk,    id_varout )
-     ierr  = putheadervar(ncout,  cf_tfil, npiglo, npjglo, npk       )
-  ELSE
-     stypvar(1)%cname = 'volspv' ; stypvar(1)%clong_name = 'Large Scale Potential_vorticity'
-     stypvar(1)%cshort_name = 'volspv'
-     ncout = create      (cf_out, cf_tfil, npiglo, npjglo, npk, cdep=TRIM(cn_vdepthw) )
-     ierr  = createvar   (ncout,  stypvar, nvar,   ipk,    id_varout                  )
-     ierr  = putheadervar(ncout,  cf_tfil, npiglo, npjglo, npk, pdep=gdepw            )
+  IF ( lfull  ) THEN
+    IF ( l_metric ) THEN
+      e31d = getvare3( cn_fzgr, cn_ve3w1d, npk )
+    ELSE
+      e31d = 1.
+    ENDIF
   ENDIF
 
-  IF ( lfull ) e31d = getvare3( cn_fzgr, cn_ve3w1d, npk )
-
-  tim  = getvar1d(cf_ufil, cn_vtimec, npt      )
-  ierr = putvar1d(ncout,   tim,       npt, 'T' )
-
   DO jt=1,npt
+     IF ( lg_vvl ) THEN ; it=jt
+     ELSE               ; it=1
+     ENDIF
      !  2 levels of T and S are required : iup,idown (with respect to W level)
      !  Compute from bottom to top (for vertical integration)
-     PRINT *,'time=',jt,'(days:',tim(jt)/86400.,')'
+     PRINT *,'time=',jt,'(days:',dtim(jt)/86400.,')'
      ztemp(:,:,idown) = getvar(cf_tfil, cn_votemper, npk-1, npiglo, npjglo, ktime=jt)
-     zsal( :,:,idown) = getvar(cf_tfil, cn_vosaline, npk-1, npiglo, npjglo, ktime=jt)
+     zsal( :,:,idown) = getvar(cf_sfil, cn_vosaline, npk-1, npiglo, npjglo, ktime=jt)
 
      ! -------------------------------- LOOP OVER LEVELS
      DO jk = npk-1, 1, -1 
@@ -269,10 +279,10 @@ PROGRAM cdfpvor
            DO jj = 2, npjglo -1 
               DO ji = 2, npiglo -1    
                  drotn(ji,jj) = ( dun(ji-1,jj-1)  + dun(ji,jj-1)  &
-                      &        -dun(ji-1,jj+1)  - dun(ji,jj+1)  &
-                      &        -dvn(ji-1,jj-1)  - dvn(ji-1,jj)  &
-                      &        +dvn(ji+1,jj-1)  + dvn(ji+1,jj)) &
-                      / dareat(ji,jj) 
+                        &        -dun(ji-1,jj+1)  - dun(ji,jj+1)  &
+                        &        -dvn(ji-1,jj-1)  - dvn(ji-1,jj)  &
+                        &        +dvn(ji+1,jj-1)  + dvn(ji+1,jj)) &
+                        / dareat(ji,jj) 
               END DO
            END DO
         ENDIF
@@ -281,13 +291,16 @@ PROGRAM cdfpvor
         IF ( jk > 1) THEN 
            tmask(:,:)=1.
            ztemp(:,:,iup) = getvar(cf_tfil, cn_votemper, jk-1 ,npiglo, npjglo, ktime=jt)
-           zsal(:,:,iup)  = getvar(cf_tfil, cn_vosaline, jk-1 ,npiglo, npjglo, ktime=jt)
-           WHERE(zsal(:,:,idown) == zsps ) tmask = 0.
-           IF ( lfull ) THEN
-              e3w(:,:) = e31d(jk)
-           ELSE
-              e3w(:,:) = getvar(cn_fzgr, 'e3w_ps', jk, npiglo, npjglo ,ldiom=.TRUE.)
+           zsal(:,:,iup)  = getvar(cf_sfil, cn_vosaline, jk-1 ,npiglo, npjglo, ktime=jt)
+           WHERE(zsal(:,:,idown) == zsps ) tmask = 0
+
+           IF ( l_metric ) THEN
+              IF ( lfull ) THEN ; e3w(:,:) = e31d(jk)
+              ELSE              ; e3w(:,:) = getvar(cn_fe3w, cn_ve3w, jk, npiglo, npjglo, ktime=it, ldiom=.NOT.lg_vvl )
+              ENDIF
+           ELSE                 ; e3w(:,:) = 1.
            ENDIF
+              
 
            WHERE (e3w == 0 ) e3w = 1.
 
@@ -296,7 +309,7 @@ PROGRAM cdfpvor
            IF ( lertel ) THEN ! put zn2 at T level (k )
               WHERE ( zwk(:,:,idown) == 0 ) 
                  zn2(:,:) =  zwk(:,:,iup)
-              ELSEWHERE
+              ELSEWHERE                     
                  zn2(:,:) = 0.5 * ( zwk(:,:,iup) + zwk(:,:,idown) ) * tmask(:,:)
               END WHERE
            ELSE               ! keep bn2 at w points
@@ -338,4 +351,53 @@ PROGRAM cdfpvor
 
   ierr = closeout(ncout)
 
+CONTAINS
+
+  SUBROUTINE CreateOutput
+    !!---------------------------------------------------------------------
+    !!                  ***  ROUTINE CreateOutput  ***
+    !!
+    !! ** Purpose :  Create netcdf output file(s) 
+    !!
+    !! ** Method  :  Use stypvar global description of variables
+    !!
+    !!----------------------------------------------------------------------
+    ipk(:)= npk                   ! Those three variables are  3D
+    stypvar%cunits            = '1.e-7 kg.m-4.s-1'
+    stypvar%rmissing_value    = 0.
+    stypvar%valid_min         = -10000.
+    stypvar%valid_max         = 10000.
+    stypvar%conline_operation = 'N/A'
+    stypvar%caxis             = 'TZYX'
+
+    DO ji = 1, nvar
+       stypvar(ji)%ichunk = (/npiglo,MAX(1,npjglo/30), 1, 1 /)
+    ENDDO
+
+    IF (lertel ) THEN
+       ! define variable name and attribute
+       stypvar(1)%cname = 'vorelvor' ; stypvar(1)%clong_name = 'Relative_component_of_Ertel_PV'
+       stypvar(2)%cname = 'vostrvor' ; stypvar(2)%clong_name = 'Stretching_component_of_Ertel_PV'
+       stypvar(3)%cname = 'vototvor' ; stypvar(3)%clong_name = 'Ertel_potential_vorticity'
+
+       stypvar(1)%cshort_name = 'vorelvor'
+       stypvar(2)%cshort_name = 'vostrvor'
+       stypvar(3)%cshort_name = 'vototvor'
+
+       ncout = create      (cf_out, cf_tfil, npiglo, npjglo, npk       , ld_nc4=lnc4 )
+       ierr  = createvar   (ncout,  stypvar, nvar,   ipk,    id_varout , ld_nc4=lnc4 )
+       ierr  = putheadervar(ncout,  cf_tfil, npiglo, npjglo, npk       )
+    ELSE
+       stypvar(1)%cname = 'volspv' ; stypvar(1)%clong_name = 'Large Scale Potential_vorticity'
+       stypvar(1)%cshort_name = 'volspv'
+       ncout = create      (cf_out, cf_tfil, npiglo, npjglo, npk, cdep=TRIM(cn_vdepthw) , ld_nc4=lnc4 )
+       ierr  = createvar   (ncout,  stypvar, nvar,   ipk,    id_varout                  , ld_nc4=lnc4 )
+       ierr  = putheadervar(ncout,  cf_tfil, npiglo, npjglo, npk, pdep=gdepw            )
+    ENDIF
+
+    dtim = getvar1d(cf_ufil, cn_vtimec, npt      )
+    ierr = putvar1d(ncout,   dtim,      npt, 'T' )
+
+  END SUBROUTINE CreateOutput
+
 END PROGRAM cdfpvor
