@@ -16,15 +16,17 @@ PROGRAM cdfsigintegr_bottom
   !!
   !! History : 2.1  : 12/2007  : J.M. Molines : Original code
   !!           3.0  : 06/2011  : J.M. Molines : Doctor norm + Lic.
+  !!         : 4.0  : 03/2017  : J.M. Molines  
   !!----------------------------------------------------------------------
   USE cdfio
   USE modcdfnames
   USE modutils
   !!----------------------------------------------------------------------
-  !! CDFTOOLS_3.0 , MEOM 2011
+  !! CDFTOOLS_4.0 , MEOM 2017 
   !! $Id: cdfsigintegr.f90 657 2013-05-21 16:36:14Z molines $
-  !! Copyright (c) 2011, J.-M. Molines
+  !! Copyright (c) 2017, J.-M. Molines 
   !! Software governed by the CeCILL licence (Licence/CDFTOOLSCeCILL.txt)
+  !! @class integration
   !!----------------------------------------------------------------------
   IMPLICIT NONE
 
@@ -44,22 +46,22 @@ PROGRAM cdfsigintegr_bottom
   INTEGER(KIND=4)                               :: ncout, ierr      ! ncid and status variable
   INTEGER(KIND=4), DIMENSION(4)                 :: ipk, id_varout   ! levels and id's of output variables
   !
-  REAL(KIND=4), DIMENSION(:,:,:),   ALLOCATABLE :: v3d              ! 3D working array (npk)
-  REAL(KIND=4), DIMENSION(:,:,:),   ALLOCATABLE :: zint             ! pseudo 3D working array (2)
+  REAL(KIND=4)                                  :: zspval=999999.   ! output missing value
+  REAL(KIND=4)                                  :: maskvalue=0.     ! masked values for ocean velocity output
+  REAL(KIND=4)                                  :: zspvalz          ! missing value from rho file      
+  REAL(KIND=4), DIMENSION(:),       ALLOCATABLE :: rho_lev          ! value of isopycnals
+  REAL(KIND=4), DIMENSION(:),       ALLOCATABLE :: h1d              ! depth of rho points
+  REAL(KIND=4), DIMENSION(:),       ALLOCATABLE :: gdepw            ! depth of W points
+  REAL(KIND=4), DIMENSION(:),       ALLOCATABLE :: e31d             ! vertical metrics in full step
   REAL(KIND=4), DIMENSION(:,:),     ALLOCATABLE :: v2d              ! 2D working array
   REAL(KIND=4), DIMENSION(:,:),     ALLOCATABLE :: e3               ! vertical metrics
   REAL(KIND=4), DIMENSION(:,:),     ALLOCATABLE :: eu               ! T horizontal metrics
   REAL(KIND=4), DIMENSION(:,:),     ALLOCATABLE :: tmask            ! mask of t points from rho
   REAL(KIND=4), DIMENSION(:,:),     ALLOCATABLE :: zdum             ! dummy array for I/O
-  REAL(KIND=4), DIMENSION(:),       ALLOCATABLE :: rho_lev          ! value of isopycnals
-  REAL(KIND=4), DIMENSION(:),       ALLOCATABLE :: tim              ! time counter
-  REAL(KIND=4), DIMENSION(:),       ALLOCATABLE :: h1d              ! depth of rho points
-  REAL(KIND=4), DIMENSION(:),       ALLOCATABLE :: gdepw            ! depth of W points
-  REAL(KIND=4), DIMENSION(:),       ALLOCATABLE :: e31d             ! vertical metrics in full step
-  REAL(KIND=4)                                  :: zspval=999999.   ! output missing value
-  REAL(KIND=4)                                  :: maskvalue=0.     ! masked values for ocean velocity output
-  REAL(KIND=4)                                  :: zspvalz          ! missing value from rho file      
+  REAL(KIND=4), DIMENSION(:,:,:),   ALLOCATABLE :: v3d              ! 3D working array (npk)
+  REAL(KIND=4), DIMENSION(:,:,:),   ALLOCATABLE :: zint             ! pseudo 3D working array (2)
 
+  REAL(KIND=8), DIMENSION(:),       ALLOCATABLE :: dtim             ! time counter
   REAL(KIND=8), DIMENSION(:,:,:),   ALLOCATABLE :: dv2dint          ! interpolated value 
   REAL(KIND=8), DIMENSION(:,:,:),   ALLOCATABLE :: dalpha           ! 3D coefficient (npiso)
 
@@ -83,7 +85,7 @@ PROGRAM cdfsigintegr_bottom
   CALL ReadCdfNames()
 
   narg=iargc()
-  IF ( narg < 3 ) THEN
+  IF ( narg == 0 ) THEN
      PRINT *,' usage : cdfsigintegr_bottom IN-var RHO-file list_of_files [ VAR-type ] ...'
      PRINT *,'              ... [ -sig sigma_name] [ -full ] '
      PRINT *,'      '
@@ -125,7 +127,7 @@ PROGRAM cdfsigintegr_bottom
      PRINT *,'     SEE ALSO :'
      PRINT *,'      cdfrhoproj, cdfsigtrp, cdfisopycdep'
      PRINT *,'      '
-     STOP
+     STOP 
   ENDIF
 
   ijarg = 1 ; ireq = 0 ; nfiles = 0
@@ -331,10 +333,10 @@ PROGRAM cdfsigintegr_bottom
 
      ! copy time arrays in output file
      npt = getdim ( cf_in, cn_t)
-     ALLOCATE ( tim(npt) )
-     tim(:) = getvar1d(cf_in, cn_vtimec, npt     )
-     ierr   = putvar1d(ncout, tim,       npt, 'T')
-     DEALLOCATE ( tim )
+     ALLOCATE ( dtim(npt) )
+     dtim = getvar1d(cf_in, cn_vtimec, npt     )
+     ierr = putvar1d(ncout, dtim,      npt, 'T')
+     DEALLOCATE ( dtim )
 
      DO jt =1, npt
         DO jk=1,npk
