@@ -3,22 +3,15 @@ PROGRAM cdfsigtrp_broken
   !!                     ***  PROGRAM  cdfsigtrp_broken  ***
   !!======================================================================
   !!  ** Purpose : Compute density class Mass transport across a section.
-  !!               prviously extracted with cdf_xtrac_broken_line
+  !!               previously extracted with cdf_xtrac_broken_line
   !!           
-  !!  ** Method  :- The begining and end point of the section are given in 
-  !!                term of f-points index.
-  !!              - The program works for zonal or meridional sections.
-  !!              - The section definitions are given in an ASCII FILE 
-  !!                dens_section.dat:
-  !!                 foreach sections, 2 lines :
-  !!                       (i) : section name (String, no blank)
-  !!                      (ii) : imin imax jmin jmax for the section
-  !!              - Only vertical slices corrsponding to the sections are
-  !!                read in the files.
+  !!  ** Method  : 
+  !!              - The program works for sections extracted with cdf_xtrac_brokenline
+  !!              - Hence it is considered as a pseudo zonal section with 
+  !!                normal velocity seen as V
   !!              - read metrics, depth, etc
-  !!              - read normal velocity (either vozocrtx oy vomecrty )
-  !!              - read 2 rows of T and S ( i i+1  or j j+1 )
-  !!              - compute the mean value at velocity point
+  !!              - read normal velocity 
+  !!              - read T and S already interpolated at V point
   !!              - compute sigma0 (can be easily modified for sigmai )
   !!              - compute the depths of isopyncal surfaces
   !!              - compute the transport from surface to the isopycn
@@ -26,22 +19,23 @@ PROGRAM cdfsigtrp_broken
   !!              - compute the total transport (for information)
   !!
   !! History : 3.0  : 12/2014  : P. Colombo   : from cdfsigtrp 
+  !!         : 4.0  : 03/2017  : J.M. Molines  
   !!----------------------------------------------------------------------
   !!----------------------------------------------------------------------
   !!   routines      : description
   !!  section_init   : initialize section names and positions
   !!  print_out      : routine which performs standard output if required
-  !!  bimg_writ      : routine which performs bimg output if required
   !!----------------------------------------------------------------------
   USE cdfio
   USE eos          ! for sigma0, sigmai
   USE modcdfnames  ! for ReadCdfNames
   USE modutils     ! for SetGlobalAtt
   !!----------------------------------------------------------------------
-  !! CDFTOOLS_3.0 , MEOM 2011
+  !! CDFTOOLS_4.0 , MEOM 2017 
   !! $Id: cdfsigtrp.f90 699 2013-06-24 14:17:21Z molines $
-  !! Copyright (c) 2011, J.-M. Molines
+  !! Copyright (c) 2017, J.-M. Molines 
   !! Software governed by the CeCILL licence (Licence/CDFTOOLSCeCILL.txt)
+  !! @class transport
   !!----------------------------------------------------------------------
   IMPLICIT NONE
 
@@ -50,9 +44,8 @@ PROGRAM cdfsigtrp_broken
   INTEGER(KIND=4)                               :: nbins                ! number of density classes
   INTEGER(KIND=4)                               :: ipos                 ! working variable
   INTEGER(KIND=4)                               :: narg, iargc          ! command line 
-  INTEGER(KIND=4)                               :: ijarg, ireq          ! command line
+  INTEGER(KIND=4)                               :: ijarg                ! command line
   INTEGER(KIND=4)                               :: npk, nk              ! vertical size, number of wet layers
-  INTEGER(KIND=4)                               :: numbimg=10           ! optional bimg logical unit
   INTEGER(KIND=4)                               :: numout=11            ! ascii output
   INTEGER(KIND=4)                               :: nsection             ! number of sections (overall)
   INTEGER(KIND=4)                               :: iimin, iimax         ! working section limits
@@ -67,7 +60,6 @@ PROGRAM cdfsigtrp_broken
   INTEGER(KIND=4), DIMENSION(:),    ALLOCATABLE :: ipk, id_varout       ! variable levels and id
 
   REAL(KIND=4)                                  :: refdep =0.e0         ! reference depth (m)
-  REAL(KIND=4), DIMENSION(1)                    :: tim                  ! time counter
   REAL(KIND=4), DIMENSION(1)                    :: rdummy1, rdummy2     ! working variable
   REAL(KIND=4), DIMENSION(:),       ALLOCATABLE :: gdept, gdepw         ! depth of T and W points 
   REAL(KIND=4), DIMENSION(:),       ALLOCATABLE :: eu                   ! either e1v or e2u
@@ -83,6 +75,7 @@ PROGRAM cdfsigtrp_broken
   REAL(KIND=8)                                  :: dsigma_min           ! minimum density for bining
   REAL(KIND=8)                                  :: dsigma_max, dltsig   ! maximum density for bining, step
   REAL(KIND=8)                                  :: dsigma, dalfa        ! working sigma, interpolation coeff.
+  REAL(KIND=8), DIMENSION(1)                    :: dtim                 ! time counter
   REAL(KIND=8), DIMENSION(:),       ALLOCATABLE :: dsigma_lev           ! built array with sigma levels
   REAL(KIND=8), DIMENSION(:,:),     ALLOCATABLE :: de3                  ! vertical metrics
   REAL(KIND=8), DIMENSION(:,:),     ALLOCATABLE :: ddepu                ! depth of vel points
@@ -94,10 +87,9 @@ PROGRAM cdfsigtrp_broken
   TYPE(variable), DIMENSION(:),     ALLOCATABLE :: stypvar              ! structure of output
 
   CHARACTER(LEN=256)                            :: cf_brfi              ! temperature salinity velocity file 
-  !(from cdf_xtrac_brokenline)
+                                                                        !(from cdf_xtrac_brokenline)
   CHARACTER(LEN=256)                            :: cf_section='dens_section.dat'  ! input section file
   CHARACTER(LEN=256)                            :: cf_out='trpsig.txt'  ! output  ascii file
-  CHARACTER(LEN=256)                            :: cf_bimg              ! output bimg file (2d)
   CHARACTER(LEN=256)                            :: cf_nc                ! output netcdf file (2d)
   CHARACTER(LEN=256)                            :: cf_outnc             ! output netcdf file (1d, 0d))
   CHARACTER(LEN=256)                            :: cv_dep               ! depth variable
@@ -117,7 +109,6 @@ PROGRAM cdfsigtrp_broken
 
   LOGICAL                                       :: ltemp  =.FALSE.      ! flag for use of temperature
   LOGICAL                                       :: lprint =.FALSE.      ! flag for extra print
-  LOGICAL                                       :: lbimg  =.FALSE.      ! flag for bimg output
   LOGICAL                                       :: lncdf  =.FALSE.      ! flag for extra netcdf output
   LOGICAL                                       :: lfull  =.FALSE.      ! flag for full step 
   LOGICAL                                       :: lneutral  =.FALSE.   ! flag for neutral density
@@ -126,9 +117,9 @@ PROGRAM cdfsigtrp_broken
   CALL ReadCdfNames()
 
   narg= iargc()
-  IF ( narg < 4 ) THEN
-     PRINT *,' usage :  cdfsigtrp_broken TSV-file sigma_min sigma_max nbins ...'
-     PRINT *,'              ... [-print ] [-bimg ] [-full ] [ -refdep ref_depth] ...'
+  IF ( narg == 0 ) THEN
+     PRINT *,' usage :  cdfsigtrp_broken -f BRK-file -bin sigma_min sigma_max nbins ...'
+     PRINT *,'              ... [-print ] [-full ] [ -refdep ref_depth] ...'
      PRINT *,'              ... [-neutral ] [-section file ] [-temp ]'
      PRINT *,'      '
      PRINT *,'     PURPOSE :'
@@ -152,30 +143,28 @@ PROGRAM cdfsigtrp_broken
      PRINT *,'       sigma_min, sigma_max as temperatures.'
      PRINT *,'      '
      PRINT *,'     ARGUMENTS :'
-     PRINT *,'       TSV-file : netcdf_broken_line file with temperature, salinity' 
-     PRINT *,'       and the normal velocity through the section'
-     PRINT *,'       sigma_min : minimum density for binning'
-     PRINT *,'       sigma_max : maximum density for binning'
-     PRINT *,'       nbins : number of bins. This will fix the bin ''width'' '
+     PRINT *,'       -f BRK-file : netcdf_broken_line file with temperature, salinity' 
+     PRINT *,'                   and the normal velocity through the section + some '
+     PRINT *,'                   usefull metrics.'
+     PRINT *,'       -bin  sigma_min sigma_max nbins : 3 values follow -bin switch :'
+     PRINT *,'               sigma_min : minimum density for binning'
+     PRINT *,'               sigma_max : maximum density for binning'
+     PRINT *,'               nbins : number of bins. This will fix the bin ''width'' '
      PRINT *,'      '
      PRINT *,'     OPTIONS :'
-     PRINT *,'       [ -full ] : for full step configuration' 
-     PRINT *,'       [ -bimg ] : produce extra bimg output file which shows the details'
-     PRINT *,'               of the sections (normal velocity, density, temperature, '
-     PRINT *,'               salinity, transports, isopycnal depths. (to be change to '
-     PRINT *,'               netcdf files for more common use.'
-     PRINT *,'       [ -ncdf ] : produce extra netcdf output file which shows the details'
+     PRINT *,'       [-full ] : for full step configuration' 
+     PRINT *,'       [-ncdf ] : produce extra netcdf output file which shows the details'
      PRINT *,'               of the sections (normal velocity, density, temperature, '
      PRINT *,'               salinity, transports, isopycnal depths. '
-     PRINT *,'       [ -print ]: write the binned transports on standard output, for each'
+     PRINT *,'       [-print ]: write the binned transports on standard output, for each'
      PRINT *,'               sections.'
-     PRINT *,'       [ -refdep ref_depth ]: give a reference depths for the computation of'
+     PRINT *,'       [-refdep ref_depth ]: give a reference depths for the computation of'
      PRINT *,'               potential density. Sigma_min, sigma_max must be adapted '
      PRINT *,'               accordingly.'
-     PRINT *,'       [ -neutral ]: use neutral density instead of potential density '
-     PRINT *,'       [ -section file] : give the name of section file.'
+     PRINT *,'       [-neutral ]: use neutral density instead of potential density '
+     PRINT *,'       [-section file] : give the name of section file.'
      PRINT *,'               Default is ', TRIM(cf_section)
-     PRINT *,'       [ -temp ] : use temperature instead of density for binning'
+     PRINT *,'       [-temp ] : use temperature instead of density for binning'
      PRINT *,'      '
      PRINT *,'     REQUIRED FILES :'
      PRINT *,'       ', TRIM(cn_fzgr),' and ', TRIM(cf_section)
@@ -188,49 +177,42 @@ PROGRAM cdfsigtrp_broken
      PRINT *,'      '
      PRINT *,'       ascii file  : ', TRIM(cf_out) 
      PRINT *,'      '
-     PRINT *,'       bimg  file  :  There are 2 bimg files whose name is build from section'
-     PRINT *,'         name : section_name_trpdep.bimg and section_name_trpsig.bimg.'
-     PRINT *,'         This file is written only if -bimg option is used.'
-     PRINT *,'      '
      PRINT *,'      Standard output : the results are written on standard output only if '
      PRINT *,'         the -print option is used.'
      PRINT *,'      '
      PRINT *,'     SEE ALSO :'
      PRINT *,'      cdfrhoproj, cdftransport, cdfsigintegr, cdfsigtrp, cdf_xtrac_brokenline '
      PRINT *,'      '
-     STOP
+     STOP 
   ENDIF
 
   ! browse command line
-  ijarg = 1 ; ireq = 0
+  ijarg = 1 
   DO WHILE ( ijarg <= narg )
      CALL getarg(ijarg, cldum ) ; ijarg=ijarg+1
      SELECT CASE ( cldum )
-     CASE ( '-full' ) ; lfull  = .TRUE.
-     CASE ( '-bimg' ) ; lbimg  = .TRUE.
-     CASE ( '-ncdf' ) ; lncdf  = .TRUE.
-     CASE ( '-print') ; lprint = .TRUE.
-     CASE ( '-temp')  ; ltemp  = .TRUE. 
+     CASE ( '-f'      ) ; CALL getarg(ijarg, cf_brfi    ) ; ijarg=ijarg+1 
+     CASE ( '-bin'    ) ; CALL getarg(ijarg, cldum      ) ; ijarg=ijarg+1 ; READ(cldum,*) dsigma_min 
+                        ; CALL getarg(ijarg, cldum      ) ; ijarg=ijarg+1 ; READ(cldum,*) dsigma_max
+                        ; CALL getarg(ijarg, cldum      ) ; ijarg=ijarg+1 ; READ(cldum,*) nbins
+     ! options
+     CASE ( '-full'   ) ; lfull  = .TRUE.
+     CASE ( '-ncdf'   ) ; lncdf  = .TRUE.
+     CASE ( '-print'  ) ; lprint = .TRUE.
+     CASE ( '-temp'   ) ; ltemp  = .TRUE. 
      CASE ( '-refdep' ) ; CALL getarg(ijarg, cldum      ) ; ijarg=ijarg+1 ; READ(cldum,*) refdep
      CASE ( '-section') ; CALL getarg(ijarg, cf_section ) ; ijarg=ijarg+1 
      CASE ( '-neutral') ; lneutral = .TRUE.
-     CASE DEFAULT
-        ireq=ireq+1
-        SELECT CASE ( ireq)
-        CASE ( 1 ) ; cf_brfi = cldum
-        CASE ( 2 ) ; READ(cldum,*) dsigma_min
-        CASE ( 3 ) ; READ(cldum,*) dsigma_max
-        CASE ( 4 ) ; READ(cldum,*) nbins
-        CASE DEFAULT 
-           PRINT *,' Too many arguments ' ; STOP 99
-        END SELECT
+     CASE DEFAULT       ; PRINT *,' ERROR : ', TRIM(cldum), ' : unknown option.' ; STOP 99
      END SELECT
   END DO
+
   ! check for file existence
   lchk = lchk .OR. chkfile( cn_fzgr    )
   lchk = lchk .OR. chkfile( cf_section )
   lchk = lchk .OR. chkfile( cf_brfi    )
   IF ( lchk ) STOP 99 ! missing file
+
   IF ( ltemp)  THEN  ! temperature decrease downward. Change sign and swap min/max
      refdep = -10. ! flag value
      dltsig     = dsigma_max  ! use dltsig as dummy variable for swapping
@@ -311,7 +293,6 @@ PROGRAM cdfsigtrp_broken
      ALLOCATE ( dwtrp(npts, nbins+1), dhiso(npts,nbins+1), dwtrpbin(npts,nbins) )
      ALLOCATE ( rlonlat(npts,1) )
 
-
      ! zonal section at j=ijmin=ijmax
      tmpz(:,:,1)  = getvar(cf_brfi, cn_ve1v,   1, npts, 1, kimin=iimin, kjmin=ijmin)
      eu(:)        = tmpz(:,1,1)
@@ -420,7 +401,6 @@ PROGRAM cdfsigtrp_broken
 
      ! output of the code for 1 section
      IF (lprint) CALL print_out(jsec)
-     IF (lbimg ) CALL bimg_writ(jsec)
      IF (lncdf ) CALL cdf_writ(jsec)
      PRINT *,' Total transport in all bins :',TRIM(csection(jsec)),' ',SUM(dtrpbin(jsec,:) )/1.d6
 
@@ -512,8 +492,8 @@ PROGRAM cdfsigtrp_broken
      ierr  = putheadervar(ncout,    cf_brfi, ikx,      iky, nbins, &
           &   pnavlon=rdumlon, pnavlat=rdumlat, pdep=REAL(dsigma_lev), cdep=cv_dep           )
 
-     tim  = getvar1d(cf_brfi, cn_vtimec, 1     )
-     ierr = putvar1d(ncout,   tim,       1, 'T')
+     dtim = getvar1d(cf_brfi, cn_vtimec, 1     )
+     ierr = putvar1d(ncout,   dtim,      1, 'T')
 
      DO jiso=1,nbins
         rdummy1 = dsigma_lev(jiso)
@@ -601,67 +581,6 @@ CONTAINS
 
   END SUBROUTINE section_init
 
-  SUBROUTINE bimg_writ( ksec)
-    !!---------------------------------------------------------------------
-    !!                  ***  ROUTINE bimg_writ  ***
-    !!
-    !! ** Purpose :  Write output bimg files if required 
-    !!
-    !! ** Method  :  Most of the variables are global 
-    !!
-    !!----------------------------------------------------------------------
-    INTEGER(KIND=4), INTENT(in) :: ksec  ! number of the section
-
-    INTEGER(KIND=4)             :: ji, jk
-    !!----------------------------------------------------------------------
-    ! (along section, depth ) 2D variables
-    cf_bimg=TRIM(csection(ksec))//'_trpdep.bimg'
-    OPEN(numbimg,FILE=cf_bimg,FORM='UNFORMATTED')
-    cldum=' 4 dimensions in this isopycnal file '
-    WRITE(numbimg) cldum
-
-    cldum=' 1: T ;  2: S ; 3: sigma ; 4: Velocity '
-    WRITE(numbimg) cldum
-
-    WRITE(cldum,'(a,4i5.4)') ' from '//TRIM(csection(ksec)), iimin,iimax,ijmin,ijmax
-    WRITE(numbimg) cldum
-
-    cldum=' file '//TRIM(cf_brfi)
-    WRITE(numbimg) cldum
-
-    WRITE(numbimg) npts,nk,1,1,4,0
-    WRITE(numbimg) 1.,-float(nk),1.,1., 0.
-    WRITE(numbimg) 0.
-    WRITE(numbimg) 0.
-
-    WRITE(numbimg) (( REAL(zt(ji,jk)  ), ji=1,npts), jk=nk,1,-1 ) ! temperature 
-    WRITE(numbimg) (( REAL(zs(ji,jk)  ), ji=1,npts), jk=nk,1,-1 ) ! salinity
-    WRITE(numbimg) (( REAL(dsig(ji,jk)), ji=1,npts), jk=nk,1,-1 ) ! density
-    WRITE(numbimg) (( REAL(zu(ji,jk)  ), ji=1,npts), jk=nk,1,-1 ) ! normal velocity
-    CLOSE(numbimg)
-
-    ! (along section, sigma ) 2D variables
-    cf_bimg=TRIM(csection(ksec))//'_trpsig.bimg'
-    OPEN(numbimg,FILE=cf_bimg,FORM='UNFORMATTED')
-    cldum=' 3 dimensions in this isopycnal file '
-    WRITE(numbimg) cldum
-    cldum=' 1: hiso ;  2: bin trp ; 3: cumulated  trp '
-    WRITE(numbimg) cldum
-    WRITE(cldum,'(a,4i5.4)') ' from '//TRIM(csection(ksec)), iimin,iimax,ijmin,ijmax
-    WRITE(numbimg) cldum
-    cldum=' file '//TRIM(cf_brfi)
-    WRITE(numbimg) cldum
-    WRITE(numbimg) npts,nbins,1,1,3,0
-    WRITE(numbimg) 1.,-REAL(dsigma_lev(nbins)),1.,REAL(dltsig), 0.
-    WRITE(numbimg) 0.
-    WRITE(numbimg) 0.
-    WRITE(numbimg) (( REAL(dhiso(ji,jiso)   ),      ji=1,npts), jiso=nbins,1,-1) ! isopyc depth
-    WRITE(numbimg) (( REAL(dwtrpbin(ji,jiso))/1.e6, ji=1,npts), jiso=nbins,1,-1) ! binned transport
-    WRITE(numbimg) (( REAL(dwtrp(ji,jiso)   )/1.e6, ji=1,npts), jiso=nbins,1,-1) ! cumulated transport
-    CLOSE(numbimg)
-
-  END SUBROUTINE bimg_writ
-
   SUBROUTINE cdf_writ( ksec)
     !!---------------------------------------------------------------------
     !!                  ***  ROUTINE cdf_writ  ***
@@ -745,8 +664,8 @@ CONTAINS
     ierr  = putheadervar(icout, cf_brfi,   npts, 1, nk, &
          &   pnavlon=rlonlat, pnavlat=rlonlat, cdep=cn_vdeptht                              )
 
-    !    tim  = getvar1d(cf_brfi, cn_vtimec, 1     )
-    !    ierr = putvar1d(icout,   tim,       1, 'T')
+    !    dtim = getvar1d(cf_brfi, cn_vtimec, 1     )
+    !    ierr = putvar1d(icout,   dtim,      1, 'T')
 
     DO jk = 1, nk
        zdum(:,1)=zt(:,jk)   ; ierr = putvar ( icout, id_varout(1), zdum, jk, npts, 1 )
@@ -876,8 +795,6 @@ CONTAINS
        PRINT  cfmt_9003, dsigma_lev(jbin),(dwtrpbin(ji,jbin)/1.d6,ji=1,npts), dtrpbin(ksec,jbin)/1.d6
     END DO
 
-
-
   END SUBROUTINE print_out
 
 END PROGRAM cdfsigtrp_broken
