@@ -16,6 +16,7 @@ PROGRAM cdfsmooth
   !!         : 2.1  : 05/2010  : R. Dussin    : Add shapiro filter
   !!           3.0  : 01/2011  : J.M. Molines : Doctor norm + Lic.
   !!           3.0  : 07/2011  : R. Dussin    : Add anisotropic box 
+  !!         : 4.0  : 03/2017  : J.M. Molines  
   !!----------------------------------------------------------------------
   !!----------------------------------------------------------------------
   !!   routines      : description
@@ -34,10 +35,11 @@ PROGRAM cdfsmooth
   USE modcdfnames
   USE modutils
   !!----------------------------------------------------------------------
-  !! CDFTOOLS_3.0 , MEOM 2011
+  !! CDFTOOLS_4.0 , MEOM 2017 
   !! $Id$
-  !! Copyright (c) 2011, J.-M. Molines
+  !! Copyright (c) 2017, J.-M. Molines 
   !! Software governed by the CeCILL licence (Licence/CDFTOOLSCeCILL.txt)
+  !! @class data_transformation
   !!----------------------------------------------------------------------
   IMPLICIT NONE
   !
@@ -56,20 +58,20 @@ PROGRAM cdfsmooth
   INTEGER(KIND=4)                               :: ncout             ! ncid of output file
   INTEGER(KIND=4)                               :: ilev              ! level to process if not 0
   INTEGER(KIND=4)                               :: ijk               ! indirect level addressing
-  INTEGER(KIND=4), DIMENSION(:,:),  ALLOCATABLE :: iw                ! flag for bad values (or land masked )
   INTEGER(KIND=4), DIMENSION(:),    ALLOCATABLE :: id_var            ! arrays of var id's
   INTEGER(KIND=4), DIMENSION(:),    ALLOCATABLE :: ipk               ! arrays of vertical level for each var
   INTEGER(KIND=4), DIMENSION(:),    ALLOCATABLE :: id_varout         ! id of output variables
   INTEGER(KIND=4), DIMENSION(:),    ALLOCATABLE :: iklist            ! list of k-level to process
+  INTEGER(KIND=4), DIMENSION(:,:),  ALLOCATABLE :: iw                ! flag for bad values (or land masked )
 
-  REAL(KIND=4), DIMENSION(:,:),     ALLOCATABLE :: v2d, w2d          ! raw data,  filtered result
-  REAL(KIND=4), DIMENSION(:),       ALLOCATABLE :: tim               ! time array
-  REAL(KIND=4), DIMENSION(:),       ALLOCATABLE :: gdep, gdeptmp     ! depth array 
   REAL(KIND=4)                                  :: fn, rspval        ! cutoff freq/wavelength, spval
   REAL(KIND=4)                                  :: ranis             ! anistropy
+  REAL(KIND=4), DIMENSION(:),       ALLOCATABLE :: gdep, gdeptmp     ! depth array 
+  REAL(KIND=4), DIMENSION(:,:),     ALLOCATABLE :: v2d, w2d          ! raw data,  filtered result
 
-  REAL(KIND=8), DIMENSION(:,:),     ALLOCATABLE :: dec2d             ! working array
+  REAL(KIND=8), DIMENSION(:),       ALLOCATABLE :: dtim              ! time array
   REAL(KIND=8), DIMENSION(:),       ALLOCATABLE :: dec, de           ! weight in r8, starting index 0:nband
+  REAL(KIND=8), DIMENSION(:,:),     ALLOCATABLE :: dec2d             ! working array
 
   TYPE (variable), DIMENSION(:),    ALLOCATABLE :: stypvar           ! struture for attribute
 
@@ -79,22 +81,21 @@ PROGRAM cdfsmooth
   CHARACTER(LEN=256)                            :: ctyp              ! filter type
   CHARACTER(LEN=256)                            :: cldum             ! dummy character variable
   CHARACTER(LEN=256)                            :: clklist           ! ciphered k-list of level
- 
-  LOGICAL                                       :: lnc4 = .false.    ! flag for netcdf4 output with chinking and deflation
+
+  LOGICAL                                       :: lnc4 = .FALSE.    ! flag for netcdf4 output with chinking and deflation
 
   !!----------------------------------------------------------------------
   CALL ReadCdfNames()
 
   narg=iargc()
   IF ( narg == 0 ) THEN
-     PRINT *,' usage : cdfsmooth -f IN-file -c ncut [-t filter_type] [ -k level_list ] ...'
-     PRINT *,'       [-a anisotripoc ratio ] [-nc4 ] '
+     PRINT *,' usage : cdfsmooth -f IN-file -c ncut [-t FLT-type] [-k LST-level] ...'
+     PRINT *,'       [-anis ratio ] [-nc4 ] '
      PRINT *,'      '
      PRINT *,'     PURPOSE :'
-     PRINT *,'       Perform a spatial smoothing on the file using a particular'
-     PRINT *,'       filter as specified in the option. Available filters' 
-     PRINT *,'       are : Lanczos, Hanning, Shapiro, Box car average. Default'
-     PRINT *,'       is Lanczos filter.'
+     PRINT *,'       Perform a spatial smoothing on the file using a particular filter as'
+     PRINT *,'       specified in the ''-t'' option. Available filters are : Lanczos, Hanning,' 
+     PRINT *,'       Shapiro and Box car average. Default is Lanczos filter.'
      PRINT *,'      '
      PRINT *,'     ARGUMENTS :'
      PRINT *,'       -f  IN-file  : input data file. All variables will be filtered'
@@ -102,22 +103,25 @@ PROGRAM cdfsmooth
      PRINT *,'                    of iteration of the Shapiro filter.'
      PRINT *,'      '
      PRINT *,'     OPTIONS :'
-     PRINT *,'       -t filter_type : Lanczos      , L, l  (default)'
-     PRINT *,'                        Hanning      , H, h'
-     PRINT *,'                        Shapiro      , S, s'
-     PRINT *,'                        Box          , B, b'
-     PRINT *,'       -a aniso       : anisotropic ratio for Box car '
-     PRINT *,'       -k level_list  : levels to be filtered (default = all levels)'
-     PRINT *,'               level_list is a comma-separated list of levels.'
-     PRINT *,'                  the syntax 1-3,6,9-12 will select 1 2 3 6 9 10 11 12'
-     PRINT *,'       -nc4 : produce netcdf4 output file with chunking and deflation.'
+     PRINT *,'       [-t FLT-type] : Lanczos      , L, l  (default)'
+     PRINT *,'                       Hanning      , H, h'
+     PRINT *,'                       Shapiro      , S, s'
+     PRINT *,'                       Box          , B, b'
+     PRINT *,'       [-anis ratio ] : Specify an anisotropic ratio in case of Box-car filter.'
+     PRINT *,'               With ratio=1, the box is a square 2.ncut x 2.ncut grid points.'
+     PRINT *,'               In general, the box is then a rectangle 2.ncut*ratio x 2.ncut.'
+     PRINT *,'       [-k LST-level ] : levels to be filtered (default = all levels)'
+     PRINT *,'               LST-level is a comma-separated list of levels. For example,'
+     PRINT *,'               the syntax 1-3,6,9-12 will select 1 2 3 6 9 10 11 12'
+     PRINT *,'       [-nc4] : produce netcdf4 output file with chunking and deflation.'
      PRINT *,'      '
      PRINT *,'     OUTPUT : '
      PRINT *,'       Output file name is build from input file name with indication'
      PRINT *,'       of the filter type (1 letter) and of ncut.'
      PRINT *,'       netcdf file :   IN-file[LHSB]ncut'
      PRINT *,'         variables : same as input variables.'
-     STOP
+     PRINT *,'      '
+     STOP 
   ENDIF
   !
   ijarg = 1
@@ -128,19 +132,18 @@ PROGRAM cdfsmooth
   DO WHILE (ijarg <= narg )
      CALL getarg ( ijarg, cldum ) ; ijarg=ijarg+1
      SELECT CASE (cldum)
-     CASE ( '-f' ) ; CALL getarg ( ijarg, cf_in   ) ; ijarg=ijarg+1
-     CASE ( '-c' ) ; CALL getarg ( ijarg, cldum   ) ; ijarg=ijarg+1 ; READ(cldum,*) ncut
-     CASE ( '-t' ) ; CALL getarg ( ijarg, ctyp    ) ; ijarg=ijarg+1 
-     CASE ( '-k' ) ; CALL getarg ( ijarg, clklist ) ; ijarg=ijarg+1 
-          CALL GetList (clklist, iklist, ilev )
-     CASE ( '-a'  ) ; CALL getarg ( ijarg, cldum   ) ; ijarg=ijarg+1 ; READ(cldum,*) ranis
-     CASE ( '-nc4') ;  lnc4=.true.
+     CASE ( '-f'  ) ; CALL getarg ( ijarg, cf_in   ) ; ijarg=ijarg+1
+     CASE ( '-c'  ) ; CALL getarg ( ijarg, cldum   ) ; ijarg=ijarg+1 ; READ(cldum,*) ncut
+     CASE ( '-t'  ) ; CALL getarg ( ijarg, ctyp    ) ; ijarg=ijarg+1 
+     CASE ( '-k'  ) ; CALL getarg ( ijarg, clklist ) ; ijarg=ijarg+1 
+                    ; CALL GetList (clklist, iklist, ilev )
+     CASE ('-anis') ; CALL getarg ( ijarg, cldum   ) ; ijarg=ijarg+1 ; READ(cldum,*) ranis
+     CASE ( '-nc4') ; lnc4 = .TRUE.
+     CASE DEFAULT   ; PRINT *,' ERROR :' ,TRIM(cldum),' : unknown option.' ; STOP 99
      END SELECT
   ENDDO
 
-  IF ( ncut == 0 ) THEN
-     PRINT *, ' cdfsmooth : ncut = 0 --> nothing to do !'
-     STOP 99
+  IF ( ncut == 0 ) THEN ; PRINT *, ' cdfsmooth : ncut = 0 --> nothing to do !' ; STOP 99
   ENDIF
 
   IF ( chkfile(cf_in) ) STOP 99 ! missing file
@@ -153,31 +156,31 @@ PROGRAM cdfsmooth
 
   WRITE(cf_out,'(a,a,i3.3)') TRIM(cf_in),'L',ncut   ! default name
 
-     SELECT CASE ( ctyp)
-     CASE ( 'Lanczos','L','l') 
-        nfilter=jp_lanc
-        WRITE(cf_out,'(a,a,i3.3)') TRIM(cf_in),'L',ncut
-        PRINT *,' Working with Lanczos filter'
-     CASE ( 'Hanning','H','h')
-        nfilter=jp_hann
-        ALLOCATE ( dec2d(0:2,0:2) )
-        WRITE(cf_out,'(a,a,i3.3)') TRIM(cf_in),'H',ncut
-        PRINT *,' Working with Hanning filter'
-     CASE ( 'Shapiro','S','s')
-        nfilter=jp_shap
-        WRITE(cf_out,'(a,a,i3.3)') TRIM(cf_in),'S',ncut
-        PRINT *,' Working with Shapiro filter'
-     CASE ( 'Box','B','b')
-        nfilter=jp_boxc
-        WRITE(cf_out,'(a,a,i3.3)') TRIM(cf_in),'B',ncut
-        IF ( ranis /=1. ) THEN
-          PRINT *, 'Anisotropic box car with ratio Lx = ', ranis, 'x Ly'
-        ELSE
-          PRINT *,' Working with Box filter'
-        ENDIF
-     CASE DEFAULT
-        PRINT *, TRIM(ctyp),' : undefined filter ' ; STOP 99
-     END SELECT
+  SELECT CASE ( ctyp)
+  CASE ( 'Lanczos','L','l') 
+     nfilter=jp_lanc
+     WRITE(cf_out,'(a,a,i3.3)') TRIM(cf_in),'L',ncut
+     PRINT *,' Working with Lanczos filter'
+  CASE ( 'Hanning','H','h')
+     nfilter=jp_hann
+     ALLOCATE ( dec2d(0:2,0:2) )
+     WRITE(cf_out,'(a,a,i3.3)') TRIM(cf_in),'H',ncut
+     PRINT *,' Working with Hanning filter'
+  CASE ( 'Shapiro','S','s')
+     nfilter=jp_shap
+     WRITE(cf_out,'(a,a,i3.3)') TRIM(cf_in),'S',ncut
+     PRINT *,' Working with Shapiro filter'
+  CASE ( 'Box','B','b')
+     nfilter=jp_boxc
+     WRITE(cf_out,'(a,a,i3.3)') TRIM(cf_in),'B',ncut
+     IF ( ranis /=1. ) THEN
+        PRINT *, 'Anisotropic box car with ratio Lx = ', ranis, 'x Ly'
+     ELSE
+        PRINT *,' Working with Box filter'
+     ENDIF
+  CASE DEFAULT
+     PRINT *, TRIM(ctyp),' : undefined filter ' ; STOP 99
+  END SELECT
 
   CALL filterinit (nfilter, fn, nband)
   ! Look for input file and create outputfile
@@ -205,19 +208,17 @@ PROGRAM cdfsmooth
   PRINT *, 'npk    = ',npk
   PRINT *, 'npt    = ',npt
 
-  ALLOCATE ( v2d(npiglo,npjglo),iw(npiglo,npjglo), w2d(npiglo,npjglo), tim(npt) )
+  ALLOCATE ( v2d(npiglo,npjglo),iw(npiglo,npjglo), w2d(npiglo,npjglo), dtim(npt) )
   nvars = getnvar(cf_in)
   PRINT *, 'nvars = ', nvars
   ALLOCATE (cv_names(nvars) )
   ALLOCATE (stypvar(nvars) )
   ALLOCATE (id_var(nvars),ipk(nvars),id_varout(nvars) )
 
-    ALLOCATE ( gdeptmp(npk)  )
-    IF (npkf /= 0 ) THEN 
-       gdeptmp(:) = getvar1d(cf_in, cv_dep, npk )  
-    ELSE
-       gdeptmp(:)=0.  ! dummy value
-    ENDIF
+  ALLOCATE ( gdeptmp(npk)  )
+  IF (npkf /= 0 ) THEN ; gdeptmp(:) = getvar1d(cf_in, cv_dep, npk )  
+  ELSE                 ; gdeptmp(:) = 0.  ! dummy value
+  ENDIF
 
   ! get list of variable names and collect attributes in stypvar (optional)
   cv_names(:) = getvarname(cf_in, nvars, stypvar)
@@ -249,7 +250,7 @@ PROGRAM cdfsmooth
   ncout = create      (cf_out, cf_in,   npiglo, npjglo, npkf, cdep=cv_dep, ld_nc4=lnc4)
   ierr  = createvar   (ncout , stypvar, nvars,  ipk,    id_varout        , ld_nc4=lnc4)
   ierr  = putheadervar(ncout , cf_in,   npiglo, npjglo, npkf, pdep=gdep, cdep=cv_dep)
-  tim   = getvar1d(cf_in, cv_tim, npt)
+  dtim  = getvar1d(cf_in, cv_tim, npt)
   !
   DO jvar = 1,nvars
      IF ( cv_names(jvar) == cn_vlon2d .OR.                     &
@@ -273,8 +274,8 @@ PROGRAM cdfsmooth
         END DO
      ENDIF
   END DO
-  ierr = putvar1d(ncout, tim, npt, 'T')
-  ierr = closeout(ncout               )
+  ierr = putvar1d(ncout, dtim, npt, 'T')
+  ierr = closeout(ncout                )
 
 CONTAINS
 
@@ -290,19 +291,14 @@ CONTAINS
     INTEGER(KIND=4), INTENT(in) :: kband    ! filter bandwidth
     !!----------------------------------------------------------------------
     SELECT CASE ( kfilter)
-    CASE ( jp_lanc )
-       CALL initlanc (pfn, kband)
-    CASE ( jp_hann )
-       CALL inithann (pfn, kband)
-    CASE ( jp_shap )
-       CALL initshap (pfn, kband)
-    CASE ( jp_boxc )
-       CALL initbox  (pfn, kband)
+    CASE ( jp_lanc ) ; CALL initlanc (pfn, kband)
+    CASE ( jp_hann ) ; CALL inithann (pfn, kband)
+    CASE ( jp_shap ) ; CALL initshap (pfn, kband)
+    CASE ( jp_boxc ) ; CALL initbox  (pfn, kband)
     END SELECT
 
   END SUBROUTINE filterinit
 
-
   SUBROUTINE filter (kfilter, px, kpx, py)
     !!---------------------------------------------------------------------
     !!                  ***  ROUTINE filter  ***
@@ -316,19 +312,14 @@ CONTAINS
     REAL(KIND=4), DIMENSION(:,:),   INTENT(out) :: py      ! output data
     !!----------------------------------------------------------------------
     SELECT CASE ( kfilter)
-    CASE ( jp_lanc )
-       CALL lislanczos2d (px, kpx, py, npiglo, npjglo, fn, nband)
-    CASE ( jp_hann )
-       CALL lishan2d     (px, kpx, py, ncut, npiglo, npjglo)
-    CASE ( jp_shap )
-       CALL lisshapiro1d (px, kpx, py, ncut, npiglo, npjglo)
-    CASE ( jp_boxc )
-       CALL lisbox       (px, kpx, py, npiglo, npjglo, fn, nband, ranis)
+    CASE ( jp_lanc ) ; CALL lislanczos2d (px, kpx, py, npiglo, npjglo, fn, nband)
+    CASE ( jp_hann ) ; CALL lishan2d     (px, kpx, py, ncut, npiglo, npjglo)
+    CASE ( jp_shap ) ; CALL lisshapiro1d (px, kpx, py, ncut, npiglo, npjglo)
+    CASE ( jp_boxc ) ; CALL lisbox       (px, kpx, py, npiglo, npjglo, fn, nband, ranis)
     END SELECT
 
   END SUBROUTINE filter
 
-
   SUBROUTINE initlanc(pfn, knj)
     !!---------------------------------------------------------------------
     !!                  ***  ROUTINE initlanc  ***
@@ -358,7 +349,6 @@ CONTAINS
 
   END SUBROUTINE initlanc
 
-
   SUBROUTINE inithann(pfn, knj)
     !!---------------------------------------------------------------------
     !!                  ***  ROUTINE inithann  ***
@@ -379,13 +369,8 @@ CONTAINS
     ! and the other 
     dec2d(0,1) = 1.d0 ;  dec2d(2,1) = 1.d0
 
-    ! normalize
-    dl_sum     = SUM(dec2d)
-    dec2d(:,:) = dec2d(:,:) / dl_sum
-
   END SUBROUTINE inithann
 
-
   SUBROUTINE initshap(pfn, knj)
     !!---------------------------------------------------------------------
     !!                  ***  ROUTINE initshap  ***
@@ -400,7 +385,6 @@ CONTAINS
 
   END SUBROUTINE initshap
 
-
   SUBROUTINE initbox(pfn, knj)
     !!---------------------------------------------------------------------
     !!                  ***  ROUTINE initbox  ***
@@ -536,7 +520,8 @@ CONTAINS
 
              ! we don't compute in land
              IF ( kiw(ji,jj) == 1 ) THEN
-                py(ji,jj) = SUM( dec2d(:,:) * ztmp(iiminus1:iiplus1,ijminus1:ijplus1) )
+                py(ji,jj) = SUM( dec2d(:,:) * ztmp(iiminus1:iiplus1,ijminus1:ijplus1) * kiw(iiminus1:iiplus1,ijminus1:ijplus1) )
+                py(ji,jj) = py(ji,jj) / SUM( dec2d(:,:) * kiw(iiminus1:iiplus1,ijminus1:ijplus1) )   ! normalisation
              ENDIF
           ENDDO
        ENDDO
@@ -627,8 +612,7 @@ CONTAINS
 
   END SUBROUTINE lisshapiro1d
 
-
-  SUBROUTINE lisbox(px, kiw, py, kpi, kpj, pfn, knj,anis)
+  SUBROUTINE lisbox(px, kiw, py, kpi, kpj, pfn, knj,panis)
     !!---------------------------------------------------------------------
     !!                  ***  ROUTINE lisbox  ***
     !!
@@ -641,7 +625,7 @@ CONTAINS
     INTEGER(KIND=4),                 INTENT(in ) :: kpi, kpj         ! size of input/output
     REAL(KIND=4),                    INTENT(in ) :: pfn              ! cutoff frequency/wavelength
     INTEGER(KIND=4),                 INTENT(in ) :: knj              ! filter bandwidth
-    REAL(KIND=4),                    INTENT(in ) :: anis             ! anisotrop
+    REAL(KIND=4),                    INTENT(in ) :: panis            ! anisotrop
 
     INTEGER(KIND=4)                              :: ji, jj
     INTEGER(KIND=4)                              :: ik1x, ik2x, ik1y, ik2y
@@ -651,8 +635,8 @@ CONTAINS
     ll_mask=.TRUE.
     WHERE (kiw == 0 ) ll_mask=.FALSE.
     DO ji=1,kpi
-       ik1x = ji-NINT( anis * knj)  ; ik2x = ji+NINT( anis * knj)
-       ik1x = MAX(1,ik1x)           ; ik2x = MIN(kpi,ik2x)
+       ik1x = ji-NINT( panis * knj)  ; ik2x = ji+NINT( panis * knj)
+       ik1x = MAX(1,ik1x)            ; ik2x = MIN(kpi,ik2x)
        DO jj=1,kpj
           ik1y = jj-knj       ; ik2y = jj+knj
           ik1y = MAX(1,ik1y)  ; ik2y = MIN(kpj,ik2y)
