@@ -6,6 +6,7 @@ MODULE cdftools
   !!=====================================================================
   !! History : 2.1  !  05/2010  : J.M. Molines, A. Melet : Original
   !!           3.0  !  12/2010  : J.M. Molines : Doctor + Lic.
+  !!         : 4.0  : 03/2017  : J.M. Molines  
   !!----------------------------------------------------------------------
   !!----------------------------------------------------------------------
   !!   routines      : description
@@ -22,16 +23,17 @@ MODULE cdftools
   ! list of public subroutines that can be called
   PUBLIC  :: cdf_findij 
   PUBLIC  :: broken_line
+  PUBLIC  :: dist
 
   PRIVATE :: NearestPoint
-  PRIVATE :: dist 
   PRIVATE :: interm_pt
 
   !!----------------------------------------------------------------------
-  !! CDFTOOLS_3.0 , MEOM 2011
+  !! CDFTOOLS_4.0 , MEOM 2017 
   !! $Id$
-  !! Copyright (c) 2010, J.-M. Molines
+  !! Copyright (c) 2017, J.-M. Molines 
   !! Software governed by the CeCILL licence (Licence/CDFTOOLSCeCILL.txt)
+  !! @class system
   !!----------------------------------------------------------------------
 
 CONTAINS
@@ -69,7 +71,7 @@ CONTAINS
     CHARACTER(LEN=256)                        :: cl_type='F'
     CHARACTER(LEN=256)                        :: clcoo
 
-    LOGICAL                                   :: ll_again, ll_bnd, ll_verbose=.false.
+    LOGICAL                                   :: ll_again, ll_bnd, ll_verbose=.FALSE.
     !!--------------------------------------------------------------------------
     CALL ReadCdfNames()
     imin = 0 ; jmin = 0
@@ -84,56 +86,56 @@ CONTAINS
     IF ( PRESENT( cd_coord)  ) clcoo  = cd_coord
     IF ( PRESENT( cd_point)  ) cl_type= cd_point
     IF ( PRESENT( cd_verbose))   THEN
-       IF ( cd_verbose(1:1) == 'Y' .OR. cd_verbose(1:1) == 'y' ) ll_verbose=.true.
+       IF ( cd_verbose(1:1) == 'Y' .OR. cd_verbose(1:1) == 'y' ) ll_verbose=.TRUE.
     ENDIF
 
     IF ( .NOT. ALLOCATED (dl_glam) ) THEN 
 
-    IF (chkfile (clcoo) ) STOP 99 ! missing file
+       IF (chkfile (clcoo) ) STOP 96 ! missing file
 
-    ipiglo= getdim (clcoo, cn_x)
-    ipjglo= getdim (clcoo, cn_y)
+       ipiglo= getdim (clcoo, cn_x)
+       ipjglo= getdim (clcoo, cn_y)
 
-    ALLOCATE (dl_glam(ipiglo,ipjglo), dl_gphi(ipiglo,ipjglo) )
-    ALLOCATE (dl_e1  (ipiglo,ipjglo), dl_e2  (ipiglo,ipjglo) )
+       ALLOCATE (dl_glam(ipiglo,ipjglo), dl_gphi(ipiglo,ipjglo) )
+       ALLOCATE (dl_e1  (ipiglo,ipjglo), dl_e2  (ipiglo,ipjglo) )
 
-    SELECT CASE ( cl_type )
-    CASE ('T' , 't' )
-       dl_glam(:,:) = getvar(clcoo, cn_glamt, 1, ipiglo, ipjglo)
-       dl_gphi(:,:) = getvar(clcoo, cn_gphit, 1, ipiglo, ipjglo)
-       dl_e1  (:,:) = getvar(clcoo, cn_ve1t,  1, ipiglo, ipjglo)
-       dl_e2  (:,:) = getvar(clcoo, cn_ve2t,  1, ipiglo, ipjglo)
-    CASE ('U','u' )
-       dl_glam(:,:) = getvar(clcoo, cn_glamu, 1, ipiglo, ipjglo)
-       dl_gphi(:,:) = getvar(clcoo, cn_gphiu, 1, ipiglo, ipjglo)
-       dl_e1  (:,:) = getvar(clcoo, cn_ve1u,  1, ipiglo, ipjglo)
-       dl_e2  (:,:) = getvar(clcoo, cn_ve2u,  1, ipiglo, ipjglo)
-    CASE ('V','v' )
-       dl_glam(:,:) = getvar(clcoo, cn_glamv, 1, ipiglo, ipjglo)
-       dl_gphi(:,:) = getvar(clcoo, cn_gphiv, 1, ipiglo, ipjglo)
-       dl_e1  (:,:) = getvar(clcoo, cn_ve1v,  1, ipiglo, ipjglo)
-       dl_e2  (:,:) = getvar(clcoo, cn_ve2v,  1, ipiglo, ipjglo)
-    CASE ('F','f' )
-       dl_glam(:,:) = getvar(clcoo, cn_glamf, 1, ipiglo, ipjglo)
-       dl_gphi(:,:) = getvar(clcoo, cn_gphif, 1, ipiglo, ipjglo)
-       dl_e1  (:,:) = getvar(clcoo, cn_ve1f,  1, ipiglo, ipjglo)
-       dl_e2  (:,:) = getvar(clcoo, cn_ve2f,  1, ipiglo, ipjglo)
-    CASE DEFAULT
-       PRINT *,' ERROR : type of point not known: ', TRIM(cl_type)
-    END SELECT
+       SELECT CASE ( cl_type )
+       CASE ('T' , 't' )
+          dl_glam(:,:) = getvar(clcoo, cn_glamt, 1, ipiglo, ipjglo)
+          dl_gphi(:,:) = getvar(clcoo, cn_gphit, 1, ipiglo, ipjglo)
+          dl_e1  (:,:) = getvar(clcoo, cn_ve1t,  1, ipiglo, ipjglo)
+          dl_e2  (:,:) = getvar(clcoo, cn_ve2t,  1, ipiglo, ipjglo)
+       CASE ('U','u' )
+          dl_glam(:,:) = getvar(clcoo, cn_glamu, 1, ipiglo, ipjglo)
+          dl_gphi(:,:) = getvar(clcoo, cn_gphiu, 1, ipiglo, ipjglo)
+          dl_e1  (:,:) = getvar(clcoo, cn_ve1u,  1, ipiglo, ipjglo)
+          dl_e2  (:,:) = getvar(clcoo, cn_ve2u,  1, ipiglo, ipjglo)
+       CASE ('V','v' )
+          dl_glam(:,:) = getvar(clcoo, cn_glamv, 1, ipiglo, ipjglo)
+          dl_gphi(:,:) = getvar(clcoo, cn_gphiv, 1, ipiglo, ipjglo)
+          dl_e1  (:,:) = getvar(clcoo, cn_ve1v,  1, ipiglo, ipjglo)
+          dl_e2  (:,:) = getvar(clcoo, cn_ve2v,  1, ipiglo, ipjglo)
+       CASE ('F','f' )
+          dl_glam(:,:) = getvar(clcoo, cn_glamf, 1, ipiglo, ipjglo)
+          dl_gphi(:,:) = getvar(clcoo, cn_gphif, 1, ipiglo, ipjglo)
+          dl_e1  (:,:) = getvar(clcoo, cn_ve1f,  1, ipiglo, ipjglo)
+          dl_e2  (:,:) = getvar(clcoo, cn_ve2f,  1, ipiglo, ipjglo)
+       CASE DEFAULT
+          PRINT *,' ERROR : type of point not known: ', TRIM(cl_type)
+       END SELECT
 
-    ! work with longitude between 0 and 360 to avoid  the date line.
-    WHERE( dl_glam < 0 ) dl_glam = dl_glam + 360.d0
+       ! work with longitude between 0 and 360 to avoid  the date line.
+       WHERE( dl_glam < 0 ) dl_glam = dl_glam + 360.d0
 
-    ! For Orca grid, the longitude of ji=1 is about 70 E
-    dl_glam0 = dl_glam(1, ipjglo/2)
-    WHERE( dl_glam < dl_glam0 ) dl_glam =dl_glam + 360.d0
+       ! For Orca grid, the longitude of ji=1 is about 70 E
+       dl_glam0 = dl_glam(1, ipjglo/2)
+       WHERE( dl_glam < dl_glam0 ) dl_glam =dl_glam + 360.d0
 
-    IF (dl_xmin < 0.) dl_xmin = dl_xmin + 360.d0
-    IF (dl_xmax < 0.) dl_xmax = dl_xmax + 360.d0
+       IF (dl_xmin < 0.) dl_xmin = dl_xmin + 360.d0
+       IF (dl_xmax < 0.) dl_xmax = dl_xmax + 360.d0
 
-    IF (dl_xmin < dl_glam0) dl_xmin = dl_xmin + 360.d0
-    IF (dl_xmax < dl_glam0) dl_xmax = dl_xmax + 360.d0
+       IF (dl_xmin < dl_glam0) dl_xmin = dl_xmin + 360.d0
+       IF (dl_xmax < dl_glam0) dl_xmax = dl_xmax + 360.d0
     ENDIF  ! 
 
 
@@ -154,11 +156,11 @@ CONTAINS
        IF (dl_dis  > dl_emax ) THEN
           zglamfound = dl_glam(iloc,jloc) ; IF (zglamfound > 180.)  zglamfound=zglamfound - 360.
 
-!         PRINT 9000, 'Long= ',zglamfound,' Lat = ',dl_gphi(iloc,jloc) , iloc, jloc 
-!         PRINT *,' Algorithm does''nt converge ', dl_dis
+          !         PRINT 9000, 'Long= ',zglamfound,' Lat = ',dl_gphi(iloc,jloc) , iloc, jloc 
+          !         PRINT *,' Algorithm does''nt converge ', dl_dis
 
           IF ( initer >= jp_itermax ) THEN
-!            PRINT *, ' no convergence after ', jp_itermax,' iterations'
+             !            PRINT *, ' no convergence after ', jp_itermax,' iterations'
              iloc     = -1000
              jloc     = -1000
              ll_again = .FALSE.
@@ -205,11 +207,11 @@ CONTAINS
           IF (dl_dis >  dl_emax ) THEN
              zglamfound=dl_glam(iloc,jloc) ; IF (zglamfound > 180.)  zglamfound=zglamfound -360.
 
-!            PRINT 9000, 'Long= ',zglamfound,' Lat = ',dl_gphi(iloc,jloc), iloc, jloc
-!            PRINT *,' Algorithm does''nt converge ', dl_dis
+             !            PRINT 9000, 'Long= ',zglamfound,' Lat = ',dl_gphi(iloc,jloc), iloc, jloc
+             !            PRINT *,' Algorithm does''nt converge ', dl_dis
 
              IF ( initer >= jp_itermax ) THEN
-!               PRINT *, ' no convergence after ', jp_itermax,' iterations'
+                !               PRINT *, ' no convergence after ', jp_itermax,' iterations'
                 iloc     = -1000
                 jloc     = -1000
                 ll_again = .FALSE.
@@ -237,13 +239,13 @@ CONTAINS
     IF (ll_verbose) PRINT 9001, imin, imax, jmin, jmax
 
     kimin   = imin ; kimax = imax ; kjmin   = jmin ; kjmax = jmax
-   IF ( ll_bnd .OR. imin < 0 ) THEN
-     zglamin= -9999. ; zglamax = -9999
-     zgphmin= -9999. ; zgphmax = -9999
-   ELSE
-    zglamin = dl_glam(imin,jmin)  ; zglamax = dl_glam(imax,jmax)
-    zgphmin = dl_gphi(imin,jmin)  ; zgphmax = dl_gphi(imax,jmax)
-   ENDIF
+    IF ( ll_bnd .OR. imin < 0 ) THEN
+       zglamin= -9999. ; zglamax = -9999
+       zgphmin= -9999. ; zgphmax = -9999
+    ELSE
+       zglamin = dl_glam(imin,jmin)  ; zglamax = dl_glam(imax,jmax)
+       zgphmin = dl_gphi(imin,jmin)  ; zgphmax = dl_gphi(imax,jmax)
+    ENDIF
 
     IF ( zglamin > 180 ) zglamin=zglamin-360.
     IF ( zglamax > 180 ) zglamax=zglamax-360.
@@ -277,7 +279,7 @@ CONTAINS
     INTEGER(KIND=4),            INTENT(in ) :: kimin, kimax, kjmin, kjmax
     INTEGER(KIND=4),            INTENT(out) :: knn
     INTEGER(KIND=4),            INTENT(in ) :: kpi, kpj
-    INTEGER(KIND=4),            INTENT(out) :: knormu, knormv
+    INTEGER(KIND=4), OPTIONAL,  INTENT(out) :: knormu, knormv
     REAL(KIND=4), DIMENSION(kpi+kpj), INTENT(out) :: pxx, pyy
 
     INTEGER(KIND=4) :: ji, jj, jk   ! dummy loop index
@@ -314,8 +316,8 @@ CONTAINS
     !This will produce the following sign convention:
     !    West-to-est line (dx>0, dy=0)=> -My*dx (-ve for a northward flow)
     !    South-to-north   (dy>0, dx=0)=>  Mx*dy (+ve for an eastward flow)
-    knormu =  idiry
-    knormv = -idirx
+    IF ( PRESENT (knormu) ) knormu =  idiry
+    IF ( PRESENT (knormv) ) knormv = -idirx
 
     ! .. Compute equation:  ryj = zaj rxi + zbj [valid in the (i,j) plane]
     IF ( (zxi1 -zxi0) /=  0 ) THEN
@@ -356,9 +358,7 @@ CONTAINS
        ! ... compute the nearest ji point on the line crossing at ji
        DO ji=ii0, ii1
           ipts=ipts+1
-          IF (ipts > kpi+kpj) THEN
-             PRINT *, 'ipts > kpi+kpj !'
-             STOP 96
+          IF (ipts > kpi+kpj) THEN ; PRINT *,' ... ERROR in broken_line : ipts > kpi+kpj !' ; STOP 96
           ENDIF
           ij=NINT(zaj*ji + zbj )
           ylpt(ipts) = CMPLX(ji,ij)
@@ -382,9 +382,7 @@ CONTAINS
        ! ... compute the nearest ji point on the line crossing at jj
        DO jj=ij0,ij1
           ipts=ipts+1
-          IF (ipts > kpi+kpj) THEN
-             PRINT *, 'ipts > kpi+kpj !'
-             STOP 96
+          IF (ipts > kpi+kpj) THEN ; PRINT *,' ... ERROR in broken_line : ipts > kpi+kpj !' ; STOP 96
           ENDIF
           ii=NINT(zai*jj + zbi)
           ylpt(ipts) = CMPLX(ii,jj)
@@ -404,17 +402,13 @@ CONTAINS
        IF ( zd > 1 ) THEN
           CALL interm_pt(ylpt, jk, zai, zbi, zaj, zbj, ylpti)
           knn=knn+1
-          IF (knn > kpi+kpj) THEN
-             PRINT *, 'knn>kpi+kpj !'
-             STOP 96
+          IF (knn > kpi+kpj) THEN ; PRINT *,' ... ERROR in broken_line : knn > kpi+kpj !' ; STOP 96
           ENDIF
           pxx(knn) = REAL(ylpti)
           pyy(knn) = IMAG(ylpti)
        END IF
        knn=knn+1
-       IF (knn > kpi+kpj) THEN
-          PRINT *, 'knn>kpi+kpj !'
-          STOP 96
+       IF (knn > kpi+kpj) THEN ; PRINT *,' ... ERROR in broken_line : knn > kpi+kpj !' ; STOP 96
        ENDIF
        pxx(knn) = REAL(ylpt(jk))
        pyy(knn) = IMAG(ylpt(jk))
@@ -422,12 +416,12 @@ CONTAINS
 
   END SUBROUTINE broken_line
 
-! Private subroutines and functions
+  ! Private subroutines and functions
   SUBROUTINE NearestPoint(ddlon, ddlat, kpi, kpj, ddlam, ddphi, kpiloc, kpjloc, ld_bnd)
     !!---------------------------------------------------------------------
     !!                  ***  ROUTINE NearestPoint  ***
     !!
-    !! ** Purpose : Computes the positions of the nearest i,j in the grid
+    !! ** Purpose : Compute the positions of the nearest i,j in the grid
     !!              from the given longitudes and latitudes
     !!
     !! ** Method  : Starts on the middle of the grid, search in a 20x20 box,
