@@ -11,25 +11,28 @@ PROGRAM cdftransig_xy3d
   !!           2.1  : 02/2011  : A.M. Treguier : Allow increased resolution in density
   !!                                             in deeper layers
   !!           3.0  : 06/2011  : J.M. Molines : Doctor norm + Lic.
+  !!         : 4.0  : 03/2017  : J.M. Molines  
   !!----------------------------------------------------------------------
   USE cdfio
   USE eos
   USE modcdfnames
   USE modutils      ! for SetFileName, SetGlobalAtt
   !!----------------------------------------------------------------------
-  !! CDFTOOLS_3.0 , MEOM 2011
+  !! CDFTOOLS_4.0 , MEOM 2017 
   !! $Id$
-  !! Copyright (c) 2011, J.-M. Molines
+  !! Copyright (c) 2017, J.-M. Molines 
   !! Software governed by the CeCILL licence (Licence/CDFTOOLSCeCILL.txt)
+  !! @class transport
   !!----------------------------------------------------------------------
   IMPLICIT NONE
 
   INTEGER(KIND=4)                              :: ji, jj, jk         ! dummy loop index
   INTEGER(KIND=4)                              :: jt, jtag           ! dummy loop index
+  INTEGER(KIND=4)                              :: it                 ! time index for vvl
   INTEGER(KIND=4)                              :: ijb   
   INTEGER(KIND=4)                              :: ierr               ! working integer
   INTEGER(KIND=4)                              :: narg, iargc        ! command line 
-  INTEGER(KIND=4)                              :: ijarg, ireq, istag
+  INTEGER(KIND=4)                              :: ijarg, icur, istag
   INTEGER(KIND=4)                              :: iset
   INTEGER(KIND=4)                              :: npiglo, npjglo     ! size of the domain
   INTEGER(KIND=4)                              :: npk, npt           ! size of the domain
@@ -41,29 +44,30 @@ PROGRAM cdftransig_xy3d
   INTEGER(KIND=4), DIMENSION(:),   ALLOCATABLE :: itab               ! look up table for density intervals
   INTEGER(KIND=4), DIMENSION(:,:), ALLOCATABLE :: ibinu, ibinv       ! integer value corresponding to density for binning
 
+  REAL(KIND=4)                                 :: zpref              !  reference for density 
+  REAL(KIND=4), DIMENSION(:),      ALLOCATABLE :: gdept              ! array for depth of T points  
+  REAL(KIND=4), DIMENSION(:),      ALLOCATABLE :: e31d               ! vertical metric in case of full step
   REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: zmasku,zmaskv      !  masks x,1,nbins
   REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: e1v,  gphiv        !  2D x,y metrics, velocity
   REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: e2u                !  metrics, velocity
   REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: zt, zs, zv, e3v    !  x,1,z arrays metrics, velocity
   REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: zu, e3u            !  metrics, velocity
   REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: zdensu, zdensv     ! density on u and v points 
-  REAL(KIND=4), DIMENSION(:),      ALLOCATABLE :: gdept              ! array for depth of T points  
-  REAL(KIND=4), DIMENSION(:),      ALLOCATABLE :: e31d               ! vertical metric in case of full step
-  REAL(KIND=4), DIMENSION(:),      ALLOCATABLE :: tim
-  REAL(KIND=4), DIMENSION(1)                   :: timean
-  REAL(KIND=4)                                 :: pref               !  reference for density 
 
-  REAL(KIND=8), DIMENSION(:,:,:),  ALLOCATABLE :: dusigsig,dvsigsig  ! cumulated transports,   
-  REAL(KIND=8), DIMENSION(:,:),    ALLOCATABLE :: dens2d 
-  REAL(KIND=8), DIMENSION(:),      ALLOCATABLE :: dsigma             ! density coordinate, center of bins
-  REAL(KIND=8), DIMENSION(:),      ALLOCATABLE :: dsig_edge          ! density coordinate, edge of bins.
   REAL(KIND=8)                                 :: dsigtest
   REAL(KIND=8)                                 :: ds1min, ds1scal    !  min sigma and delta_sigma
   REAL(KIND=8)                                 :: ds1zoom = 999., ds1scalmin  !  min sigma for increased resolution
   REAL(KIND=8)                                 :: dtotal_time
+  REAL(KIND=8), DIMENSION(1)                   :: dtimean
+  REAL(KIND=8), DIMENSION(:),      ALLOCATABLE :: dtim
+  REAL(KIND=8), DIMENSION(:),      ALLOCATABLE :: dsigma             ! density coordinate, center of bins
+  REAL(KIND=8), DIMENSION(:),      ALLOCATABLE :: dsig_edge          ! density coordinate, edge of bins.
+  REAL(KIND=8), DIMENSION(:,:),    ALLOCATABLE :: dens2d 
+  REAL(KIND=8), DIMENSION(:,:,:),  ALLOCATABLE :: dusigsig,dvsigsig  ! cumulated transports,   
 
   CHARACTER(LEN=80 )                           :: cf_out='uvxysig.nc'
   CHARACTER(LEN=80 )                           :: cf_tfil
+  CHARACTER(LEN=80 )                           :: cf_sfil
   CHARACTER(LEN=80 )                           :: cf_ufil
   CHARACTER(LEN=80 )                           :: cf_vfil
   CHARACTER(LEN=80 )                           :: cv_outu='vouxysig'
@@ -74,6 +78,7 @@ PROGRAM cdftransig_xy3d
   CHARACTER(LEN=80 )                           :: cldepcode='1000'
   CHARACTER(LEN=256)                           :: cglobal
   CHARACTER(LEN=7  )                           :: clsigma 
+  CHARACTER(LEN=80 ),DIMENSION(:), ALLOCATABLE :: ctag_lst
 
   TYPE (variable), DIMENSION(2)                :: stypvar     ! structure for attributes
 
@@ -81,30 +86,36 @@ PROGRAM cdftransig_xy3d
   LOGICAL                                      :: lfull   = .FALSE.
   LOGICAL                                      :: lnotset = .FALSE.
   LOGICAL                                      :: lchk    = .FALSE.  ! flag for missing files
-  LOGICAL                                      :: lperio  = .FALSE.  ! flag for missing files
+  LOGICAL                                      :: lperio  = .FALSE.  ! flag for periodicity
+  LOGICAL                                      :: lnc4    = .FALSE.  ! flag for nc4 output
+  LOGICAL                                      :: lsal    = .FALSE.  ! flag for salinity file
   !!----------------------------------------------------------------------
   CALL ReadCdfNames()
   narg= iargc()
   IF ( narg == 0 ) THEN
-     PRINT *,' usage : cdftransig_xyz CONFCASE ''list_of_tags'' [-depref depcode ] ...'
+     PRINT *,' usage : cdftransig_xy3d -c CONFIG-CASE -l LST-tags [-code code ] [-S] ...'
      PRINT *,'                    ... [-depref depref ] [ -nbins nbins ] ... ' 
      PRINT *,'                    ... [-sigmin smin s-scal] [-sigzoom sminr s-scalr ] ...'
-     PRINT *,'                    ... [-full ] [-v ]'
+     PRINT *,'                    ... [-full ] [-v ] [-vvl ] [-o OUT-file] [-nc4]'
      PRINT *,'      '
      PRINT *,'     PURPOSE :'
-     PRINT *,'       Computes the volume transport at each grid cell in density space ' 
+     PRINT *,'       Compute the time average volume transport at each grid cell in density'
+     PRINT *,'       space, for the list of tags given as arguments. Results must be '
+     PRINT *,'       condidered as intermediate for further integration.'
      PRINT *,'      '
      PRINT *,'     ARGUMENTS :'
-     PRINT *,'       CONFCASE  : a DRAKKAR CONFIG-CASE name '
-     PRINT *,'       list_of_tags : a list of time tags to be processed'
+     PRINT *,'       -c CONFCASE  : a DRAKKAR CONFIG-CASE name '
+     PRINT *,'       -l LST-tags : a blank-separated list of time tags to be processed.'
+     PRINT *,'         If temperature and salinity not in same file use option -S '
      PRINT *,'      '
      PRINT *,'     OPTIONS :'
-     PRINT *,'       [-depcode depcode ] : depcode corresponds to pre-defined parameter '
-     PRINT *,'              setting, in term of reference depths, density limits for '
-     PRINT *,'              binning, number of bins, deeper layer refinement.'
-     PRINT *,'          AVAILABLE depcode are :'
+     PRINT *,'       [-S         ] : indicate that salinity file is to be taken in gridS files.'
+     PRINT *,'       [-code code ] : code corresponds to pre-defined parameters settings '
+     PRINT *,'              in term of reference depths, density limits for  binning, number'
+     PRINT *,'               of bins, deeper layer refinement.'
+     PRINT *,'          AVAILABLE code are :'
      PRINT *,'              _______________________________________________________________'
-     PRINT *,'              depcode  |  depth_ref nbins   smin  s-scal  szoommin szoom-scal '
+     PRINT *,'              code     |  depth_ref nbins   smin  s-scal  szoommin szoom-scal '
      PRINT *,'              ---------------------------------------------------------------'
      PRINT *,'                0      |    0      101     23.0    0.05                      '
      PRINT *,'              1000     |   1000     93     24.2    0.10   32.3     0.05      ' 
@@ -112,7 +123,7 @@ PROGRAM cdftransig_xy3d
      PRINT *,'              2000     |   2000    174     29.0    0.05                      '
      PRINT *,'              none     |  parameters must be set individually                '
      PRINT *,'              ---------------------------------------------------------------'
-     PRINT *,'          DEFAULT depcode is : ',TRIM(cldepcode)
+     PRINT *,'          DEFAULT code is : ',TRIM(cldepcode)
      PRINT *,'              For other setting use the options to specify the settings'
      PRINT *,'              individually.'
      PRINT *,'       [-depref depref ] : give the depth reference for potential density'
@@ -123,67 +134,73 @@ PROGRAM cdftransig_xy3d
      PRINT *,'              s-scalr bin width.'
      PRINT *,'       [-full ] : indicate a full step configuration.'
      PRINT *,'       [-v ] : verbose mode : extra print are performed during execution.'
+     PRINT *,'       [-vvl ] : use time-varying vertical metrics.'
+     PRINT *,'       [-o OUT-file ] : specify output file name instead of ',TRIM(cf_out)
+     PRINT *,'       [-nc4 ] : Use netcdf4 output with chunking and deflation level 1.'
+     PRINT *,'               This option is effective only if cdftools are compiled with'
+     PRINT *,'               a netcdf library supporting chunking and deflation.'
      PRINT *,'      '
      PRINT *,'     REQUIRED FILES :'
      PRINT *,'       ',TRIM(cn_fhgr),' and ',TRIM(cn_fzgr) 
      PRINT *,'      '
      PRINT *,'     OUTPUT : '
-     PRINT *,'       netcdf file : ', TRIM(cf_out) 
+     PRINT *,'       netcdf file : ', TRIM(cf_out) ,' unless -o option is used.'
      PRINT *,'         variables : ',TRIM(cv_outu),' and ', TRIM(cv_outv),' in m3/s.'
      PRINT *,'      '
      PRINT *,'     SEE ALSO :'
-     PRINT *,'      cdfrhoproj, cdfsigtrp' 
+     PRINT *,'      cdfrhoproj, cdfsigtrp ' 
      PRINT *,'      '
-     STOP
+     STOP 
   ENDIF
 
   ! browse command line according to options
-  ijarg = 1 ; ireq = 0 ; ntags = 0 ; iset = 0
+  ijarg = 1 ;  iset = 0
   DO WHILE ( ijarg <= narg ) 
      CALL getarg(ijarg, cldum ) ; ijarg=ijarg+1
      SELECT CASE ( cldum )
-     CASE ( '-depcode' ) ; CALL getarg(ijarg, cldepcode ) ; ijarg=ijarg+1
-     CASE ( '-depref'  ) ; CALL getarg(ijarg, cldum     ) ; ijarg=ijarg+1 ; READ(cldum,*) pref       ; iset = iset+1
-                           WRITE(clsigma,'("sigma_",I1)'), NINT(pref/1000.)
+     CASE ( '-c'       ) ; CALL getarg(ijarg, config    ) ; ijarg=ijarg+1
+     CASE ( '-l'       ) ; CALL GetTagList
+     CASE ( '-code'    ) ; CALL getarg(ijarg, cldepcode ) ; ijarg=ijarg+1                            
+     CASE ( '-depref'  ) ; CALL getarg(ijarg, cldum     ) ; ijarg=ijarg+1 ; READ(cldum,*) zpref       ; iset = iset+1
+        ;                  WRITE(clsigma,'("sigma_",I1)'), NINT(zpref/1000.)
      CASE ( '-nbins'   ) ; CALL getarg(ijarg, cldum     ) ; ijarg=ijarg+1 ; READ(cldum,*) nbins      ; iset = iset+1
      CASE ( '-sigmin'  ) ; CALL getarg(ijarg, cldum     ) ; ijarg=ijarg+1 ; READ(cldum,*) ds1min
      CASE ( '-sigzoom' ) ; CALL getarg(ijarg, cldum     ) ; ijarg=ijarg+1 ; READ(cldum,*) ds1zoom    ; iset = iset+1
-                           CALL getarg(ijarg, cldum     ) ; ijarg=ijarg+1 ; READ(cldum,*) ds1scalmin
+        ;                  CALL getarg(ijarg, cldum     ) ; ijarg=ijarg+1 ; READ(cldum,*) ds1scalmin
+     CASE ( '-S'       ) ; lsal   = .TRUE.
      CASE ( '-full'    ) ; lfull  = .TRUE.
      CASE ( '-v'       ) ; lprint = .TRUE.
-     CASE DEFAULT    ! mandatory arguments 
-        ireq=ireq+1      
-        SELECT CASE (ireq)
-        CASE ( 1 ) ;  config=cldum
-        CASE DEFAULT
-         IF ( ntags == 0 ) istag = ijarg - 1 ! remember the argument number corresponding to 1rst tag
-         ntags=ntags + 1
-        END SELECT
+     CASE ( '-vvl'     ) ; lg_vvl = .TRUE. 
+     CASE ( '-o'       ) ; CALL getarg(ijarg, cf_out    ) ; ijarg=ijarg+1 
+     CASE ( '-nc4'     ) ; lnc4   = .TRUE.
+     CASE DEFAULT        ; PRINT *,' ERROR : ', TRIM(cldum),' : unkown option.' ; STOP 99
      END SELECT
   ENDDO
 
   ! set parameters for pre-defined depcode
   SELECT CASE ( cldepcode )
   CASE ( '0'                    ) 
-      pref = 0.    ; nbins = 101 ; ds1min = 23.0 ; ds1scal = 0.03 ; ds1zoom = 999. ; ds1scalmin = 999. ; clsigma='sigma_0'
+      zpref = 0.    ; nbins = 101 ; ds1min = 23.0d0 ; ds1scal = 0.03d0 ; ds1zoom = 999.d0 ; ds1scalmin = 999.d0 ; clsigma='sigma_0'
   CASE ( '1000'                 ) 
-      pref = 1000. ; nbins =  93 ; ds1min = 24.2d0 ; ds1scal = 0.10d0 ; ds1zoom = 32.3d0 ; ds1scalmin = 0.05d0 ; clsigma='sigma_1'
+      zpref = 1000. ; nbins =  93 ; ds1min = 24.2d0 ; ds1scal = 0.10d0 ; ds1zoom = 32.3d0 ; ds1scalmin = 0.05d0 ; clsigma='sigma_1'
   CASE ( '1000-acc', '1000-ACC' ) 
-      pref = 1000. ; nbins =  88 ; ds1min = 24.5 ; ds1scal = 0.10 ; ds1zoom = 999. ; ds1scalmin = 999. ; clsigma='sigma_1'
+      zpref = 1000. ; nbins =  88 ; ds1min = 24.5d0 ; ds1scal = 0.10d0 ; ds1zoom = 999.d0 ; ds1scalmin = 999.d0 ; clsigma='sigma_1'
   CASE ( '2000'                 ) 
-      pref = 2000. ; nbins = 174 ; ds1min = 29.0 ; ds1scal = 0.05 ; ds1zoom = 999. ; ds1scalmin = 999. ; clsigma='sigma_2'
+      zpref = 2000. ; nbins = 174 ; ds1min = 29.0d0 ; ds1scal = 0.05d0 ; ds1zoom = 999.d0 ; ds1scalmin = 999.d0 ; clsigma='sigma_2'
   CASE ( 'none'                 ) 
       ! in this case check that all parameters are set individually
-      IF ( iset /= 3  ) THEN 
-         PRINT *, ' You must set depref, nbins, sigmin  individually' ; STOP 99
+      IF ( iset /= 3  ) THEN  ; PRINT *, ' You must set depref, nbins, sigmin  individually'    ; STOP 99 
       ENDIF
-  CASE DEFAULT 
-      PRINT *, ' this depcode :',TRIM(cldepcode),' is not available.' ; STOP 99
+  CASE DEFAULT                ; PRINT *, ' This depcode :',TRIM(cldepcode),' is not available.' ; STOP 99 
   END SELECT
 
+  lchk = lchk .OR. chkfile (cn_fzgr)
+  lchk = lchk .OR. chkfile (cn_fhgr)
+  IF ( lchk ) STOP 99 ! missing files
+
   ds1scalmin = MIN ( ds1scalmin, ds1scal )
   IF ( lprint ) THEN
-     PRINT *,' DEP REF  : ', pref, ' m'
+     PRINT *,' DEP REF  : ', zpref, ' m'
      PRINT *,' NBINS    : ', nbins
      PRINT *,' SIGMIN   : ', ds1min
      PRINT *,' SIGSTP   : ', ds1scal
@@ -191,8 +208,9 @@ PROGRAM cdftransig_xy3d
      PRINT *,' SIGSTP R : ', ds1scalmin
   ENDIF
   ! use first tag to look for file dimension
-  CALL getarg (istag, ctag)
+  ctag=ctag_lst(1)
   cf_vfil = SetFileName (config, ctag, 'V' )
+
   IF ( chkfile(cf_vfil) ) STOP 99 ! missing file
 
   npiglo = getdim (cf_vfil, cn_x)
@@ -242,18 +260,6 @@ PROGRAM cdftransig_xy3d
   ! define output variables
   CALL SetGlobalAtt(cglobal)
 
-  ipk(:)                    = nbins   ! output file has  nbins sigma values
-  stypvar%cunits            = 'm3/s'  ! transports
-  stypvar%rmissing_value    = 0.
-  stypvar%valid_min         = -10.    ! seem to be small
-  stypvar%valid_max         = 10.
-  stypvar%conline_operation = 'N/A'
-  stypvar%caxis             = 'TSYX'
-
-  stypvar(1)%cname          = cv_outu                ;    stypvar(2)%cname       = cv_outv
-  stypvar(1)%clong_name     = 'Zonal_trsp_sig_coord' ;    stypvar(2)%clong_name  = 'Meridional_trsp_sig_coord'
-  stypvar(1)%cshort_name    = cv_outu                ;    stypvar(2)%cshort_name = cv_outv
-
   PRINT *, 'npiglo = ', npiglo
   PRINT *, 'npjglo = ', npjglo
   PRINT *, 'npk    = ', npk
@@ -285,11 +291,7 @@ PROGRAM cdftransig_xy3d
      e31d(:)  = getvare3(cn_fzgr, cn_ve3t1d, npk              )
   ENDIF
 
-  ! create output fileset
-  IF (lprint) PRINT *, ' ready to create file:',TRIM( cf_out), ' from reference:',TRIM(cf_vfil )
-  ncout = create      (cf_out, cf_vfil, npiglo, npjglo, nbins,    cdep=clsigma            )
-  ierr  = createvar   (ncout,  stypvar, 2,      ipk,    id_varout, cdglobal=TRIM(cglobal) )
-  ierr  = putheadervar(ncout,  cf_vfil, npiglo, npjglo, nbins,     pdep=REAL(dsigma)      )
+  CALL CreateOutput
 
   dtotal_time = 0.d0
 
@@ -299,38 +301,56 @@ PROGRAM cdftransig_xy3d
   ! 
   DO jk= 1, npk-1
      IF ( lprint ) PRINT *, ' working on depth jk=',jk
-     IF ( lfull ) THEN 
-       e3v(:,:) = e31d(jk)
-       e3u(:,:) = e31d(jk)
-     ELSE
-       e3v(:,:) = getvar(cn_fzgr, 'e3v_ps', jk, npiglo,npjglo )
-       e3u(:,:) = getvar(cn_fzgr, 'e3u_ps', jk, npiglo,npjglo )
-     ENDIF
 
-     ijarg = istag ; nframes = 0
+     nframes = 0
      DO jtag = 1, ntags
-
-        CALL getarg (ijarg, ctag) ; ijarg = ijarg + 1
+        ctag = ctag_lst(jtag)
         IF (lprint   ) PRINT *, ' working on  ctag=',TRIM(ctag)
         cf_tfil = SetFileName(config, ctag, 'T') 
+        IF ( lsal ) THEN 
+           cf_sfil = SetFileName(config, ctag, 'S')
+        ELSE
+           cf_sfil = cf_tfil
+        ENDIF
         cf_ufil = SetFileName(config, ctag, 'U')
         cf_vfil = SetFileName(config, ctag, 'V')
 
+        IF ( lg_vvl ) THEN
+          cn_fe3u = cf_ufil
+          cn_fe3v = cf_vfil
+          cn_ve3u = cn_ve3uvvl
+          cn_ve3v = cn_ve3vvvl
+        ENDIF
+
         ! check existence of files
         lchk =           chkfile ( cf_tfil) 
+        lchk = lchk .OR. chkfile ( cf_sfil) 
         lchk = lchk .OR. chkfile ( cf_ufil) 
         lchk = lchk .OR. chkfile ( cf_vfil) 
+        IF ( lg_vvl ) lchk = lchk .OR. chkfile ( cf_vfil) 
         IF ( lchk ) STOP 99 ! missing file
         
         IF (jk== 1 ) THEN
            npt = getdim (cf_tfil, cn_t)  ! assuming all files (U V ) contains same number of time frame
-           ALLOCATE ( tim(npt) )
-           tim         = getvar1d(cf_tfil, cn_vtimec, npt )
-           dtotal_time = dtotal_time + SUM( DBLE(tim) )
-           DEALLOCATE ( tim )
+           ALLOCATE ( dtim(npt) )
+           dtim        = getvar1d(cf_tfil, cn_vtimec, npt )
+           dtotal_time = dtotal_time + SUM(dtim )
+           DEALLOCATE ( dtim )
         ENDIF
 
         DO jt = 1, npt
+           IF ( lg_vvl ) THEN ; it=jt
+           ELSE               ; it=1
+           ENDIF
+           IF ( it == jt ) THEN ! read e3 at each time step ( vvl case)
+             IF ( lfull ) THEN 
+               e3v(:,:) = e31d(jk)
+               e3u(:,:) = e31d(jk)
+             ELSE
+               e3v(:,:) = getvar(cn_fe3v, cn_ve3v, jk, npiglo,npjglo, ktime=it, ldiom=.NOT.lg_vvl )
+               e3u(:,:) = getvar(cn_fe3u, cn_ve3u, jk, npiglo,npjglo ,ktime=it, ldiom=.NOT.lg_vvl)
+            ENDIF
+          ENDIF
            nframes = nframes + 1
            ! Get velocities u, v  and mask   if first time slot only 
            zv(:,:)= getvar ( cf_vfil, cn_vomecrty, jk, npiglo, npjglo, ktime=jt )
@@ -338,17 +358,17 @@ PROGRAM cdftransig_xy3d
            IF (jtag == 1) THEN 
               zmasku(:,:)= 1; zmaskv(:,:)= 1.0 ;
               WHERE( zu == 0) zmasku(:,:)= 0.0 ;
-              WHERE( zv == 0) zmaskv(:,:)= 0.0;
+              WHERE( zv == 0) zmaskv(:,:)= 0.0 ;
               IF (lprint  ) PRINT *, ' min,max u:',MINVAL(zu),MAXVAL(zu)
            ENDIF
            !                     density  
            zt(:,:)= getvar ( cf_tfil, cn_votemper, jk, npiglo, npjglo, ktime=jt )
-           zs(:,:)= getvar ( cf_tfil, cn_vosaline, jk, npiglo, npjglo, ktime=jt )
+           zs(:,:)= getvar ( cf_sfil, cn_vosaline, jk, npiglo, npjglo, ktime=jt )
 
-           IF ( pref == 0. ) THEN
+           IF ( zpref == 0. ) THEN
               dens2d = sigma0(zt, zs,       npiglo, npjglo)
            ELSE
-              dens2d = sigmai(zt, zs, pref, npiglo, npjglo)
+              dens2d = sigmai(zt, zs, zpref, npiglo, npjglo)
            ENDIF
 
            !  density on u points masked by u  , single precision 
@@ -393,8 +413,8 @@ PROGRAM cdftransig_xy3d
      ! -----------------  end of loop on jk
   END DO
 
-  timean(1) = dtotal_time/nframes
-  ierr      = putvar1d(ncout, timean, 1, 'T')
+  dtimean(1) = dtotal_time/nframes
+  ierr      = putvar1d(ncout, dtimean, 1, 'T')
   DO jk=1, nbins
      zt   = dusigsig(:,:,jk) / nframes
      ierr = putvar (ncout, id_varout(1), zt, jk, npiglo, npjglo, kwght=nframes)
@@ -405,6 +425,70 @@ PROGRAM cdftransig_xy3d
   ENDDO
   ierr = closeout(ncout)
 
+CONTAINS
+
+  SUBROUTINE CreateOutput
+    !!---------------------------------------------------------------------
+    !!                  ***  ROUTINE CreateOutput  ***
+    !!
+    !! ** Purpose :  Create netcdf output file(s) 
+    !!
+    !! ** Method  :  Use stypvar global description of variables
+    !!
+    !!----------------------------------------------------------------------
+  ipk(:)                    = nbins   ! output file has  nbins sigma values
+  stypvar%cunits            = 'm3/s'  ! transports
+  stypvar%rmissing_value    = 0.
+  stypvar%valid_min         = -10.    ! seem to be small
+  stypvar%valid_max         = 10.
+  stypvar%conline_operation = 'N/A'
+  stypvar%caxis             = 'TSYX'
+
+  stypvar(1)%ichunk            = (/npiglo,MAX(1,npjglo/30),1,1 /)
+  stypvar(2)%ichunk            = (/npiglo,MAX(1,npjglo/30),1,1 /)
+
+  stypvar(1)%cname          = cv_outu                ;    stypvar(2)%cname       = cv_outv
+  stypvar(1)%clong_name     = 'Zonal_trsp_sig_coord' ;    stypvar(2)%clong_name  = 'Meridional_trsp_sig_coord'
+  stypvar(1)%cshort_name    = cv_outu                ;    stypvar(2)%cshort_name = cv_outv
+
+  ! create output fileset
+  IF (lprint) PRINT *, ' ready to create file:',TRIM( cf_out), ' from reference:',TRIM(cf_vfil )
+
+  ncout = create      (cf_out, cf_vfil, npiglo, npjglo, nbins,    cdep=clsigma           , ld_nc4=lnc4  )
+  ierr  = createvar   (ncout,  stypvar, 2,      ipk,    id_varout, cdglobal=TRIM(cglobal), ld_nc4=lnc4  )
+  ierr  = putheadervar(ncout,  cf_vfil, npiglo, npjglo, nbins,     pdep=REAL(dsigma)      )
+
+  END SUBROUTINE CreateOutput
+
+  SUBROUTINE GetTagList
+    !!---------------------------------------------------------------------
+    !!                  ***  ROUTINE GetTagList  ***
+    !!
+    !! ** Purpose :  Set up a tag list given on the command line as 
+    !!               blank separated list
+    !!
+    !! ** Method  :  Scan the command line until a '-' is found
+    !!----------------------------------------------------------------------
+    INTEGER (KIND=4)  :: ji
+    INTEGER (KIND=4)  :: icur
+    !!----------------------------------------------------------------------
+    !!
+    ntags=0
+    ! need to read a list of file ( number unknow ) 
+    ! loop on argument till a '-' is found as first char
+    icur=ijarg                          ! save current position of argument number
+    DO ji = icur, narg                  ! scan arguments till - found
+       CALL getarg ( ji, cldum )
+       IF ( cldum(1:1) /= '-' ) THEN ; ntags = ntags+1
+       ELSE                          ; EXIT
+       ENDIF
+    ENDDO
+    ALLOCATE (ctag_lst(ntags) )
+    DO ji = icur, icur + ntags -1
+       CALL getarg(ji, ctag_lst( ji -icur +1 ) ) ; ijarg=ijarg+1
+    END DO
+  END SUBROUTINE GetTagList
+
 END PROGRAM cdftransig_xy3d
 
    
