@@ -17,6 +17,7 @@ PROGRAM cdfweight
   !! History : 2.0  : 11/2005  : J.M. Molines : Original code
   !!                : 05/2007  : J.M. Molines : for weight
   !!           3.0  : 03/2011  : J.M. Molines : Doctor norm + Lic.
+  !!         : 4.0  : 03/2017  : J.M. Molines  
   !!----------------------------------------------------------------------
   !!----------------------------------------------------------------------
   !!   routines      : description
@@ -26,12 +27,14 @@ PROGRAM cdfweight
   !!----------------------------------------------------------------------
   USE cdfio
   USE cdftools       ! cdf_find_ij
+  USE modutils
   USE modcdfnames
   !!----------------------------------------------------------------------
-  !! CDFTOOLS_3.0 , MEOM 2011
+  !! CDFTOOLS_4.0 , MEOM 2017 
   !! $Id$
-  !! Copyright (c) 2011, J.-M. Molines
+  !! Copyright (c) 2017, J.-M. Molines 
   !! Software governed by the CeCILL licence (Licence/CDFTOOLSCeCILL.txt)
+  !! @class data_transformation
   !!----------------------------------------------------------------------
   IMPLICIT NONE
 
@@ -45,7 +48,7 @@ PROGRAM cdfweight
   INTEGER(KIND=4)                           :: numgreg=10              ! logical unit of ASCII input file
   INTEGER(KIND=4)                           :: numbin=20               ! logical unit of BINARY weight file
   INTEGER(KIND=4)                           :: ios                     ! iostat variable
-                                                                       ! Greg Holloway input data  ( 5 variables)
+  ! Greg Holloway input data  ( 5 variables)
   INTEGER(KIND=4)                           :: id                      ! station ID
   REAL(KIND=4)                              :: xmin, ymin, rdep        ! longitude, latitude, depth
 
@@ -58,7 +61,7 @@ PROGRAM cdfweight
   REAL(KIND=8)                              :: dl_lamS, dl_phiS, dl_hS ! grid point South of NP, true heading from NP
   REAL(KIND=8)                              :: dl_lamW, dl_phiW, dl_hW ! grid point West of NP, true heading from NP
   REAL(KIND=8), DIMENSION(0:4)              :: dl_lami, dl_phii        ! the 4 grid points around target (1-4)
-                                                                       !  + the target (0)
+  !  + the target (0)
   REAL(KIND=8), DIMENSION(:,:), ALLOCATABLE :: dl_lam, dl_phi          ! grid layout and metrics
   REAL(KIND=8), DIMENSION(:),   ALLOCATABLE :: dl_dept                 ! vertical depth
   REAL(KIND=8)                              :: dl_hP                   ! true heading of target point from NP
@@ -81,21 +84,20 @@ PROGRAM cdfweight
 
   narg= iargc()
   IF ( narg == 0 ) THEN
-     PRINT *,' usage : cdfweight  [-f] IN-file [-c COORD-file] ... '
-     PRINT *,'              ...  [-t point_type] [-2d] [-v] '
+     PRINT *,' usage : cdfweight  -f IN-file [-c COORD-file] [-p C-type] [-2d] [-v] '
      PRINT *,'      '
      PRINT *,'     PURPOSE :'
-     PRINT *,'       Produce a weight file for further bilinear collocalisation ' 
-     PRINT *,'       with cdfcoloc program. It takes the position of the points'
-     PRINT *,'       to be collocated into a simple ascii file. '
+     PRINT *,'       Produces a weight file for further bilinear colocalisation with ' 
+     PRINT *,'       cdfcoloc program. It takes the position of the points to be '
+     PRINT *,'       colocated into a simple ascii file. '
      PRINT *,'      '
      PRINT *,'     ARGUMENTS :'
-     PRINT *,'       [-f ] IN-file   : input file is a iyxz ASCII file, 1 line per point.'  
+     PRINT *,'       -f  IN-file   : input file is a iyxz ASCII file, 1 line per point.'  
      PRINT *,'      '
      PRINT *,'     OPTIONS :'
      PRINT *,'       [-c COORD-file] : coordinate file [',TRIM(cf_coord),']'
-     PRINT *,'       [-t point_type] : point type on C-grid (either T U V or F ) [',TRIM(ctype),']'
-     PRINT *,'       [-2d ]          : tell cdfweight that only 2D weights are to be computed.'
+     PRINT *,'       [-p C-type    ] : point type on C-grid (either T U V or F ) [',TRIM(ctype),']'
+     PRINT *,'       [-2d ]          : tell cdfweight that only 2D weights will be computed.'
      PRINT *,'       [-v ]           : Verbose mode for extra information (debug mode).'
      PRINT *,'      '
      PRINT *,'     REQUIRED FILES :'
@@ -106,7 +108,11 @@ PROGRAM cdfweight
      PRINT *,'       binary weight file : weight_point_type.bin'
      PRINT *,'       standard output : almost the same info that is saved in the binary file'
      PRINT *,'                   When using -v option, even more informations !'
-     STOP
+     PRINT *,'      '
+     PRINT *,'     SEE ALSO : '
+     PRINT *,'        cdfcoloc'
+     PRINT *,'      '
+     STOP 
   ENDIF
 
   iarg=1
@@ -114,11 +120,12 @@ PROGRAM cdfweight
      CALL getarg(iarg, cldum) ; iarg=iarg+1
      SELECT CASE ( cldum )
      CASE ('-f' ) ; CALL getarg(iarg, cf_in   ) ; iarg=iarg+1 
+        ! options
      CASE ('-c' ) ; CALL getarg(iarg, cf_coord) ; iarg=iarg+1 
-     CASE ('-t' ) ; CALL getarg(iarg, ctype   ) ; iarg=iarg+1 
+     CASE ('-p' ) ; CALL getarg(iarg, ctype   ) ; iarg=iarg+1 
      CASE ('-2d') ; ll2d    = .TRUE.
      CASE ('-v' ) ; lldebug = .TRUE.
-     CASE DEFAULT ; CALL getarg(iarg, cf_in   ) ; iarg=iarg+1  ! if no switch assume file name
+     CASE DEFAULT ; PRINT *,' ERROR : ',TRIM(cldum),' unknown option.' ; STOP 99
      END SELECT
   END DO
 
@@ -382,48 +389,4 @@ CONTAINS
     det = dp1*dp4 - dp2*dp3
   END FUNCTION det
 
-  FUNCTION heading(dplona, dplonb, dplata, dplatb)
-    !!---------------------------------------------------------------------
-    !!                  ***  FUNCTION heading  ***
-    !!
-    !! ** Purpose : Compute true heading between point a and b
-    !!
-    !! ** Method  : Suppose that the 2 points are not too far away 
-    !!              from each other so that heading can be computed 
-    !!              with loxodromy.
-    !!
-    !!----------------------------------------------------------------------
-    REAL(KIND=8), INTENT(in) :: dplata, dplona ! lat lon of point a
-    REAL(KIND=8), INTENT(in) :: dplatb, dplonb ! lat lon of point b
-    REAL(KIND=8)             :: heading        ! return value in degree
-
-    REAL(KIND=8)             :: dlpi, dlconv   ! pi and conversion factor
-    REAL(KIND=8)             :: dlxa,dlya      ! working variable
-    REAL(KIND=8)             :: dlxb,dlyb      ! working variable
-    REAL(KIND=8)             :: dlxb_xa        !  ""        ""
-    !!----------------------------------------------------------------------
-
-    dlpi   = ACOS(-1.d0)
-    dlconv = dlpi/180.d0  ! for degree to radian conversion
-
-    ! there is a problem if the Greenwich meridian pass between a and b
-    IF ( lldebug) PRINT *,' Plonb  Plona ' , dplonb, dplona
-    dlxa = dplona*dlconv
-    dlxb = dplonb*dlconv
-
-    dlya = -LOG(TAN(dlpi/4.-dlconv*dplata/2.d0))
-    dlyb = -LOG(TAN(dlpi/4.-dlconv*dplatb/2.d0))
-
-    IF (lldebug) PRINT *,' dlxa_xb , modulo 2pi', dlxb-dlxa, MOD((dlxb-dlxa),2*dlpi)
-    dlxb_xa = MOD((dlxb-dlxa),2*dlpi)
-
-    IF ( dlxb_xa >=  dlpi ) dlxb_xa = dlxb_xa -2*dlpi
-    IF ( dlxb_xa <= -dlpi ) dlxb_xa = dlxb_xa +2*dlpi
-    IF (lldebug)  PRINT *, 'dlyb -dlya, dlxb_xa ',dlyb -dlya , dlxb_xa
-
-    heading=ATAN2(dlxb_xa,(dlyb-dlya))*180.d0/dlpi
-
-    IF (heading < 0) heading=heading+360.d0
-  END FUNCTION heading
-
 END PROGRAM cdfweight
