@@ -5,6 +5,7 @@ MODULE modutils
   !!=====================================================================
   !! History : 3.0  : 04/2011  : J.M. Molines : Original code
   !!                : 10/2012  : N. Ferry, E. Durand, F. Hernandez : add shapiro
+  !!         : 4.0  : 03/2017  : J.M. Molines  
   !!----------------------------------------------------------------------
   !!----------------------------------------------------------------------
   !!   routines      : description
@@ -13,10 +14,11 @@ MODULE modutils
   !!   shapiro_fill_smooth : shapiro smoother or filler
   !!----------------------------------------------------------------------
   !!----------------------------------------------------------------------
-  !! CDFTOOLS_3.0 , MEOM 2011
+  !! CDFTOOLS_4.0 , MEOM 2017 
   !! $Id$
-  !! Copyright (c) 2010, J.-M. Molines
+  !! Copyright (c) 2017, J.-M. Molines 
   !! Software governed by the CeCILL licence (Licence/CDFTOOLSCeCILL.txt)
+  !! @class system
   !!----------------------------------------------------------------------
   USE cdfio
 
@@ -27,8 +29,10 @@ MODULE modutils
   PUBLIC SetFileName
   PUBLIC GetList
   PUBLIC shapiro_fill_smooth
+  PUBLIC FillPool2D_full
   PUBLIC FillPool2D
   PUBLIC FillPool3D
+  PUBLIC heading         ! compute true heading between point A and B
 
 CONTAINS
   SUBROUTINE SetGlobalAtt(cdglobal, cd_append)
@@ -136,20 +140,16 @@ CONTAINS
 
        iposm=INDEX(cldum2,'-')
        IF ( iposm == 0 ) THEN
-          ksiz=ksiz+1 ; 
-          IF (ksiz > jp_maxlist) THEN 
-             PRINT *, 'jp_maxlist too small in getlist '
-             STOP 97
+          ksiz=ksiz+1 
+          IF (ksiz > jp_maxlist) THEN ; PRINT *, 'jp_maxlist too small in getlist ' ; STOP 97
           ENDIF
           READ(cldum2,* ) itmp(ksiz)
        ELSE
           READ(cldum2(1:iposm-1),*) ik1
           READ(cldum2(iposm+1:),* ) ik2
           DO jk = ik1,ik2
-             ksiz=ksiz+1 ; 
-             IF (ksiz > jp_maxlist) THEN
-                PRINT *, 'jp_maxlist too small in getlist '
-                STOP 97
+             ksiz=ksiz+1 
+             IF (ksiz > jp_maxlist) THEN ; PRINT *, 'jp_maxlist too small in getlist ' ; STOP 97
              ENDIF
              itmp(ksiz)=jk
           ENDDO
@@ -213,7 +213,7 @@ CONTAINS
     ilmasktrue_e(kpi+1,:) = ilmasktrue_e(1,:)      
 
     ! check cdfs compliance
-    IF ( cdfs(1:4)  .NE. 'fill' .AND. cdfs(1:6) .NE. 'smooth' ) THEN
+    IF ( cdfs(1:4)  /= 'fill' .AND. cdfs(1:6) /= 'smooth' ) THEN
        PRINT*, 'cdfs = ',cdfs ,' <> fill or smooth'
        STOP 97
     ENDIF
@@ -309,11 +309,160 @@ CONTAINS
 
   END SUBROUTINE shapiro_fill_smooth
 
-  SUBROUTINE FillPool2D(kiseed, kjseed, kdta, kifill)
+  SUBROUTINE FillPool2D_full(kcrit, pdta, kiimin, kiimax, kijmin, kijmax, &
+     &           kiseed, kjseed, pfillmin, pfillmax, pfillval, ld_perio)
+    !!---------------------------------------------------------------------
+    !!                  ***  ROUTINE FillPool2d_full ***
+    !!
+    !! ** Purpose :  Replace all area surrounding by mask value by mask value
+    !!
+    !! ** Method  :  flood fill algorithm
+    !!
+    !!----------------------------------------------------------------------
+    INTEGER(KIND=4),              INTENT(in   ) :: kcrit                         ! max size of the pool
+    REAL(KIND=4), DIMENSION(:,:), INTENT(inout) :: pdta                          ! mask
+    INTEGER(KIND=4),              INTENT(in   ) :: kiimin, kiimax, kijmin, kijmax    ! position of the data windows
+    INTEGER(KIND=4),              INTENT(in   ) :: kiseed, kjseed                ! seeds
+    REAL(KIND=4),                 INTENT(in   ) :: pfillmax, pfillmin, pfillval  ! pool def criterium
+    LOGICAL,                      INTENT(in   ) :: ld_perio                      ! EW periodicity flag
+
+    INTEGER(KIND=4) :: ik                       ! number of point change
+    INTEGER(KIND=4) :: ip                       ! size of the pile
+    INTEGER(KIND=4) :: ji, jj, ii, ij           ! loop index
+    INTEGER(KIND=4) :: iip1, iim1, ijp1, ijm1   ! working integer
+    INTEGER(KIND=4) :: iimin, iimax, ijmin, ijmax   ! position of the data windows
+    INTEGER(KIND=4) :: ipiglo, ipjglo           ! size of the domain, infered from pdta size
+    INTEGER(KIND=4), DIMENSION(:,:), ALLOCATABLE :: ipile         ! pile variable
+    INTEGER(KIND=4), DIMENSION(:,:), ALLOCATABLE :: ioptm, ifill  ! matrix to check already tested value 
+
+    REAL(KIND=4), DIMENSION(:,:),    ALLOCATABLE :: zdata         ! new data
+    !!----------------------------------------------------------------------
+    ! WARNING
+    IF (ld_perio) PRINT *, 'W A R N I N G: north fold not treated properly ...'
+
+    ! allocate variable
+
+    ! infer domain size from input array
+    ipiglo = SIZE(pdta,1)
+    ipjglo = SIZE(pdta,2)
+
+    ! allocate variable
+    ALLOCATE(ipile(ipiglo*ipjglo,2))
+    ALLOCATE(zdata(ipiglo,ipjglo))
+    ALLOCATE(ioptm(ipiglo,ipjglo),ifill(ipiglo,ipjglo))
+
+    ! define initial seeds
+    ioptm(:,:) = 0
+    WHERE ( pdta  > pfillmin .AND. pdta < pfillmax)
+      ioptm = 1
+    END WHERE
+
+    IF (kiseed > 0 .AND. kjseed > 0 ) THEN
+       iimin=kiseed
+       iimax=kiseed
+       ijmin=kjseed
+       ijmax=kjseed
+       IF (ioptm(kiseed,kjseed) == 0) THEN
+          PRINT *, 'seed not in a suitable location (ioptm(kiseed,kjseed) == 0)'
+          STOP 97
+       END IF
+    ELSEIF (kiseed < 0 .AND. kjseed < 0 ) THEN
+       iimin=kiimin ; ijmin=kijmin
+       iimax=kiimax ; ijmax=kijmax
+    ELSE
+       PRINT *, 'case kiseed > 0 and kjseed < 0 not treated'
+       STOP 97
+    END IF
+
+    PRINT *, 'Filling area in progress ... (it can take a while)'    
+
+    ! initialise variables
+    zdata=pdta
+    ipile(:,:)=0
+
+    DO ji=iimin,iimax
+       IF (MOD(ji,100) == 0) PRINT *, ji,'/',iimax
+       DO jj=ijmin,ijmax
+
+          ip=0
+
+          ! initialised pile
+          IF (ioptm(ji,jj) == 1) THEN
+             ifill(:,:) = 0
+             zdata(:,:) = pdta(:,:)
+             ipile(1,:) = [ji,jj]
+             ip = 1
+          END IF
+          ik = 0
+
+          ! loop until the pile size is 0 or if the pool is larger than the critical size
+          DO WHILE ( ip /= 0 );
+             ! 
+             ! update size of the pool
+             ik = ik+1
+
+             ! next point
+             ii=ipile(ip,1); ij=ipile(ip,2)
+
+             ! update pile size
+             ipile(ip,:)  =[0,0]; ip=ip-1
+
+             ! check neighbour cells and update pile ( assume E-W periodicity )
+             IF ( ld_perio ) THEN
+                IF ( ii == ipiglo+1 ) ii=3
+                IF ( ii == 0        ) ii=ipiglo-2
+                iip1=ii+1; IF ( iip1 == ipiglo+1) iip1=3
+                iim1=ii-1; IF ( iim1 == 0       ) iim1=ipiglo-2
+             ELSE
+                IF ( ii == ipiglo+1 ) ii=ipiglo
+                IF ( ii == 0        ) ii=1
+                iip1=ii+1; IF ( iip1 == ipiglo+1) iip1=ipiglo
+                iim1=ii-1; IF ( iim1 == 0       ) iim1=1
+             END IF
+             ijp1=MIN(ij+1,ipjglo)  ! north fold not treated
+             ijm1=MAX(ij-1,1)
+
+             ! update ifill
+             ifill(ii,ij) = 1
+
+             ! check neighbour cells and update pile
+             IF (zdata(ii, ijp1) > pfillmin .AND. zdata(ii, ijp1) < pfillmax .AND. ioptm(ii, ijp1) == 1) THEN
+                ip=ip+1; ipile(ip,:)=[ii  ,ijp1]
+                ioptm (ii,ijp1) = 0
+             END IF
+             IF (zdata(ii, ijm1) > pfillmin .AND. zdata(ii, ijm1) < pfillmax .AND. ioptm(ii, ijm1) == 1) THEN
+                ip=ip+1; ipile(ip,:)=[ii  ,ijm1]
+                ioptm (ii,ijm1) = 0
+             END IF
+             IF (zdata(iip1, ij) > pfillmin .AND. zdata(iip1, ij) < pfillmax .AND. ioptm(iip1, ij) == 1) THEN
+                ip=ip+1; ipile(ip,:)=[iip1,ij  ]
+                ioptm (iip1,ij) = 0
+             END IF
+             IF (zdata(iim1, ij) > pfillmin .AND. zdata(iim1, ij) < pfillmax .AND. ioptm(iim1, ij) == 1) THEN
+                ip=ip+1; ipile(ip,:)=[iim1,ij  ]
+                ioptm (iim1,ij) = 0
+             END IF
+          END DO
+          !
+          ! check size to fill only small pool
+          IF ( 0 < ik .AND. ik < kcrit) THEN
+             IF (ik > 0 ) PRINT *, 'Fill area size : ',ik,' seed : ',ji, jj
+             WHERE (ifill(:,:) == 1)
+               pdta(:,:)=pfillval;
+             END WHERE
+          END IF
+       END DO
+    END DO
+
+    DEALLOCATE(ipile); DEALLOCATE(zdata, ioptm)
+
+  END SUBROUTINE FillPool2D_full
+
+  SUBROUTINE FillPool2D(kiseed, kjseed, kdta, kifill, ld_perio, ld_diagonal)
     !!---------------------------------------------------------------------
     !!                  ***  ROUTINE FillPool2D  ***
     !!  
-    !! ** Purpose :  Replace all area surrounding by mask value by mask value
+    !! ** Purpose :  Replace all area surrounding by mask value by kifill value
     !!  
     !! ** Method  :  flood fill algorithm
     !!  
@@ -321,16 +470,25 @@ CONTAINS
     INTEGER(KIND=4),                 INTENT(in)    :: kiseed, kjseed
     INTEGER(KIND=4),                 INTENT(in)    :: kifill         ! pool value
     INTEGER(KIND=2), DIMENSION(:,:), INTENT(inout) :: kdta           ! mask
+    LOGICAL, OPTIONAL              , INTENT(in)    :: ld_perio       ! treat EW peridocity
+    LOGICAL, OPTIONAL              , INTENT(in)    :: ld_diagonal    ! extend search on diagonal
 
-    INTEGER :: ik                       ! number of point change
-    INTEGER :: ip                       ! size of the pile
-    INTEGER :: ji, jj                   ! loop index
-    INTEGER :: iip1, iim1, ii, ij       ! working integer
+    INTEGER :: ik                              ! number of point change
+    INTEGER :: ip                              ! size of the pile
+    INTEGER :: ji, jj                          ! loop index
+    INTEGER :: iip1, iim1, ii, ij, ijp1, ijm1  ! working integer
     INTEGER :: ipiglo, ipjglo           ! size of the domain, infered from kdta size
+    LOGICAL :: lperio = .TRUE.
+    LOGICAL :: ldiag  = .FALSE.
 
     INTEGER(KIND=2), DIMENSION(:,:), ALLOCATABLE :: ipile    ! pile variable
-    INTEGER(KIND=2), DIMENSION(:,:), ALLOCATABLE :: idata   ! new bathymetry
+    INTEGER(KIND=2), DIMENSION(:,:), ALLOCATABLE :: idata    ! new data
     !!----------------------------------------------------------------------
+    IF ( PRESENT(ld_perio   )) lperio = ld_perio
+    IF ( PRESENT(ld_diagonal)) ldiag  = ld_diagonal
+    ! WARNING
+    IF (lperio) PRINT *, 'W A R N I N G: north fold not treated properly ...'
+
     ! infer domain size from input array
     ipiglo = SIZE(kdta,1)
     ipjglo = SIZE(kdta,2)
@@ -356,14 +514,25 @@ CONTAINS
        ipile(ip,:)  =[0,0]; ip=ip-1
 
        ! check neighbour cells and update pile ( assume E-W periodicity )
-       iip1=ii+1; IF ( iip1 == ipiglo+1 ) iip1=2
-       iim1=ii-1; IF ( iim1 == 0        ) iim1=ipiglo-1
-
-       IF (idata(ii, ij+1) > 0 ) THEN
-          ip=ip+1; ipile(ip,:)=[ii  ,ij+1]
+       IF ( lperio ) THEN
+          IF ( ii == ipiglo+1 ) ii=3
+          IF ( ii == 0        ) ii=ipiglo-2
+          iip1=ii+1; IF ( iip1 == ipiglo+1) iip1=3
+          iim1=ii-1; IF ( iim1 == 0       ) iim1=ipiglo-2
+       ELSE
+          IF ( ii == ipiglo+1 ) ii=ipiglo
+          IF ( ii == 0        ) ii=1
+          iip1=ii+1; IF ( iip1 == ipiglo+1) iip1=ipiglo
+          iim1=ii-1; IF ( iim1 == 0       ) iim1=1
        END IF
-       IF (idata(ii, ij-1) > 0 ) THEN
-          ip=ip+1; ipile(ip,:)=[ii  ,ij-1]
+       ijp1=MIN(ij+1,ipjglo)  ! north fold not treated
+       ijm1=MAX(ij-1,1)
+
+       IF (idata(ii, ijp1) > 0 ) THEN
+          ip=ip+1; ipile(ip,:)=[ii  ,ijp1]
+       END IF
+       IF (idata(ii, ijm1) > 0 ) THEN
+          ip=ip+1; ipile(ip,:)=[ii  ,ijm1]
        END IF
        IF (idata(iip1, ij) > 0 ) THEN
           ip=ip+1; ipile(ip,:)=[iip1,ij  ]
@@ -372,6 +541,21 @@ CONTAINS
           ip=ip+1; ipile(ip,:)=[iim1,ij  ]
        END IF
 
+       IF ( ldiag ) THEN
+          IF (idata(iim1, ijp1) > 0 ) THEN
+             ip=ip+1; ipile(ip,:)=[iim1,ijp1]
+          END IF
+          IF (idata(iim1, ijm1) > 0 ) THEN
+             ip=ip+1; ipile(ip,:)=[iim1,ijm1]
+          END IF
+          IF (idata(iip1, ijp1) > 0 ) THEN
+             ip=ip+1; ipile(ip,:)=[iip1,ijp1]
+          END IF
+          IF (idata(iip1, ijm1) > 0 ) THEN
+             ip=ip+1; ipile(ip,:)=[iip1,ijm1]
+          END IF
+       END IF
+
     END DO
     kdta=idata;
 
@@ -380,17 +564,19 @@ CONTAINS
   END SUBROUTINE FillPool2D
 
 
-  SUBROUTINE FillPool3D(kiseed, kjseed, kkseed, kdta, kifill)
+  SUBROUTINE FillPool3D(kiseed, kjseed, kkseed, kdta, kifill, ld_perio)
     !!---------------------------------------------------------------------
     !!                  ***  ROUTINE FillPool3D  ***
     !!
-    !! ** Purpose :  Replace all area surrounding by mask value by mask value
+    !! ** Purpose :  Replace all area surrounding by mask value by kifill value
     !!
     !! ** Method  :  flood fill algorithm
     !!
     !!----------------------------------------------------------------------
     INTEGER(KIND=4),                   INTENT(in)    :: kiseed, kjseed, kkseed
     INTEGER(KIND=4),                   INTENT(in)    :: kifill   ! new bathymetry
+    LOGICAL, OPTIONAL,                 INTENT(in)    :: ld_perio ! EW periodicity
+
     INTEGER(KIND=2), DIMENSION(:,:,:), INTENT(inout) :: kdta     ! new bathymetry
 
     INTEGER :: ik,iik                   ! number of point change
@@ -400,8 +586,13 @@ CONTAINS
     INTEGER :: iip1, iim1, ii, ij       ! working integer
     INTEGER :: ijp1, ijm1, ikp1, ikm1
     INTEGER(KIND=2), DIMENSION(:,:),   ALLOCATABLE :: ipile    ! pile variable
-    INTEGER(KIND=2), DIMENSION(:,:,:), ALLOCATABLE :: idata   ! new bathymetry
+    INTEGER(KIND=2), DIMENSION(:,:,:), ALLOCATABLE :: idata    ! new data
+    LOGICAL :: lperio = .TRUE.
     !!----------------------------------------------------------------------
+    IF ( PRESENT (ld_perio) ) lperio = ld_perio
+    ! WARNING
+    IF (lperio) PRINT *, 'W A R N I N G: north fold not treated properly ...'
+
     ! infer domain size from input array
     ipiglo = SIZE(kdta,1)
     ipjglo = SIZE(kdta,2)
@@ -428,13 +619,23 @@ CONTAINS
        ipile(ip,:)  =[0,0,0]; ip=ip-1
 
        ! check neighbour cells and update pile ( assume E-W periodicity )
-       iip1=ii+1; IF ( iip1 == ipiglo+1 ) iip1=2
-       iim1=ii-1; IF ( iim1 == 0        ) iim1=ipiglo-1
-       ijp1=ij+1 ; ijm1 =ij-1
+       IF ( lperio ) THEN
+          IF ( ii == ipiglo+1 ) ii=3
+          IF ( ii == 0        ) ii=ipiglo-2
+          iip1=ii+1; IF ( iip1 == ipiglo+1) iip1=3
+          iim1=ii-1; IF ( iim1 == 0       ) iim1=ipiglo-2
+       ELSE
+          IF ( ii == ipiglo+1 ) ii=ipiglo
+          IF ( ii == 0        ) ii=1
+          iip1=ii+1; IF ( iip1 == ipiglo+1) iip1=ipiglo
+          iim1=ii-1; IF ( iim1 == 0       ) iim1=1
+       END IF
+       ijp1=MIN(ij+1,ipjglo)  ! north fold not treated
+       ijm1=MAX(ij-1,1)
+
        ikp1=ik+1 ; IF (ikp1 == ipk+1 ) ikp1=ik
        ikm1=ik-1 ; IF (ikm1 == 0     ) ikm1=ik
 
-
        IF (idata(ii, ijp1,ik) > 0 ) THEN
           ip=ip+1; ipile(ip,:)=[ii  ,ijp1,ik]
        END IF
@@ -464,4 +665,46 @@ CONTAINS
 
   END SUBROUTINE FillPool3D
 
+  FUNCTION heading(dplona, dplonb, dplata, dplatb)
+    !!---------------------------------------------------------------------
+    !!                  ***  FUNCTION heading  ***
+    !!
+    !! ** Purpose : Compute true heading between point a and b
+    !!
+    !! ** Method  : Suppose that the 2 points are not too far away 
+    !!              from each other so that heading can be computed 
+    !!              with loxodromy.
+    !!
+    !!----------------------------------------------------------------------
+    REAL(KIND=8), INTENT(in) :: dplata, dplona ! lat lon of point a
+    REAL(KIND=8), INTENT(in) :: dplatb, dplonb ! lat lon of point b
+    REAL(KIND=8)             :: heading        ! return value in degree
+
+    REAL(KIND=8)             :: dlpi, dlconv   ! pi and conversion factor
+    REAL(KIND=8)             :: dlxa,dlya      ! working variable
+    REAL(KIND=8)             :: dlxb,dlyb      ! working variable
+    REAL(KIND=8)             :: dlxb_xa        !  ""        ""
+    !!----------------------------------------------------------------------
+
+    dlpi   = ACOS(-1.d0)
+    dlconv = dlpi/180.d0  ! for degree to radian conversion
+
+    ! there is a problem if the Greenwich meridian pass between a and b
+    dlxa = dplona*dlconv
+    dlxb = dplonb*dlconv
+
+    dlya = -LOG(TAN(dlpi/4.-dlconv*dplata/2.d0))
+    dlyb = -LOG(TAN(dlpi/4.-dlconv*dplatb/2.d0))
+
+    dlxb_xa = MOD((dlxb-dlxa),2*dlpi)
+
+    IF ( dlxb_xa >=  dlpi ) dlxb_xa = dlxb_xa -2*dlpi
+    IF ( dlxb_xa <= -dlpi ) dlxb_xa = dlxb_xa +2*dlpi
+
+    heading=ATAN2(dlxb_xa,(dlyb-dlya))*180.d0/dlpi
+
+    IF (heading < 0) heading=heading+360.d0
+  END FUNCTION heading
+
+
 END MODULE modutils
